searchData={"items":[{"type":"task","title":"mix oban.install","doc":"Install and configure Oban for use in an application.","ref":"Mix.Tasks.Oban.Install.html"},{"type":"task","title":"Example - mix oban.install","doc":"Install using the default Ecto repo and matching engine:\n\n```bash\nmix oban.install\n```\n\nSpecify a `SQLite3` repo and `Lite` engine explicitly:\n\n```bash\nmix oban.install --repo MyApp.LiteRepo --engine Oban.Engines.Lite\n```","ref":"Mix.Tasks.Oban.Install.html#module-example"},{"type":"task","title":"Options - mix oban.install","doc":"* `--engine` or `-e` — Select the engine for your repo, defaults to `Oban.Engines.Postgres`\n* `--notifier` or `-n` — Select the pubsub notifier, defaults to `Oban.Notifiers.Postgres`\n* `--repo` or `-r` — Specify an Ecto repo for Oban to use","ref":"Mix.Tasks.Oban.Install.html#module-options"},{"type":"module","title":"Oban","doc":"Oban is a robust background job framework which uses PostgreSQL, MySQL, or SQLite3 for\npersistence.","ref":"Oban.html"},{"type":"module","title":"Features - Oban","doc":"Oban's primary goals are **reliability**, **consistency** and **observability**.\n\nOban is a powerful and flexible library that can handle a wide range of background job use cases,\nand it is well-suited for systems of any size. It provides a simple and consistent API for\nscheduling and performing jobs, and it is built to be fault-tolerant and easy to monitor.\n\nOban is fundamentally different from other background job processing tools because _it retains job\ndata for historic metrics and inspection_. You can leave your application running indefinitely\nwithout worrying about jobs being lost or orphaned due to crashes.\n\n#### Advantages Over Other Tools\n\n- **Fewer Dependencies** — If you are running a web app there is a _very good_ chance that you're\n  running on top of a SQL database. Running your job queue within a SQL database minimizes system\n  dependencies and simplifies data backups.\n\n- **Transactional Control** — Enqueue a job along with other database changes, ensuring that\n  everything is committed or rolled back atomically.\n\n- **Database Backups** — Jobs are stored inside of your primary database, which means they are\n  backed up together with the data that they relate to.\n\n#### Advanced Features\n\n- **Isolated Queues** — Jobs are stored in a single table but are executed in distinct queues.\n  Each queue runs in isolation, ensuring that a job in a single slow queue can't back up other\n  faster queues.\n\n- **Queue Control** — Queues can be started, stopped, paused, resumed and scaled independently at\n  runtime locally or across _all_ running nodes (even in environments like Heroku, without\n  distributed Erlang).\n\n- **Resilient Queues** — Failing queries won't crash the entire supervision tree, instead a\n  backoff mechanism will safely retry them again in the future.\n\n- **Job Canceling** — Jobs can be canceled in the middle of execution regardless of which node\n  they are running on. This stops the job at once and flags it as `cancelled`.\n\n- **Triggered Execution** — Insert triggers ensure that jobs are dispatched on all connected nodes\n  as soon as they are inserted into the database.\n\n- **Unique Jobs** — Duplicate work can be avoided through unique job controls. Uniqueness can be\n  enforced at the argument, queue, worker and even sub-argument level for any period of time.\n\n- **Scheduled Jobs** — Jobs can be scheduled at any time in the future, down to the second.\n\n- **Periodic (CRON) Jobs** — Automatically enqueue jobs on a cron-like schedule. Duplicate jobs\n  are never enqueued, no matter how many nodes you're running.\n\n- **Job Priority** — Prioritize jobs within a queue to run ahead of others with ten levels of\n  granularity.\n\n- **Historic Metrics** — After a job is processed the row isn't deleted. Instead, the job is\n  retained in the database to provide metrics. This allows users to inspect historic jobs and to\n  see aggregate data at the job, queue or argument level.\n\n- **Node Metrics** — Every queue records metrics to the database during runtime. These are used to\n  monitor queue health across nodes and may be used for analytics.\n\n- **Graceful Shutdown** — Queue shutdown is delayed so that slow jobs can finish executing before\n  shutdown. When shutdown starts queues are paused and stop executing new jobs. Any jobs left\n  running after the shutdown grace period may be rescued later.\n\n- **Telemetry Integration** — Job life-cycle events are emitted via [Telemetry][tele] integration.\n  This enables simple logging, error reporting and health checkups without plug-ins.\n\n[tele]: https://github.com/beam-telemetry/telemetry","ref":"Oban.html#module-features"},{"type":"module","title":"🌟 Oban Pro - Oban","doc":"An official set of extensions, plugins, and workers that expand what Oban is capable of is\navailable as a licensed package. It includes features like:\n\n* 🖇️ [Workflows][wor]\n* 🎨 [Decorators][dec]\n* ⛓️  [Chains][cha]\n* 🏗️ [Structured Jobs][str]\n* 🪝 [Worker Hooks][hoo]\n* 🌎 [Global Limits][glo]\n* 🔪 [Queue Partitioning][par]\n* 🎢 [Dynamic Queues][dyn]\n\nPlus [much more][ove]. Learn more about [Oban Pro](https://oban.pro#oban-pro)\n\n[cha]: https://oban.pro/docs/pro/Oban.Pro.Worker.html#module-chained-jobs\n[dec]: https://oban.pro/docs/pro/Oban.Pro.Decorator.html\n[dyn]: https://oban.pro/docs/pro/Oban.Pro.Plugins.DynamicQueues.html\n[glo]: https://oban.pro/docs/pro/Oban.Pro.Engines.Smart.html#module-global-concurrency\n[hoo]: https://oban.pro/docs/pro/Oban.Pro.Worker.html#module-worker-hooks\n[par]: https://oban.pro/docs/pro/Oban.Pro.Engines.Smart.html#module-queue-partitioning\n[ove]: https://oban.pro/docs/pro/overview.html\n[str]: https://oban.pro/docs/pro/Oban.Pro.Worker.html#module-structured-jobs\n[wor]: https://oban.pro/docs/pro/Oban.Pro.Workers.Workflow.html","ref":"Oban.html#module-oban-pro"},{"type":"module","title":"Engines - Oban","doc":"Oban ships with engines for [PostgreSQL][postgres], [MySQL][mysql], and [SQLite3][sqlite3]. Each\nengine supports the same core functionality, though they have differing levels of maturity and\nsuitability for production.\n\n[postgres]: https://www.postgresql.org/\n[mysql]: https://www.mysql.com/\n[sqlite3]: https://www.sqlite.org/","ref":"Oban.html#module-engines"},{"type":"module","title":"Requirements - Oban","doc":"Oban requires:\n\n* Elixir 1.15+\n* Erlang 24+\n* PostgreSQL 12.0+, MySQL 8.0+, or SQLite3 3.37.0+","ref":"Oban.html#module-requirements"},{"type":"module","title":"Installation - Oban","doc":"See the [installation guide](https://hexdocs.pm/oban/installation.html) for details on installing\nand configuring Oban in your application.","ref":"Oban.html#module-installation"},{"type":"module","title":"Quick Getting Started - Oban","doc":"1. [Configure queues](https://hexdocs.pm/oban/Oban.html#module-configuring-queues) and an Ecto repo for Oban to\n     use:\n\n     ```elixir\n     # In config/config.exs\n     config :my_app, Oban,\n       repo: MyApp.Repo,\n       queues: [mailers: 20]\n     ```\n\n  2. Define a worker to process jobs in the `mailers` queue (see `Oban.Worker`):\n\n     ```elixir\n     defmodule MyApp.MailerWorker do\n       use Oban.Worker, queue: :mailers\n\n       @impl Oban.Worker\n       def perform(%Oban.Job{args: %{\"email\" => email} = _args}) do\n         _ = Email.deliver(email)\n         :ok\n       end\n     end\n     ```\n\n  3. Enqueue a job (see [the documentation](https://hexdocs.pm/oban/Oban.Job.html#enqueueing-jobs)):\n\n     ```elixir\n     %{email: %{to: \"foo@example.com\", body: \"Hello from Oban!\"}}\n     |> MyApp.MailerWorker.new()\n     |> Oban.insert()\n     ```\n\n  4. The magic happens! Oban executes the job when there is available bandwidth in the\n     `mailer` queue.","ref":"Oban.html#module-quick-getting-started"},{"type":"module","title":"Learning - Oban","doc":"Learn the fundamentals of Oban, all the way through to preparing for production with our LiveBook\npowered [Oban Training](https://github.com/oban-bg/oban_training/) curriculum.","ref":"Oban.html#module-learning"},{"type":"macro","title":"Oban.__using__/1","doc":"Creates a facade for `Oban` functions and automates fetching configuration from the application\nenvironment.\n\nFacade modules support configuration via the application environment under an OTP application\nkey that you specify with `:otp_app`. For example, to define a facade:\n\n    defmodule MyApp.Oban do\n      use Oban, otp_app: :my_app\n    end\n\nThen, you can configure the facade with:\n\n    config :my_app, MyApp.Oban, repo: MyApp.Repo\n\nThen you can include `MyApp.Oban` in your application's supervision tree without passing extra\noptions:\n\n    defmodule MyApp.Application do\n      use Application\n\n      def start(_type, _args) do\n        children = [\n          MyApp.Repo,\n          MyApp.Oban\n        ]\n\n        opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n        Supervisor.start_link(children, opts)\n      end\n    end","ref":"Oban.html#__using__/1"},{"type":"macro","title":"Calling Functions - Oban.__using__/1","doc":"Facade modules allow you to call `Oban` functions on instances with custom names (rather than\n`Oban`), without passing a `t:Oban.name/0` as the first argument.\n\nFor example:\n\n    # Instead of:\n    Oban.config(MyApp.Oban)\n\n    # You can do:\n    MyApp.Oban.config()\n\nFacades also make piping into Oban functions far more convenient:\n\n    %{some: :args}\n    |> MyWorker.new()\n    |> MyOban.insert()","ref":"Oban.html#__using__/1-calling-functions"},{"type":"macro","title":"Merging Configuration - Oban.__using__/1","doc":"All configuration can be provided through the `use` macro or through the application\nconfiguration, and options from the application supersedes those passed through `use`.\nConfiguration is prioritized in this order:\n\n1. Options passed through `use`\n2. Options pulled from the OTP app specified by `:otp_app` via `Application.get_env/3`\n3. Options passed through a child spec in the supervisor","ref":"Oban.html#__using__/1-merging-configuration"},{"type":"function","title":"Oban.cancel_all_jobs/2","doc":"Cancel many jobs based on a queryable and mark them as `cancelled` to prevent them from running.\n\nAny currently `executing` jobs are killed. If executing jobs happen to fail before cancellation\nthen the state is set to `cancelled`. However, any that complete successfully will remain\n`completed`.\n\nOnly jobs with the statuses `executing`, `available`, `scheduled`, or `retryable` can be cancelled.","ref":"Oban.html#cancel_all_jobs/2"},{"type":"function","title":"Example - Oban.cancel_all_jobs/2","doc":"Cancel all jobs:\n\n    Oban.cancel_all_jobs(Oban.Job)\n    {:ok, 9}\n\nCancel all jobs for a specific worker:\n\n    Oban.Job\n    |> Ecto.Query.where(worker: \"MyApp.MyWorker\")\n    |> Oban.cancel_all_jobs()\n    {:ok, 2}","ref":"Oban.html#cancel_all_jobs/2-example"},{"type":"function","title":"Oban.cancel_job/2","doc":"Cancel an `executing`, `available`, `scheduled` or `retryable` job and mark it as `cancelled` to\nprevent it from running. If the job is currently `executing` it will be killed and otherwise it\nis ignored.\n\nIf an executing job happens to fail before it can be cancelled the state is set to `cancelled`.\nHowever, if it manages to complete successfully then the state will still be `completed`.","ref":"Oban.html#cancel_job/2"},{"type":"function","title":"Example - Oban.cancel_job/2","doc":"Cancel a job:\n\n    Oban.cancel_job(job)\n    :ok\n\nCancel a job for a custom instance:\n\n    Oban.cancel_job(MyOban, job)\n    :ok","ref":"Oban.html#cancel_job/2-example"},{"type":"type","title":"Oban.changeset_or_fun/0","doc":"","ref":"Oban.html#t:changeset_or_fun/0"},{"type":"type","title":"Oban.changeset_wrapper/0","doc":"","ref":"Oban.html#t:changeset_wrapper/0"},{"type":"type","title":"Oban.changesets_or_wrapper/0","doc":"","ref":"Oban.html#t:changesets_or_wrapper/0"},{"type":"type","title":"Oban.changesets_or_wrapper_or_fun/0","doc":"","ref":"Oban.html#t:changesets_or_wrapper_or_fun/0"},{"type":"function","title":"Oban.check_all_queues/1","doc":"Check the current state of all queue producers.","ref":"Oban.html#check_all_queues/1"},{"type":"function","title":"Example - Oban.check_all_queues/1","doc":"Get information about all running queues for the default instance:\n\n    Enum.map(Oban.check_all_queues(), &Map.take(&1, [:queue, :limit]))\n    [%{queue: \"default\", limit: 10}, %{queue: \"other\", limit: 5}]\n\nGet information for an alternate instance:\n\n    Oban.check_all_queues(Other.Oban)","ref":"Oban.html#check_all_queues/1-example"},{"type":"function","title":"Oban.check_queue/2","doc":"Check the current state of a queue producer.\n\nThis allows you to introspect on a queue's health by retrieving key attributes of the producer's\nstate; values such as the current `limit`, the `running` job ids, and when the producer was\nstarted.","ref":"Oban.html#check_queue/2"},{"type":"function","title":"Options - Oban.check_queue/2","doc":"* `:queue` - a string or atom specifying the queue to check, required","ref":"Oban.html#check_queue/2-options"},{"type":"function","title":"Example - Oban.check_queue/2","doc":"Get details about the `default` queue:\n\n    Oban.check_queue(queue: :default)\n    %{\n      limit: 10,\n      node: \"me@local\",\n      paused: false,\n      queue: \"default\",\n      running: [100, 102],\n      started_at: ~D[2020-10-07 15:31:00],\n      updated_at: ~D[2020-10-07 15:31:00]\n    }\n\nAttempt to get details about a queue that isn't running locally:\n\n    Oban.check_queue(queue: :not_really_running)\n    nil","ref":"Oban.html#check_queue/2-example"},{"type":"function","title":"Oban.config/1","doc":"Retrieve the `Oban.Config` struct for a named Oban supervision tree.","ref":"Oban.html#config/1"},{"type":"function","title":"Example - Oban.config/1","doc":"Retrieve the default `Oban` instance config:\n\n    %Oban.Config{} = Oban.config()\n\nRetrieve the config for an instance started with a custom name:\n\n    %Oban.Config{} = Oban.config(MyCustomOban)","ref":"Oban.html#config/1-example"},{"type":"function","title":"Oban.delete_all_jobs/2","doc":"Delete many jobs based on a queryable.\n\nOnly jobs that aren't `executing` may be deleted.","ref":"Oban.html#delete_all_jobs/2"},{"type":"function","title":"Example - Oban.delete_all_jobs/2","doc":"Delete all jobs for a specific worker:\n\n    Oban.Job\n    |> Ecto.Query.where(worker: \"MyApp.MyWorker\")\n    |> Oban.delete_all_jobs()\n    {:ok, 9}","ref":"Oban.html#delete_all_jobs/2-example"},{"type":"function","title":"Oban.delete_job/2","doc":"Delete a job that's not currently `executing`.","ref":"Oban.html#delete_job/2"},{"type":"function","title":"Example - Oban.delete_job/2","doc":"Delete a job:\n\n    Oban.delete_job(job)\n    :ok\n\nDelete a job for a custom instance:\n\n    Oban.delete_job(MyOban, job)\n    :ok","ref":"Oban.html#delete_job/2-example"},{"type":"type","title":"Oban.drain_option/0","doc":"","ref":"Oban.html#t:drain_option/0"},{"type":"function","title":"Oban.drain_queue/2","doc":"Synchronously execute all available jobs in a queue.\n\nAll execution happens within the current process and it is guaranteed not to raise an error or\nexit.\n\nDraining a queue from within the current process is especially useful for testing. Jobs that are\nenqueued by a process when `Ecto` is in sandbox mode are only visible to that process. Calling\n`drain_queue/2` allows you to control when the jobs are executed and to wait synchronously for\nall jobs to complete.","ref":"Oban.html#drain_queue/2"},{"type":"function","title":"Failures & Retries - Oban.drain_queue/2","doc":"Draining a queue uses the same execution mechanism as regular job dispatch. That means that any\njob failures or crashes are captured and result in a retry. Retries are scheduled in the future\nwith backoff and won't be retried immediately.\n\nBy default jobs are executed in `safe` mode, just as they are in production. Safe mode catches\nany errors or exits and records the formatted error in the job's `errors` array.  That means\nexceptions and crashes are _not_ bubbled up to the calling process.\n\nIf you expect jobs to fail, would like to track failures, or need to check for specific errors\nyou can pass the `with_safety: false` flag. See the \"Options\" section below for more details.","ref":"Oban.html#drain_queue/2-failures-retries"},{"type":"function","title":"Scheduled Jobs - Oban.drain_queue/2","doc":"By default, `drain_queue/2` will execute all currently available jobs. In order to execute\nscheduled jobs, you may pass the `with_scheduled: true` which will cause all scheduled jobs to\nbe marked as `available` beforehand. To run jobs scheduled up to a specific point in time, pass\na `DateTime` instead.","ref":"Oban.html#drain_queue/2-scheduled-jobs"},{"type":"function","title":"Options - Oban.drain_queue/2","doc":"* `:queue` - a string or atom specifying the queue to drain, required\n\n* `:with_limit` — the maximum number of jobs to drain at once. When recursion is enabled this is\n  how many jobs are processed per-iteration.\n\n* `:with_recursion` — whether to keep draining a queue repeatedly when jobs insert _more_ jobs\n\n* `:with_safety` — whether to silently catch errors when draining, defaults to `true`. When\n  `false`, raised exceptions or unhandled exits are reraised (unhandled exits are wrapped in\n  `Oban.CrashError`).\n\n* `:with_scheduled` — whether to include any scheduled jobs when draining, default `false`.\n   When `true`, drains all scheduled jobs. When a `DateTime` is provided, drains all jobs\n   scheduled up to, and including, that point in time.","ref":"Oban.html#drain_queue/2-options"},{"type":"function","title":"Example - Oban.drain_queue/2","doc":"Drain a queue with three available jobs, two of which succeed and one of which fails:\n\n    Oban.drain_queue(queue: :default)\n    %{failure: 1, snoozed: 0, success: 2}\n\nDrain a queue including any scheduled jobs:\n\n    Oban.drain_queue(queue: :default, with_scheduled: true)\n    %{failure: 0, snoozed: 0, success: 1}\n\nDrain a queue including jobs scheduled up to a minute:\n\n    Oban.drain_queue(queue: :default, with_scheduled: DateTime.add(DateTime.utc_now(), 60, :second))\n\nDrain a queue and assert an error is raised:\n\n    assert_raise RuntimeError, fn -> Oban.drain_queue(queue: :risky, with_safety: false) end\n\nDrain a queue repeatedly until there aren't any more jobs to run. This is particularly useful\nfor testing jobs that enqueue other jobs:\n\n    Oban.drain_queue(queue: :default, with_recursion: true)\n    %{failure: 1, snoozed: 0, success: 2}\n\nDrain only the top (by scheduled time and priority) five jobs off a queue:\n\n    Oban.drain_queue(queue: :default, with_limit: 5)\n    %{failure: 0, snoozed: 0, success: 1}\n\nDrain a queue recursively, only one job at a time:\n\n    Oban.drain_queue(queue: :default, with_limit: 1, with_recursion: true)\n    %{failure: 0, snoozed: 0, success: 3}","ref":"Oban.html#drain_queue/2-example"},{"type":"type","title":"Oban.drain_result/0","doc":"","ref":"Oban.html#t:drain_result/0"},{"type":"function","title":"Oban.insert/3","doc":"Insert a new job into the database for execution.\n\nThis and the other `insert` variants are the recommended way to enqueue jobs because they\nsupport features like unique jobs.\n\nSee the section on \"Unique Jobs\" for more details.","ref":"Oban.html#insert/3"},{"type":"function","title":"Example - Oban.insert/3","doc":"Insert a single job:\n\n    {:ok, job} = Oban.insert(MyApp.Worker.new(%{id: 1}))\n\nInsert a job while ensuring that it is unique within the past 30 seconds:\n\n    {:ok, job} = Oban.insert(MyApp.Worker.new(%{id: 1}, unique: [period: 30]))\n\nInsert a job using a custom timeout:\n\n    {:ok, job} = Oban.insert(MyApp.Worker.new(%{id: 1}), timeout: 10_000)\n\nInsert a job using an alternative instance name:\n\n    {:ok, job} = Oban.insert(MyOban, MyApp.Worker.new(%{id: 1}))","ref":"Oban.html#insert/3-example"},{"type":"function","title":"Oban.insert/5","doc":"Put a job insert operation into an `Ecto.Multi`.\n\nLike `insert/2`, this variant is recommended over `Ecto.Multi.insert` because it supports all of\nOban's features, i.e. unique jobs.\n\nSee the section on \"Unique Jobs\" for more details.","ref":"Oban.html#insert/5"},{"type":"function","title":"Example - Oban.insert/5","doc":"Ecto.Multi.new()\n    |> Oban.insert(\"job-1\", MyApp.Worker.new(%{id: 1}))\n    |> Oban.insert(\"job-2\", fn _ -> MyApp.Worker.new(%{id: 2}) end)\n    |> MyApp.Repo.transaction()","ref":"Oban.html#insert/5-example"},{"type":"function","title":"Oban.insert!/3","doc":"Similar to `insert/3`, but raises an `Ecto.InvalidChangesetError` if the job can't be inserted.","ref":"Oban.html#insert!/3"},{"type":"function","title":"Example - Oban.insert!/3","doc":"Insert a single job:\n\n    job = Oban.insert!(MyApp.Worker.new(%{id: 1}))\n\nInsert a job using a custom timeout:\n\n    job = Oban.insert!(MyApp.Worker.new(%{id: 1}), timeout: 10_000)\n\nInsert a job using an alternative instance name:\n\n    job = Oban.insert!(MyOban, MyApp.Worker.new(%{id: 1}))","ref":"Oban.html#insert!/3-example"},{"type":"function","title":"Oban.insert_all/3","doc":"Insert multiple jobs into the database for execution.\n\nThere are a few important differences between this function and `c:Ecto.Repo.insert_all/3`:\n\n1. This function always returns a list rather than a tuple of `{count, records}`\n2. This function accepts a list of changesets rather than a list of maps or keyword lists\n\n#### Error Handling and Rollbacks\n\nIf `insert_all` encounters an issue, the function will raise an error based on your database\nadapter. This behavior is valuable in conjunction with `c:Ecto.Repo.transaction/2` because it\nallows for rollbacks.\n\nFor example, an invalid changeset raises:\n\n`* (Ecto.InvalidChangesetError) could not perform insert because changeset is invalid.`\n\n#### Dolphin Engine and Generated Values\n\nMySQL doesn't return anything on insertion into the database. That means any values generated by\nthe database, namely the primary key and timestamps, aren't included in the job structs returned\nfrom `insert_all`.\n\n> #### 🌟 Unique Jobs and Batching {: .tip}\n>\n> Only the [Smart Engine](https://oban.pro/docs/pro/Oban.Pro.Engines.Smart.html) in [Oban\n> Pro](https://oban.pro) supports bulk unique jobs, automatic insert batching, and minimizes\n> parameters sent over the wire. With the basic engine, you must use `insert/3` to insert unique\n> jobs one at a time.","ref":"Oban.html#insert_all/3"},{"type":"function","title":"Options - Oban.insert_all/3","doc":"Accepts any of Ecto's \"Shared Options\" such as `timeout` and `log`.","ref":"Oban.html#insert_all/3-options"},{"type":"function","title":"Example - Oban.insert_all/3","doc":"Insert a list of 100 jobs at once:\n\n    1..100\n    |> Enum.map(&MyApp.Worker.new(%{id: &1}))\n    |> Oban.insert_all()\n\nInsert a stream of jobs at once (be sure the stream terminates!):\n\n    (fn -> MyApp.Worker.new(%{}))\n    |> Stream.repeatedly()\n    |> Stream.take(100)\n    |> Oban.insert_all()\n\nInsert with a custom timeout:\n\n    1..100\n    |> Enum.map(&MyApp.Worker.new(%{id: &1}))\n    |> Oban.insert_all(timeout: 10_000)\n\nInsert with an alternative instance name:\n\n    changesets = Enum.map(1..100, &MyApp.Worker.new(%{id: &1}))\n    jobs = Oban.insert_all(MyOban, changesets)","ref":"Oban.html#insert_all/3-example"},{"type":"function","title":"Oban.insert_all/5","doc":"Put an `insert_all` operation into an `Ecto.Multi`.\n\nThis function supports the same features and has the same caveats as `insert_all/2`.","ref":"Oban.html#insert_all/5"},{"type":"function","title":"Example - Oban.insert_all/5","doc":"Insert job changesets within a multi:\n\n    changesets = Enum.map(0..100, &MyApp.Worker.new(%{id: &1}))\n\n    Ecto.Multi.new()\n    |> Oban.insert_all(:jobs, changesets)\n    |> MyApp.Repo.transaction()\n\nInsert job changesets using a function:\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.insert(:user, user_changeset)\n    |> Oban.insert_all(:jobs, fn %{user: user} ->\n      email_job = EmailWorker.new(%{id: user.id})\n      staff_job = StaffWorker.new(%{id: user.id})\n      [email_job, staff_job]\n    end)\n    |> MyApp.Repo.transaction()","ref":"Oban.html#insert_all/5-example"},{"type":"type","title":"Oban.multi/0","doc":"","ref":"Oban.html#t:multi/0"},{"type":"type","title":"Oban.multi_name/0","doc":"","ref":"Oban.html#t:multi_name/0"},{"type":"type","title":"Oban.name/0","doc":"The name of an Oban instance. This is used to identify instances in the internal registry for\nconfiguration lookup.","ref":"Oban.html#t:name/0"},{"type":"type","title":"Oban.oban_node/0","doc":"","ref":"Oban.html#t:oban_node/0"},{"type":"type","title":"Oban.option/0","doc":"","ref":"Oban.html#t:option/0"},{"type":"function","title":"Oban.pause_all_queues/2","doc":"Pause all running queues to prevent them from executing any new jobs.\n\nSee `pause_queue/2` for options and details.","ref":"Oban.html#pause_all_queues/2"},{"type":"function","title":"Example - Oban.pause_all_queues/2","doc":"Pause all queues:\n\n    Oban.pause_all_queues()\n\nPause all queues on the local node:\n\n    Oban.pause_all_queues(local_only: true)\n\nPause all queues on a specific node:\n\n    Oban.pause_all_queues(node: \"worker.1\")\n\nPause all queues for an alternative instance name:\n\n    Oban.pause_all_queues(MyOban)","ref":"Oban.html#pause_all_queues/2-example"},{"type":"function","title":"Oban.pause_queue/2","doc":"Pause a running queue, preventing it from executing any new jobs. All running jobs will remain\nrunning until they are finished.\n\nWhen shutdown begins all queues are paused.","ref":"Oban.html#pause_queue/2"},{"type":"function","title":"Options - Oban.pause_queue/2","doc":"* `:queue` - a string or atom specifying the queue to pause, required\n\n* `:local_only` - whether the queue will be paused only on the local node, default: `false`\n\n* `:node` - restrict pausing to a particular node\n\nNote: by default, Oban does not verify that the given queue exists unless `:local_only`\nis set to `true` as even if the queue does not exist locally, it might be running on\nanother node.","ref":"Oban.html#pause_queue/2-options"},{"type":"function","title":"Example - Oban.pause_queue/2","doc":"Pause the default queue:\n\n    Oban.pause_queue(queue: :default)\n    :ok\n\nPause the default queue, but only on the local node:\n\n    Oban.pause_queue(queue: :default, local_only: true)\n    :ok\n\nPause the default queue only on a particular node:\n\n    Oban.pause_queue(queue: :default, node: \"worker.1\")\n    :ok","ref":"Oban.html#pause_queue/2-example"},{"type":"type","title":"Oban.queue_all_option/0","doc":"","ref":"Oban.html#t:queue_all_option/0"},{"type":"type","title":"Oban.queue_name/0","doc":"","ref":"Oban.html#t:queue_name/0"},{"type":"type","title":"Oban.queue_option/0","doc":"","ref":"Oban.html#t:queue_option/0"},{"type":"type","title":"Oban.queue_state/0","doc":"","ref":"Oban.html#t:queue_state/0"},{"type":"function","title":"Oban.resume_all_queues/2","doc":"Resume executing jobs in all paused queues.\n\nSee `resume_queue/2` for options and details.","ref":"Oban.html#resume_all_queues/2"},{"type":"function","title":"Example - Oban.resume_all_queues/2","doc":"Resume all queues:\n\n    Oban.resume_all_queues()\n\nResume all queues on the local node:\n\n    Oban.resume_all_queues(local_only: true)\n\nResume all queues on a specific node:\n\n    Oban.resume_all_queues(node: \"worker.1\")\n\nResume all queues for an alternative instance name:\n\n    Oban.resume_all_queues(MyOban)","ref":"Oban.html#resume_all_queues/2-example"},{"type":"function","title":"Oban.resume_queue/2","doc":"Resume executing jobs in a paused queue.","ref":"Oban.html#resume_queue/2"},{"type":"function","title":"Options - Oban.resume_queue/2","doc":"* `:queue` - a string or atom specifying the queue to resume, required\n\n* `:local_only` - whether the queue will be resumed only on the local node, default: `false`\n\n* `:node` - restrict resuming to a particular node\n\nNote: by default, Oban does not verify that the given queue exists unless `:local_only`\nis set to `true` as even if the queue does not exist locally, it might be running on\nanother node.","ref":"Oban.html#resume_queue/2-options"},{"type":"function","title":"Example - Oban.resume_queue/2","doc":"Resume a paused default queue:\n\n    Oban.resume_queue(queue: :default)\n\nResume the default queue, but only on the local node:\n\n    Oban.resume_queue(queue: :default, local_only: true)\n\nResume the default queue only on a particular node:\n\n    Oban.resume_queue(queue: :default, node: \"worker.1\")","ref":"Oban.html#resume_queue/2-example"},{"type":"function","title":"Oban.retry_all_jobs/2","doc":"Retries all jobs that match on the given queryable.\n\nIf no queryable is given, Oban will retry all jobs that aren't currently `available` or\n`executing`. Note that regardless of constraints, it will never retry `available` or\n`executing` jobs.","ref":"Oban.html#retry_all_jobs/2"},{"type":"function","title":"Example - Oban.retry_all_jobs/2","doc":"Retries jobs in _any state_ other than `available` or `executing`:\n\n    Oban.retry_all_jobs(Oban.Job)\n    {:ok, 9}\n\nRetries jobs with the `retryable` state:\n\n    Oban.Job\n    |> Ecto.Query.where(state: \"retryable\")\n    |> Oban.retry_all_jobs()\n    {:ok, 3}\n\nRetries all inactive jobs with priority 0\n\n    Oban.Job\n    |> Ecto.Query.where(priority: 0)\n    |> Oban.retry_all_jobs()\n    {:ok, 5}","ref":"Oban.html#retry_all_jobs/2-example"},{"type":"function","title":"Oban.retry_job/2","doc":"Sets a job as `available`, adding attempts if already maxed out. Jobs currently `available` or\n`executing` are ignored. The job is scheduled for immediate execution.","ref":"Oban.html#retry_job/2"},{"type":"function","title":"Example - Oban.retry_job/2","doc":"Retry a job:\n\n    Oban.retry_job(job)\n    :ok","ref":"Oban.html#retry_job/2-example"},{"type":"function","title":"Oban.scale_queue/2","doc":"Scale the concurrency for a queue.","ref":"Oban.html#scale_queue/2"},{"type":"function","title":"Options - Oban.scale_queue/2","doc":"* `:queue` - a string or atom specifying the queue to scale, required\n\n* `:limit` — the new concurrency limit, required\n\n* `:local_only` — whether the queue will be scaled only on the local node, default: `false`\n\n* `:node` - restrict scaling to a particular node\n\nIn addition, all engine-specific queue options are passed along after validation.\n\nNote: by default, Oban does not verify that the given queue exists unless `:local_only`\nis set to `true` as even if the queue does not exist locally, it might be running on\nanother node.","ref":"Oban.html#scale_queue/2-options"},{"type":"function","title":"Example - Oban.scale_queue/2","doc":"Scale a queue up, triggering immediate execution of queued jobs:\n\n    Oban.scale_queue(queue: :default, limit: 50)\n    :ok\n\nScale the queue back down, allowing executing jobs to finish:\n\n    Oban.scale_queue(queue: :default, limit: 5)\n    :ok\n\nScale the queue only on the local node:\n\n    Oban.scale_queue(queue: :default, limit: 10, local_only: true)\n    :ok\n\nScale the queue on a particular node:\n\n    Oban.scale_queue(queue: :default, limit: 10, node: \"worker.1\")\n    :ok","ref":"Oban.html#scale_queue/2-example"},{"type":"function","title":"Oban.start_link/1","doc":"Starts an `Oban` supervision tree linked to the current process.","ref":"Oban.html#start_link/1"},{"type":"function","title":"Options - Oban.start_link/1","doc":"These options are required; without them the supervisor won't start:\n\n* `:repo` — specifies the Ecto repo used to insert and retrieve jobs","ref":"Oban.html#start_link/1-options"},{"type":"function","title":"Primary Options - Oban.start_link/1","doc":"These options determine what the system does at a high level, i.e. which queues to run:\n\n* `:engine` — facilitates inserting, fetching, and otherwise managing jobs.\n\n  There are three built-in engines: `Oban.Engines.Basic` for Postgres databases,\n  `Oban.Engines.Lite` for SQLite3 databases, and `Oban.Engines.Inline` for simplified testing\n  (only available for `:inline` testing mode).\n\n  When the `Oban.Engines.Lite` engine is used the `:notifier` and `:peer` are automatically set\n  to `PG` and `isolated` mode, respectively.\n\n  Additional engines, such as Oban Pro's `SmartEngine` with advanced functionality for Postgres,\n  are also available as an add-on.\n\n  Defaults to the `Basic` engine for Postgres.\n\n* `:log` — either `false` to disable logging or a standard log level (`:error`, `:warning`,\n  `:info`, `:debug`, etc.). This determines whether queries are logged or not; overriding the\n  repo's configured log level. Defaults to `false`, where no queries are logged.\n\n* `:name` — used for supervisor registration, it must be unique across an entire VM instance.\n  Defaults to `Oban` when no name is provided.\n\n* `:node` — used to identify the node that the supervision tree is running in. If no value is\n  provided it will use the `node` name in a distributed system, or the `hostname` in an isolated\n  node. See \"Node Name\" below.\n\n* `:notifier` — used to relay messages between processes, nodes, and the Postgres database.\n\n  There are two built-in notifiers: `Oban.Notifiers.Postgres`, which uses Postgres PubSub; and\n  `Oban.Notifiers.PG`, which uses process groups with distributed erlang. Defaults to the\n  Postgres notifier.\n\n* `:peer` — used to specify which peer module to use for cluster leadership.\n\n  There are two built-in peers: `Oban.Peers.Database`, which uses table-based leadership through\n  the `oban_peers` table; and `Oban.Peers.Global`, which uses global locks through distributed Erlang.\n\n  Leadership can be disabled by setting `peer: false`, but note that centralized plugins like\n  `Cron` won't run without leadership.\n\n  Defaults to the `Database` peer.\n\n* `:plugins` — a list or modules or module/option tuples that are started as children of an Oban\n  supervisor. Any supervisable module is a valid plugin, i.e. a `GenServer` or an `Agent`. May\n  also be set to `false` to disable plugins _and_ disable leadership.\n\n* `:prefix` — the query prefix, or schema, to use for inserting and executing jobs. An\n  `oban_jobs` table must exist within the prefix. See the \"Prefix Support\" section in the module\n  documentation for more details.\n\n* `:queues` — a keyword list where the keys are queue names and the values are the concurrency\n  setting or a keyword list of queue options. For example, setting queues to `[default: 10,\n  exports: 5]` would start the queues `default` and `exports` with a combined concurrency level\n  of 15. The concurrency setting specifies how many jobs _each queue_ will run concurrently.\n\n  Queues accept additional override options to customize their behavior, e.g. by setting\n  `paused` or `dispatch_cooldown` for a specific queue.\n\n  Using an empty list or `false` prevents any queues from starting on init.\n\n* `:testing` — a mode that controls how an instance is configured for testing. When set to\n  `:inline` or `:manual` queues, peers, and plugins are automatically disabled. Defaults to\n  `:disabled`, no test mode.","ref":"Oban.html#start_link/1-primary-options"},{"type":"function","title":"Twiddly Options - Oban.start_link/1","doc":"Additional options used to tune system behaviour. These are primarily useful for testing or\ntroubleshooting and don't usually need modification.\n\n* `:dispatch_cooldown` — the minimum number of milliseconds a producer will wait before fetching\n  and running more jobs. A slight cooldown period prevents a producer from flooding with\n  messages and thrashing the database. The cooldown period _directly impacts_ a producer's\n  throughput: jobs per second for a single queue is calculated by `(1000 / cooldown) * limit`.\n  For example, with a `5ms` cooldown and a queue limit of `25` a single queue can run 5,000\n  jobs/sec.\n\n  The default is `5ms` and the minimum is `1ms`, which is likely faster than the database can\n  return new jobs to run.\n\n* `:insert_trigger` — whether to dispatch notifications to relevant queues as jobs are inserted\n  into the database. At high load, e.g. thousands or more job inserts per second, notifications\n  may become a bottleneck.\n\n  The trigger mechanism is designed to make jobs execute immediately after insert, rather than\n  up to `:stage_interval` (1 second) afterwards, and it can safely be disabled to improve insert\n  throughput.\n\n  Defaults to `true`, with triggering enabled.\n\n* `:shutdown_grace_period` — the amount of time a queue will wait for executing jobs to complete\n  before hard shutdown, specified in milliseconds. The default is `15_000`, or 15 seconds.\n\n* `:stage_interval` — the number of milliseconds between making scheduled jobs available and\n  notifying relevant queues that jobs are available. This is directly tied to the resolution of\n  `scheduled` or `retryable` jobs and how frequently the database is checked for jobs to run. To\n  minimize database load, only `5_000` jobs are staged at each interval.\n\n  Only the leader node stages jobs and notifies queues when the `:notifier's` pubsub\n  notifications are functional. If pubsub messages can't get through then staging switches to a\n  less efficient \"local\" mode in which all nodes poll for jobs to run.\n\n  Setting the interval to `:infinity` disables staging entirely. The default is `1_000ms`.","ref":"Oban.html#start_link/1-twiddly-options"},{"type":"function","title":"Example - Oban.start_link/1","doc":"Start a stand-alone `Oban` instance:\n\n    {:ok, pid} = Oban.start_link(repo: MyApp.Repo, queues: [default: 10])\n\nTo start an `Oban` instance within an application's supervision tree:\n\n    def start(_type, _args) do\n      children = [MyApp.Repo, {Oban, repo: MyApp.Repo, queues: [default: 10]}]\n\n      Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n    end\n\nStart multiple, named `Oban` supervisors within a supervision tree:\n\n      children = [\n        MyApp.Repo,\n        {Oban, name: Oban.A, repo: MyApp.Repo, queues: [default: 10]},\n        {Oban, name: Oban.B, repo: MyApp.Repo, queues: [special: 10]},\n      ]\n\n      Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n\nStart a local `Oban` instance for SQLite:\n\n    {:ok, pid} = Oban.start_link(engine: Oban.Engines.Lite, repo: MyApp.Repo)","ref":"Oban.html#start_link/1-example"},{"type":"function","title":"Node Name - Oban.start_link/1","doc":"When the `node` value has not been configured it is generated based on the environment:\n\n1. If the local node is alive (e.g. in a distributed system, or when running from a mix release)\n   the node name is used\n2. In a Heroku environment the system environment's `DYNO` value is used\n3. Otherwise, the system hostname is used\n\nWhen running a mix release on a Heroku node, the node is alive even if not part of a\ndistributed system. In order to use the `DYNO` value, configure the node value using runtime\nconfiguration via `config/runtime.exs`:\n\n      config :my_app, Oban,\n        node: System.get_env(\"DYNO\", \"nonode@nohost\")","ref":"Oban.html#start_link/1-node-name"},{"type":"function","title":"Oban.start_queue/2","doc":"Start a new supervised queue.\n\nBy default this starts a new supervised queue across all nodes running Oban on the same database\nand prefix. You can pass the option `local_only: true` if you prefer to start the queue only on\nthe local node.","ref":"Oban.html#start_queue/2"},{"type":"function","title":"Options - Oban.start_queue/2","doc":"* `:queue` - a string or atom specifying the queue to start, required\n* `:local_only` - whether the queue will be started only on the local node, default: `false`\n* `:limit` - set the concurrency limit, required\n* `:paused` — set whether the queue starts in the \"paused\" state, optional\n\nIn addition, all engine-specific queue options are passed along after validation.","ref":"Oban.html#start_queue/2-options"},{"type":"function","title":"Example - Oban.start_queue/2","doc":"Start the `:priority` queue with a concurrency limit of 10 across the connected nodes.\n\n    Oban.start_queue(queue: :priority, limit: 10)\n    :ok\n\nStart the `:media` queue with a concurrency limit of 5 only on the local node.\n\n    Oban.start_queue(queue: :media, limit: 5, local_only: true)\n    :ok\n\nStart the `:media` queue on a particular node.\n\n    Oban.start_queue(queue: :media, limit: 5, node: \"worker.1\")\n    :ok\n\nStart the `:media` queue in a `paused` state.\n\n    Oban.start_queue(queue: :media, limit: 5, paused: true)\n    :ok","ref":"Oban.html#start_queue/2-example"},{"type":"function","title":"Oban.stop_queue/2","doc":"Shutdown a queue's supervision tree and stop running jobs for that queue.\n\nBy default this action will occur across all the running nodes. Still, if you prefer to stop the\nqueue's supervision tree and stop running jobs for that queue only on the local node, you can\npass the option: `local_only: true`\n\nThe shutdown process pauses the queue first and allows current jobs to exit gracefully, provided\nthey finish within the shutdown limit.\n\nNote: by default, Oban does not verify that the given queue exists unless `:local_only`\nis set to `true` as even if the queue does not exist locally, it might be running on\nanother node.","ref":"Oban.html#stop_queue/2"},{"type":"function","title":"Options - Oban.stop_queue/2","doc":"* `:queue` - a string or atom specifying the queue to stop, required\n\n* `:local_only` - whether the queue will be stopped only on the local node, default: `false`\n\n* `:node` - restrict stopping to a particular node","ref":"Oban.html#stop_queue/2-options"},{"type":"function","title":"Example - Oban.stop_queue/2","doc":"Stop a running queue on all nodes:\n\n    Oban.stop_queue(queue: :default)\n    :ok\n\nStop the queue only on the local node:\n\n    Oban.stop_queue(queue: :media, local_only: true)\n    :ok\n\nStop the queue only on a particular node:\n\n    Oban.stop_queue(queue: :media, node: \"worker.1\")\n    :ok","ref":"Oban.html#stop_queue/2-example"},{"type":"function","title":"Oban.whereis/1","doc":"Returns the pid of the root Oban process for the given name.","ref":"Oban.html#whereis/1"},{"type":"function","title":"Example - Oban.whereis/1","doc":"Find the default instance:\n\n    Oban.whereis(Oban)\n\nFind a dynamically named instance:\n\n    Oban.whereis({:oban, 1})","ref":"Oban.html#whereis/1-example"},{"type":"module","title":"Oban.Job","doc":"A Job is an Ecto schema used for asynchronous execution.\n\nJob changesets are created by your application code and inserted into the database for\nasynchronous execution. Jobs can be inserted along with other application data as part of a\ntransaction, which guarantees that jobs will only be triggered from a successful transaction.","ref":"Oban.Job.html"},{"type":"type","title":"Oban.Job.args/0","doc":"","ref":"Oban.Job.html#t:args/0"},{"type":"type","title":"Oban.Job.changeset/0","doc":"","ref":"Oban.Job.html#t:changeset/0"},{"type":"type","title":"Oban.Job.changeset_fun/0","doc":"","ref":"Oban.Job.html#t:changeset_fun/0"},{"type":"type","title":"Oban.Job.changeset_list/0","doc":"","ref":"Oban.Job.html#t:changeset_list/0"},{"type":"type","title":"Oban.Job.changeset_list_fun/0","doc":"","ref":"Oban.Job.html#t:changeset_list_fun/0"},{"type":"type","title":"Oban.Job.errors/0","doc":"","ref":"Oban.Job.html#t:errors/0"},{"type":"function","title":"Oban.Job.format_attempt/1","doc":"Normalize, blame, and format a job's `unsaved_error` into the stored error format.\n\nFormatted errors are stored in a job's `errors` field.","ref":"Oban.Job.html#format_attempt/1"},{"type":"function","title":"Oban.Job.new/2","doc":"Construct a new job changeset ready for insertion into the database.","ref":"Oban.Job.html#new/2"},{"type":"function","title":"Options - Oban.Job.new/2","doc":"* `:max_attempts` — the maximum number of times a job can be retried if there are errors\n    during execution\n\n  * `:meta` — a map containing additional information about the job\n\n  * `:priority` — a numerical indicator from 0 to 9 of how important this job is relative to\n    other jobs in the same queue. The lower the number, the higher priority the job.\n\n  * `:queue` — a named queue to push the job into. Jobs may be pushed into any queue, regardless\n    of whether jobs are currently being processed for the queue.\n\n  * `:replace` - a list of keys to replace per state on a unique conflict\n\n  * `:scheduled_at` - a time in the future after which the job should be executed\n\n  * `:schedule_in` - the number of seconds until the job should be executed or a tuple containing\n    a number and unit\n\n  * `:tags` — a list of tags to group and organize related jobs, i.e. to identify scheduled jobs\n\n  * `:unique` — a keyword list of options specifying how uniqueness will be calculated. The\n    options define which fields will be used, for how long, with which keys, and for which states.\n\n  * `:worker` — a module to execute the job in. The module must implement the `Oban.Worker`\n    behaviour.","ref":"Oban.Job.html#new/2-options"},{"type":"function","title":"Examples - Oban.Job.new/2","doc":"Insert a job with the `:default` queue:\n\n    %{id: 1, user_id: 2}\n    |> Oban.Job.new(queue: :default, worker: MyApp.Worker)\n    |> Oban.insert()\n\nGenerate a pre-configured job for `MyApp.Worker`:\n\n    MyApp.Worker.new(%{id: 1, user_id: 2})\n\nSchedule a job to run in 5 seconds:\n\n    MyApp.Worker.new(%{id: 1}, schedule_in: 5)\n\nSchedule a job to run in 5 minutes:\n\n    MyApp.Worker.new(%{id: 1}, schedule_in: {5, :minutes})\n\nInsert a job, ensuring that it is unique within the past minute:\n\n    MyApp.Worker.new(%{id: 1}, unique: [period: {1, :minute}])\n\nInsert a unique job where the period is compared to the `scheduled_at` timestamp rather than\n`inserted_at`:\n\n    MyApp.Worker.new(%{id: 1}, unique: [period: 60, timestamp: :scheduled_at])\n\nInsert a unique job based only on the worker field, and within multiple states:\n\n    fields = [:worker]\n    states = [:available, :scheduled, :executing, :retryable, :completed]\n\n    MyApp.Worker.new(%{id: 1}, unique: [fields: fields, period: 60, states: states])\n\nInsert a unique job considering only the worker and specified keys in the args:\n\n    keys = [:account_id, :url]\n    args = %{account_id: 1, url: \"https://example.com\"}\n\n    MyApp.Worker.new(args, unique: [fields: [:args, :worker], keys: keys])\n\nInsert a unique job considering only specified keys in the meta:\n\n    unique = [fields: [:meta], keys: [:slug]]\n\n    MyApp.Worker.new(%{id: 1}, meta: %{slug: \"unique-key\"}, unique: unique)","ref":"Oban.Job.html#new/2-examples"},{"type":"type","title":"Oban.Job.option/0","doc":"","ref":"Oban.Job.html#t:option/0"},{"type":"type","title":"Oban.Job.replace_by_state_option/0","doc":"","ref":"Oban.Job.html#t:replace_by_state_option/0"},{"type":"type","title":"Oban.Job.replace_option/0","doc":"","ref":"Oban.Job.html#t:replace_option/0"},{"type":"type","title":"Oban.Job.schedule_in_option/0","doc":"","ref":"Oban.Job.html#t:schedule_in_option/0"},{"type":"function","title":"Oban.Job.states/0","doc":"A canonical list of all possible job states.\n\nThis may be used to build up `:unique` options without duplicating states in application code.","ref":"Oban.Job.html#states/0"},{"type":"function","title":"Examples - Oban.Job.states/0","doc":"iex> Oban.Job.states() -- [:completed, :discarded]\n    [:scheduled, :available, :executing, :retryable, :cancelled]","ref":"Oban.Job.html#states/0-examples"},{"type":"function","title":"Job State Transitions - Oban.Job.states/0","doc":"* `:scheduled`—Jobs inserted with `scheduled_at` in the future are `:scheduled`. After the\n  `scheduled_at` time has elapsed the `Oban.Plugins.Stager` will transition them to `:available`\n\n* `:available`—Jobs without a future `scheduled_at` timestamp are inserted as `:available` and may\n  execute immediately\n\n* `:executing`—Available jobs may be ran, at which point they are `:executing`\n\n* `:retryable`—Jobs that fail and haven't exceeded their max attempts are transitioned to\n  `:retryable` and rescheduled until after a backoff period. Once the backoff has elapsed the\n  `Oban.Plugins.Stager` will transition them back to `:available`\n\n* `:completed`—Jobs that finish executing succesfully are marked `:completed`\n\n* `:discarded`—Jobs that fail and exhaust their max attempts, or return a `:discard` tuple during\n  execution, are marked `:discarded`\n\n* `:cancelled`—Jobs that are cancelled intentionally","ref":"Oban.Job.html#states/0-job-state-transitions"},{"type":"type","title":"Oban.Job.t/0","doc":"","ref":"Oban.Job.html#t:t/0"},{"type":"type","title":"Oban.Job.tags/0","doc":"","ref":"Oban.Job.html#t:tags/0"},{"type":"type","title":"Oban.Job.time_unit/0","doc":"","ref":"Oban.Job.html#t:time_unit/0"},{"type":"function","title":"Oban.Job.to_map/1","doc":"Convert a Job changeset into a map suitable for database insertion.","ref":"Oban.Job.html#to_map/1"},{"type":"function","title":"Examples - Oban.Job.to_map/1","doc":"Convert a worker generated changeset into a plain map:\n\n    %{id: 123}\n    |> MyApp.Worker.new()\n    |> Oban.Job.to_map()","ref":"Oban.Job.html#to_map/1-examples"},{"type":"type","title":"Oban.Job.unique_field/0","doc":"","ref":"Oban.Job.html#t:unique_field/0"},{"type":"type","title":"Oban.Job.unique_option/0","doc":"","ref":"Oban.Job.html#t:unique_option/0"},{"type":"type","title":"Oban.Job.unique_period/0","doc":"","ref":"Oban.Job.html#t:unique_period/0"},{"type":"type","title":"Oban.Job.unique_state/0","doc":"","ref":"Oban.Job.html#t:unique_state/0"},{"type":"type","title":"Oban.Job.unique_timestamp/0","doc":"","ref":"Oban.Job.html#t:unique_timestamp/0"},{"type":"behaviour","title":"Oban.Migration","doc":"Migrations create and modify the database tables Oban needs to function.","ref":"Oban.Migration.html"},{"type":"behaviour","title":"Usage - Oban.Migration","doc":"To use migrations in your application you'll need to generate an `Ecto.Migration` that wraps\ncalls to `Oban.Migration`:\n\n```bash\nmix ecto.gen.migration add_oban\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`Oban.Migration`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddOban do\n  use Ecto.Migration\n\n  def up, do: Oban.Migrations.up()\n\n  def down, do: Oban.Migrations.down()\nend\n```\n\nThis will run all of Oban's versioned migrations for your database.\n\nNow, run the migration to create the table:\n\n```bash\nmix ecto.migrate\n```\n\nMigrations between versions are idempotent. As new versions are released, you may need to run\nadditional migrations. To do this, generate a new migration:\n\n```bash\nmix ecto.gen.migration upgrade_oban_to_v12\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`Oban.Migration`, passing a version number:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.UpgradeObanToV11 do\n  use Ecto.Migration\n\n  def up, do: Oban.Migrations.up(version: 12)\n\n  def down, do: Oban.Migrations.down(version: 12)\nend\n```","ref":"Oban.Migration.html#module-usage"},{"type":"behaviour","title":"Isolation with Prefixes - Oban.Migration","doc":"Oban supports namespacing through PostgreSQL schemas, also called \"prefixes\" in Ecto. With\nprefixes your jobs table can reside outside of your primary schema (usually public) and you can\nhave multiple separate job tables.\n\nTo use a prefix you first have to specify it within your migration:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedObanJobsTable do\n  use Ecto.Migration\n\n  def up, do: Oban.Migrations.up(prefix: \"private\")\n\n  def down, do: Oban.Migrations.down(prefix: \"private\")\nend\n```\n\nThe migration will create the \"private\" schema and all tables, functions and triggers within\nthat schema. With the database migrated you'll then specify the prefix in your configuration:\n\n```elixir\nconfig :my_app, Oban,\n  prefix: \"private\",\n  ...\n```\n\nIn some cases, for example if your \"private\" schema already exists and your database user in\nproduction doesn't have permissions to create a new schema, trying to create the schema from the\nmigration will result in an error. In such situations, it may be useful to inhibit the creation\nof the \"private\" schema:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedObanJobsTable do\n  use Ecto.Migration\n\n  def up, do: Oban.Migrations.up(prefix: \"private\", create_schema: false)\n\n  def down, do: Oban.Migrations.down(prefix: \"private\")\nend\n```","ref":"Oban.Migration.html#module-isolation-with-prefixes"},{"type":"behaviour","title":"Migrating Without Ecto - Oban.Migration","doc":"If your application uses something other than Ecto for migrations, be it an external system or\nanother ORM, it may be helpful to create plain SQL migrations for Oban database schema changes.\n\nThe simplest mechanism for obtaining the SQL changes is to create the migration locally and run\n`mix ecto.migrate --log-migrations-sql`. That will log all of the generated SQL, which you can\nthen paste into your migration system of choice.\n\nAlternatively, if you'd like a more automated approach, try using the [oban_migations_sql][sql]\nproject to generate `up` and `down` SQL migrations for you.\n\n[sql]: https://github.com/btwb/oban_migrations_sql","ref":"Oban.Migration.html#module-migrating-without-ecto"},{"type":"function","title":"Oban.Migration.down/1","doc":"Run the `down` changes for all migrations between the current version and the initial version.","ref":"Oban.Migration.html#down/1"},{"type":"function","title":"Example - Oban.Migration.down/1","doc":"Run all migrations from current version down to the first:\n\n    Oban.Migration.down()\n\nRun migrations down to and including a specified version:\n\n    Oban.Migration.down(version: 5)\n\nRun migrations in an alternate prefix:\n\n    Oban.Migration.down(prefix: \"payments\")","ref":"Oban.Migration.html#down/1-example"},{"type":"callback","title":"Oban.Migration.down/1","doc":"Migrates storage down to the previous version.","ref":"Oban.Migration.html#c:down/1"},{"type":"function","title":"Oban.Migration.migrated_version/1","doc":"Check the latest version the database is migrated to.","ref":"Oban.Migration.html#migrated_version/1"},{"type":"function","title":"Example - Oban.Migration.migrated_version/1","doc":"Oban.Migration.migrated_version()","ref":"Oban.Migration.html#migrated_version/1-example"},{"type":"callback","title":"Oban.Migration.migrated_version/1","doc":"Identifies the last migrated version.","ref":"Oban.Migration.html#c:migrated_version/1"},{"type":"function","title":"Oban.Migration.up/1","doc":"Run the `up` changes for all migrations between the initial version and the current version.","ref":"Oban.Migration.html#up/1"},{"type":"function","title":"Example - Oban.Migration.up/1","doc":"Run all migrations up to the current version:\n\n    Oban.Migration.up()\n\nRun migrations up to a specified version:\n\n    Oban.Migration.up(version: 2)\n\nRun migrations in an alternate prefix:\n\n    Oban.Migration.up(prefix: \"payments\")\n\nRun migrations in an alternate prefix but don't try to create the schema:\n\n    Oban.Migration.up(prefix: \"payments\", create_schema: false)\n\nDisable `UNLOGGED` tables for databases that don't support it (e.g. Yugabyte)\n\n    Oban.Migration.up(unlogged: false)","ref":"Oban.Migration.html#up/1-example"},{"type":"callback","title":"Oban.Migration.up/1","doc":"Migrates storage up to the latest version.","ref":"Oban.Migration.html#c:up/1"},{"type":"module","title":"Oban.Testing","doc":"This module simplifies testing workers and making assertions about enqueued jobs when testing in\n`:manual` mode.\n\nAssertions may be made on any property of a job, but you'll typically want to check by `args`,\n`queue` or `worker`.","ref":"Oban.Testing.html"},{"type":"module","title":"Usage - Oban.Testing","doc":"The most convenient way to use `Oban.Testing` is to `use` the module:\n\n    use Oban.Testing, repo: MyApp.Repo\n\nThat will define the helper functions you'll use to make assertions on the jobs that should (or\nshould not) be inserted in the database while testing.\n\nIf you're using namespacing through Postgres schemas, also called \"prefixes\" in Ecto, you\nshould set the `prefix` option:\n\n    use Oban.Testing, repo: MyApp.Repo, prefix: \"business\"\n\nUnless overridden, the default `prefix` is `public`.","ref":"Oban.Testing.html#module-usage"},{"type":"module","title":"Adding to Case Templates - Oban.Testing","doc":"To include helpers in all of your tests you can add it to your case template:\n\n```elixir\ndefmodule MyApp.DataCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      use Oban.Testing, repo: MyApp.Repo\n\n      import Ecto\n      import Ecto.Changeset\n      import Ecto.Query\n      import MyApp.DataCase\n\n      alias MyApp.Repo\n    end\n  end\nend\n```","ref":"Oban.Testing.html#module-adding-to-case-templates"},{"type":"module","title":"Examples - Oban.Testing","doc":"After the test helpers are imported, you can make assertions about enqueued (available or\nscheduled) jobs in your tests.\n\nHere are a few examples that demonstrate what's possible:\n\n```elixir\n# Assert that a job was already enqueued\nassert_enqueued worker: MyWorker, args: %{id: 1}\n\n# Assert that a job was enqueued or will be enqueued in the next 100ms\nassert_enqueued [worker: MyWorker, args: %{id: 1}], 100\n\n# Refute that a job was already enqueued\nrefute_enqueued queue: \"special\", args: %{id: 2}\n\n# Refute that a job was already enqueued or would be enqueued in the next 100ms\nrefute_enqueued queue: \"special\", args: %{id: 2}, 100\n\n# Make assertions on a list of all jobs matching some options\nassert [%{args: %{\"id\" => 1}}] = all_enqueued(worker: MyWorker)\n\n# Assert that no jobs are enqueued in any queues\nassert [] = all_enqueued()\n```\n\nNote that the final example, using `all_enqueued/1`, returns a raw list of matching jobs and\ndoes not make an assertion by itself. This makes it possible to test using pattern matching at\nthe expense of being more verbose.\n\nSee the docs for `assert_enqueued/1,2`, `refute_enqueued/1,2`, and `all_enqueued/1` for more\nexamples.","ref":"Oban.Testing.html#module-examples"},{"type":"module","title":"Matching Timestamps - Oban.Testing","doc":"In order to assert a job has been scheduled at a certain time, you will need to match against\nthe `scheduled_at` attribute of the enqueued job.\n\n    in_an_hour = DateTime.add(DateTime.utc_now(), 3600, :second)\n    assert_enqueued worker: MyApp.Worker, scheduled_at: in_an_hour\n\nBy default, Oban will apply a 1 second delta to all timestamp fields of jobs, so that small\ndeviations between the actual value and the expected one are ignored. You may configure this\ndelta by passing a tuple of value and a `delta` option (in seconds) to corresponding keyword:\n\n    assert_enqueued worker: MyApp.Worker, scheduled_at: {in_an_hour, delta: 10}","ref":"Oban.Testing.html#module-matching-timestamps"},{"type":"function","title":"Oban.Testing.all_enqueued/1","doc":"Retrieve all currently enqueued jobs matching a set of options.\n\nOnly jobs matching all of the provided arguments will be returned. Additionally, jobs are\nreturned in descending order where the most recently enqueued job will be listed first.","ref":"Oban.Testing.html#all_enqueued/1"},{"type":"function","title":"Examples - Oban.Testing.all_enqueued/1","doc":"Assert based on only _some_ of a job's args:\n\n    assert [%{args: %{\"id\" => 1}}] = all_enqueued(worker: MyWorker)\n\nAssert that exactly one job was inserted for a queue:\n\n    assert [%Oban.Job{}] = all_enqueued(queue: :alpha)\n\nAssert that there aren't any jobs enqueued for any queues or workers:\n\n    assert [] = all_enqueued()","ref":"Oban.Testing.html#all_enqueued/1-examples"},{"type":"function","title":"Oban.Testing.assert_enqueued/1","doc":"Assert that a job with matching fields is enqueued.\n\nOnly values for the provided fields are checked. For example, an assertion made on `worker:\n\"MyWorker\"` will match _any_ jobs for that worker, regardless of every other field.","ref":"Oban.Testing.html#assert_enqueued/1"},{"type":"function","title":"Examples - Oban.Testing.assert_enqueued/1","doc":"Assert that a job is enqueued for a certain worker and args:\n\n    assert_enqueued worker: MyWorker, args: %{id: 1}\n\nAssert that a job is enqueued for a particular queue and priority:\n\n    assert_enqueued queue: :business, priority: 3\n\nAssert that a job's args deeply match:\n\n    assert_enqueued args: %{config: %{enabled: true}}\n\nUse the `:_` wildcard to assert that a job's meta has a key with any value:\n\n    assert_enqueued meta: %{batch_id: :_}","ref":"Oban.Testing.html#assert_enqueued/1-examples"},{"type":"function","title":"Oban.Testing.assert_enqueued/2","doc":"Assert that a job with particular options is or will be enqueued within a timeout period.\n\nSee `assert_enqueued/1` for additional details.","ref":"Oban.Testing.html#assert_enqueued/2"},{"type":"function","title":"Examples - Oban.Testing.assert_enqueued/2","doc":"Assert that a job will be enqueued in the next 100ms:\n\n    assert_enqueued [worker: MyWorker], 100","ref":"Oban.Testing.html#assert_enqueued/2-examples"},{"type":"function","title":"Oban.Testing.build_job/3","doc":"Construct a job from a worker, args, and options.\n\nThe helper makes the following assertions:\n\n* That the worker implements the `Oban.Worker` behaviour\n* That the options provided build a valid job\n\nThis helper is used to build jobs for execution by `perform_job/2`.","ref":"Oban.Testing.html#build_job/3"},{"type":"function","title":"Examples - Oban.Testing.build_job/3","doc":"Build a job without args:\n\n    job = build_job(MyWorker, %{})\n\nBuild a job with stringified args:\n\n    assert %{args: %{\"id\" => 1}} = build_job(MyWorker, %{id: 1})\n\nBuild a job with custom options:\n\n    assert %{attempt: 5, priority: 9} = build_job(MyWorker, %{}, attempt: 5, priority: 9)","ref":"Oban.Testing.html#build_job/3-examples"},{"type":"function","title":"Oban.Testing.perform_job/2","doc":"Execute a job using the given config options.\n\nSee `perform_job/3` for more details and examples.","ref":"Oban.Testing.html#perform_job/2"},{"type":"function","title":"Examples - Oban.Testing.perform_job/2","doc":"Execute a job without any options:\n\n    assert :ok = perform_job(job, [])\n\nExecute a job with a custom prefix and repo:\n\n    assert :ok = perform_job(job, prefix: \"private\", repo: MyApp.Repo)","ref":"Oban.Testing.html#perform_job/2-examples"},{"type":"function","title":"Oban.Testing.perform_job/3","doc":"Construct a job and execute it with a worker module.\n\nThis reduces boilerplate when constructing jobs for unit tests and checks for common pitfalls.\nFor example, it automatically converts `args` to string keys before calling `perform/1`,\nensuring that perform clauses aren't erroneously trying to match atom keys.\n\nThe helper makes the following assertions:\n\n* That the worker implements the `Oban.Worker` behaviour\n* That the options provided build a valid job\n* That the return is valid, e.g. `:ok`, `{:ok, value}`, `{:error, value}` etc.\n\nIf all of the assertions pass, then the function returns the result of `perform/1` for you to\nmake additional assertions on.","ref":"Oban.Testing.html#perform_job/3"},{"type":"function","title":"Examples - Oban.Testing.perform_job/3","doc":"Successfully execute a job with some string arguments:\n\n    assert :ok = perform_job(MyWorker, %{\"id\" => 1})\n\nSuccessfully execute a job and assert that it returns an error tuple:\n\n    assert {:error, _} = perform_job(MyWorker, %{\"bad\" => \"arg\"})\n\nExecute a job with the args keys automatically stringified:\n\n    assert :ok = perform_job(MyWorker, %{id: 1})\n\nExercise custom attempt handling within a worker by passing options:\n\n    assert :ok = perform_job(MyWorker, %{}, attempt: 42)\n\nCause a test failure because the provided worker isn't real:\n\n    assert :ok = perform_job(Vorker, %{\"id\" => 1})","ref":"Oban.Testing.html#perform_job/3-examples"},{"type":"type","title":"Oban.Testing.perform_opts/0","doc":"","ref":"Oban.Testing.html#t:perform_opts/0"},{"type":"function","title":"Oban.Testing.refute_enqueued/1","doc":"Refute that a job with particular options has been enqueued.\n\nSee `assert_enqueued/1` for additional details.","ref":"Oban.Testing.html#refute_enqueued/1"},{"type":"function","title":"Examples - Oban.Testing.refute_enqueued/1","doc":"Refute that a job is enqueued for a certain worker:\n\n    refute_enqueued worker: MyWorker\n\nRefute that a job is enqueued for a certain worker and args:\n\n    refute_enqueued worker: MyWorker, args: %{id: 1}\n\nRefute that a job's nested args match:\n\n    refute_enqueued args: %{config: %{enabled: false}}\n\nUse the `:_` wildcard to refute that a job's meta has a key with any value:\n\n    refute_enqueued meta: %{batch_id: :_}","ref":"Oban.Testing.html#refute_enqueued/1-examples"},{"type":"function","title":"Oban.Testing.refute_enqueued/2","doc":"Refute that a job with particular options is or will be enqueued within a timeout period.\n\nSee `assert_enqueued/1` for additional details.","ref":"Oban.Testing.html#refute_enqueued/2"},{"type":"function","title":"Examples - Oban.Testing.refute_enqueued/2","doc":"Refute that a job will be enqueued in the next 100ms:\n\n    refute_enqueued [worker: MyWorker], 100","ref":"Oban.Testing.html#refute_enqueued/2-examples"},{"type":"function","title":"Oban.Testing.refute_enqueued/3","doc":"","ref":"Oban.Testing.html#refute_enqueued/3"},{"type":"function","title":"Oban.Testing.with_testing_mode/2","doc":"Change the testing mode within the context of a function.\n\nOnly `:manual` and `:inline` mode are supported, as `:disabled` implies that supervised queues\nand plugins are running and this function won't start any processes.","ref":"Oban.Testing.html#with_testing_mode/2"},{"type":"function","title":"Examples - Oban.Testing.with_testing_mode/2","doc":"Switch to `:manual` mode when an Oban instance is configured for `:inline` testing:\n\n    Oban.Testing.with_testing_mode(:manual, fn ->\n      Oban.insert(MyWorker.new(%{id: 123}))\n\n      assert_enqueued worker: MyWorker, args: %{id: 123}\n    end)\n\nVisa-versa, switch to `:inline` mode:\n\n    Oban.Testing.with_testing_mode(:inline, fn ->\n      {:ok, %Job{state: \"completed\"}} = Oban.insert(MyWorker.new(%{id: 123}))\n    end)","ref":"Oban.Testing.html#with_testing_mode/2-examples"},{"type":"behaviour","title":"Oban.Worker","doc":"Defines a behavior and macro to guide the creation of worker modules.\n\nWorker modules do the work of processing a job. At a minimum they must define a `c:perform/1`\nfunction, which is called with the full `Oban.Job` struct.","ref":"Oban.Worker.html"},{"type":"behaviour","title":"Defining Workers - Oban.Worker","doc":"Worker modules are defined by using `Oban.Worker`. `use Oban.Worker` supports the following\noptions:\n\n* `:max_attempts` — Integer specifying how many times the job will be retried. Defaults to `20`\n* `:priority` — Integer from 0 (highest priority) to 9 (lowest priority). Defaults to `0`\n* `:queue` — Name of a queue as an atom. Defaults to `:default`\n* `:tags` — A list of strings representing the tags to associate with jobs. Defaults to `[]`\n* `:replace` — A nested list of job statuses and fields to be replaced when a job is executed.\n  Defaults to `[]`\n* `:unique` — a keyword list that determines how jobs are uniquely identified. It may be `true`\n  to enable uniqueness with the default options, or `false` to explicitly disable uniqueness.\n  Defaults to `false`\n\nThe following is a basic workers that uses the defaults:\n\n    defmodule MyApp.Workers.Basic do\n      use Oban.Worker\n\n      @impl Oban.Worker\n      def perform(%Oban.Job{args: args}) do\n        IO.inspect(args)\n        :ok\n      end\n    end\n\nWhich is equivalent to this worker, which sets all options explicitly:\n\n    defmodule MyApp.Workers.Basic do\n      use Oban.Worker,\n        max_attempts: 20,\n        priority: 0,\n        queue: :default,\n        tags: [],\n        replace: [],\n        unique: false\n\n      @impl Oban.Worker\n      def perform(%Oban.Job{args: args}) do\n        IO.inspect(args)\n        :ok\n      end\n    end\n\nThe following example defines a complex worker module to process jobs in the `events` queue.\nIt then dials down the priority from `0` to `1`, limits retrying on failures to ten attempts,\nadds a `business` tag, and ensures that duplicate jobs aren't enqueued within a\n30 second period:\n\n    defmodule MyApp.Workers.Business do\n      use Oban.Worker,\n        queue: :events,\n        priority: 1,\n        max_attempts: 10,\n        tags: [\"business\"],\n        replace: [scheduled: [:scheduled_at]],\n        unique: [period: 30]\n\n      @impl Oban.Worker\n      def perform(%Oban.Job{attempt: attempt}) when attempt > 3 do\n        IO.inspect(attempt)\n      end\n\n      def perform(job) do\n        IO.inspect(job.args)\n      end\n    end\n\n> #### Options at Compile-Time {: .warning}\n>\n> Like all `use` macros, options are defined at compile time. Avoid using\n> `Application.get_env/2` to define worker options. Instead, pass dynamic options at runtime\n> by passing them to the worker's `c:new/2` function:\n>\n> ```elixir\n> MyApp.MyWorker.new(args, queue: dynamic_queue)\n> ```\n\nThe `c:perform/1` function receives an `Oban.Job` struct as an argument. This allows workers to\nchange the behavior of `c:perform/1` based on attributes of the job, such as the args, number of\nexecution attempts, or when it was inserted.\n\nThe value returned from `c:perform/1` can control whether the job is a success or a failure:\n\n* `:ok` or `{:ok, value}` — the job is successful and marked as `completed`.  The `value` from\n  success tuples is ignored.\n\n* `{:cancel, reason}` — cancel executing the job and stop retrying it. An error is recorded\n  using the provided `reason`. The job is marked as `cancelled`.\n\n* `{:error, error}` — the job failed, record the error. If `max_attempts` has not been reached\n  already, the job is marked as `retryable` and scheduled to run again. Otherwise, the job is\n  marked as `discarded` and won't be retried.\n\n* `{:snooze, seconds}` — mark the job as `snoozed` and schedule it to run again `seconds` in the\n  future. See [Snoozing](#module-snoozing-jobs) for more details.\n\nIn addition to explicit return values, any _unhandled exception_, _exit_ or _throw_ will fail\nthe job and schedule a retry under the same conditions as in the `{:error, error}` case.\n\nAs an example of error tuple handling, this worker will return an error tuple when the `value`\nis less than one:\n\n    defmodule MyApp.Workers.ErrorExample do\n      use Oban.Worker\n\n      @impl Worker\n      def perform(%{args: %{\"value\" => value}}) do\n        if value > 1 do\n          :ok\n        else\n          {:error, \"invalid value given: \" <> inspect(value)}\n        end\n      end\n    end\n\nThe error tuple is wrapped in an `Oban.PerformError` with a formatted message. The error tuple\nitself is available through the exception's `:reason` field.","ref":"Oban.Worker.html#module-defining-workers"},{"type":"behaviour","title":"Enqueuing Jobs - Oban.Worker","doc":"All workers implement a `c:new/2` function that converts an args map into a job changeset\nsuitable for inserting into the database for later execution:\n\n    %{in_the: \"business\", of_doing: \"business\"}\n    |> MyApp.Workers.Business.new()\n    |> Oban.insert()\n\nThe worker's defaults may be overridden by passing options:\n\n    %{vote_for: \"none of the above\"}\n    |> MyApp.Workers.Business.new(queue: \"special\", max_attempts: 5)\n    |> Oban.insert()\n\nUniqueness options may also be overridden by passing options:\n\n    %{expensive: \"business\"}\n    |> MyApp.Workers.Business.new(unique: [period: 120, fields: [:worker]])\n    |> Oban.insert()\n\n`unique` options aren't merged, they are overridden entirely.\n\nYou can also insert multiple jobs within a single transaction—see\n`Oban.insert/5`.\n\nJobs that are inserted are executed as soon as possible by default. If you need\nto **schedule** jobs in the future, see the guide for [*Scheduling Jobs*](scheduling_jobs.html).\n\nSee `Oban.Job` for all available options, and the\n[*Job Uniqueness* guide](job_uniqueness.html) for more information about unique jobs.","ref":"Oban.Worker.html#module-enqueuing-jobs"},{"type":"behaviour","title":"Customizing Backoff - Oban.Worker","doc":"When jobs fail they may be retried again in the future using a backoff algorithm. By default the\nbackoff is exponential with a fixed padding of 15 seconds and a small amount of jitter. The\njitter helps to prevent jobs that fail simultaneously from consistently retrying at the same\ntime. With the default backoff behavior the 20th attempt will occur around 12 days after the\n19th attempt, and a total of 25 days after the first attempt.\n\nIf the default strategy is too aggressive or otherwise unsuited to your app's workload you can\ndefine a custom backoff function using the `c:backoff/1` callback.\n\nThe following worker defines a `c:backoff/1` function that delays retries using a variant of the\nhistoric Resque/Sidekiq algorithm:\n\n    defmodule MyApp.SidekiqBackoffWorker do\n      use Oban.Worker\n\n      @impl Worker\n      def backoff(%Job{attempt: attempt}) do\n        trunc(:math.pow(attempt, 4) + 15 + :rand.uniform(30) * attempt)\n      end\n\n      @impl Worker\n      def perform(_job) do\n        :do_business\n      end\n    end\n\nHere are some alternative backoff strategies to consider:\n\n* **constant** — delay by a fixed number of seconds, e.g. 1→15, 2→15, 3→15\n* **linear** — delay for the same number of seconds as the current attempt, e.g. 1→1, 2→2, 3→3\n* **squared** — delay by attempt number squared, e.g. 1→1, 2→4, 3→9\n* **sidekiq** — delay by a base amount plus some jitter, e.g. 1→32, 2→61, 3→135","ref":"Oban.Worker.html#module-customizing-backoff"},{"type":"behaviour","title":"Contextual Backoff - Oban.Worker","doc":"Any error, catch or throw is temporarily recorded in the job's `unsaved_error` map. The unsaved\nerror map can be used by `c:backoff/1` to calculate a custom backoff based on the exact error\nthat failed the job. In this example the `c:backoff/1` callback checks to see if the error was\ndue to rate limiting and adjusts the backoff accordingly:\n\n    defmodule MyApp.ApiWorker do\n      use Oban.Worker\n\n      @five_minutes 5 * 60\n\n      @impl Worker\n      def perform(%{args: args}) do\n        MyApp.make_external_api_call(args)\n      end\n\n      @impl Worker\n      def backoff(%Job{attempt: attempt, unsaved_error: unsaved_error}) do\n        %{kind: _, reason: reason, stacktrace: _} = unsaved_error\n\n        case reason do\n          %MyApp.ApiError{status: 429} -> @five_minutes\n          _ -> trunc(:math.pow(attempt, 4))\n        end\n      end\n    end","ref":"Oban.Worker.html#module-contextual-backoff"},{"type":"behaviour","title":"Execution Timeout - Oban.Worker","doc":"By default, individual jobs may execute indefinitely. If this is undesirable you may define a\ntimeout in milliseconds with the `c:timeout/1` callback on your worker module.\n\nFor example, to limit a worker's execution time to 30 seconds:\n\n    def MyApp.Worker do\n      use Oban.Worker\n\n      @impl Oban.Worker\n      def perform(_job) do\n        something_that_may_take_a_long_time()\n\n        :ok\n      end\n\n      @impl Oban.Worker\n      def timeout(_job), do: :timer.seconds(30)\n    end\n\nThe `c:timeout/1` function accepts an `Oban.Job` struct, so you can customize the timeout using\nany job attributes.\n\nFor example, you can define the `timeout` value through job args:\n\n    def timeout(%_{args: %{\"timeout\" => timeout}}), do: timeout\n\nOr you can define the `timeout` based on the number of attempts:\n\n    def timeout(%_{attempt: attempt}), do: attempt * :timer.seconds(5)\n\n\nIf the job fails to execute before the timeout period then it will error with a dedicated\n`Oban.TimeoutError` exception. Timeouts are treated like any other failure and the job will be\nretried as usual if more attempts are available.","ref":"Oban.Worker.html#module-execution-timeout"},{"type":"behaviour","title":"Snoozing Jobs - Oban.Worker","doc":"When returning `{:snooze, snooze_time}` in `c:perform/1`, the job is postponed for at least\n`snooze_time` seconds. Snoozing is done by incrementing the job's `max_attempts` field and\nscheduling execution for `snooze_time` seconds in the future.\n\nExecuting bumps a job's `attempt` count. Despite snooze incrementing the `max_attempts` to\npreserve total retries, the change to `attempt` will affect the default backoff retry\nalgorithm.\n\n> #### 🌟 Snoozes and Attempts {: .info}\n>\n> Oban Pro's [Smart Engine](https://oban.pro/docs/pro/Oban.Pro.Engines.Smart.html) rolls back\n> the `attempt` and preserves the original `max_attempts` in order to differentiate between\n> \"real\" attempts and snoozes, which keeps backoff calculation accurate.\n>\n> Without attempt correction you may need a solution that compensates for snoozing, such as the\n> example below:\n\n    defmodule MyApp.SnoozingWorker do\n      @max_attempts 20\n\n      use Oban.Worker, max_attempts: @max_attempts\n\n      @impl Worker\n      def backoff(%Job{} = job) do\n        corrected_attempt = @max_attempts - (job.max_attempts - job.attempt)\n\n        Worker.backoff(%{job | attempt: corrected_attempt})\n      end\n\n      @impl Worker\n      def perform(job) do\n        if MyApp.something?(job), do: :ok, else: {:snooze, 60}\n      end\n    end","ref":"Oban.Worker.html#module-snoozing-jobs"},{"type":"behaviour","title":"Workers in A Different Application - Oban.Worker","doc":"Occasionally, you may need to insert a job for a worker that exists in another\napplication. In that case you can use `Oban.Job.new/2` to build the changeset\nmanually:\n\n    %{id: 1, user_id: 2}\n    |> Oban.Job.new(queue: :default, worker: OtherApp.Worker)\n    |> Oban.insert()","ref":"Oban.Worker.html#module-workers-in-a-different-application"},{"type":"behaviour","title":"Prioritizing Jobs - Oban.Worker","doc":"Normally, all available jobs within a queue are executed in the order they were scheduled.\nYou can override the normal behavior and prioritize or de-prioritize a job by\nassigning a numerical `:priority`.\n\n  * Priorities from `0` to `9` are allowed, where `0` is the *highest* priority and `9`\n    is the *lowest*.\n  * The default priority is `0`; unless specified, all jobs have an equally high priority.\n  * All jobs with a higher priority will execute before any jobs with a lower priority.\n    Jobs with the *same priority* are executed in their scheduled order.","ref":"Oban.Worker.html#module-prioritizing-jobs"},{"type":"behaviour","title":"Caveats & Guidelines - Oban.Worker","doc":"The default priority is defined in the `jobs` table. The least intrusive way to change\nit for all jobs is to change the column default through a migration:\n\n    alter table(\"oban_jobs\") do\n      modify :priority, :integer, default: 1, from: {:integer, default: 0}\n    end","ref":"Oban.Worker.html#module-caveats-guidelines"},{"type":"callback","title":"Oban.Worker.backoff/1","doc":"Calculate the execution backoff.\n\nIn this context backoff specifies the number of seconds to wait before retrying a failed job.\n\nDefaults to an exponential algorithm with a minimum delay of 15 seconds and a small amount of\njitter.","ref":"Oban.Worker.html#c:backoff/1"},{"type":"function","title":"Oban.Worker.from_string/1","doc":"Resolve a module from a worker string.","ref":"Oban.Worker.html#from_string/1"},{"type":"function","title":"Examples - Oban.Worker.from_string/1","doc":"iex> Oban.Worker.from_string(\"Oban.Integration.Worker\")\n    {:ok, Oban.Integration.Worker}\n\n    iex> defmodule NotAWorker, do: []\n    ...> Oban.Worker.from_string(\"NotAWorker\")\n    {:error, %RuntimeError{message: \"module is not a worker: NotAWorker\"}}\n\n    iex> Oban.Worker.from_string(\"RandomWorker\")\n    {:error, %RuntimeError{message: \"unknown worker: RandomWorker\"}}","ref":"Oban.Worker.html#from_string/1-examples"},{"type":"callback","title":"Oban.Worker.new/2","doc":"Build a job changeset for this worker with optional overrides.\n\nSee `Oban.Job.new/2` for the available options.","ref":"Oban.Worker.html#c:new/2"},{"type":"callback","title":"Oban.Worker.perform/1","doc":"The `c:perform/1` function is called to execute a job.\n\nEach `c:perform/1` function should return `:ok` or a success tuple. When the return is an error\ntuple, an uncaught exception or a throw then the error is recorded and the job may be retried if\nthere are any attempts remaining.\n\n> #### `args` Are Stored as JSON {: .info}\n>\n> The `args` map provided to `perform/1` will _always_ have string keys, regardless of\n> the key type when the job was enqueued. The `args` are stored as `jsonb` in PostgreSQL and the\n> serialization process automatically stringifies all keys. Because `args` are always encoded\n> as JSON, you must also ensure that all values are serializable, otherwise you'll have\n> encoding errors when inserting jobs.","ref":"Oban.Worker.html#c:perform/1"},{"type":"type","title":"Oban.Worker.result/0","doc":"Return values control whether a job is treated as a success or a failure.\n\n- `:ok` - the job is successful and marked as `completed`.\n- `{:ok, ignored}` - the job is successful, marked as `completed`, and the return value is ignored.\n- `{:cancel, reason}` - the job is marked as `cancelled` for the provided reason and no longer retried.\n- `{:error, reason}` - the job is marked as `retryable` for the provided reason, or `discarded`\n  if it has  exhausted all attempts.\n- `{:snooze, seconds}` - mark the job as `scheduled` to run again `seconds` in the future.\n\n> #### Deprecated {: .warning}\n>\n> - `:discard` - deprecated, use `{:cancel, reason}` instead.\n> - `{:discard, reason}` - deprecated, use `{:cancel, reason}` instead.","ref":"Oban.Worker.html#t:result/0"},{"type":"type","title":"Oban.Worker.t/0","doc":"","ref":"Oban.Worker.html#t:t/0"},{"type":"callback","title":"Oban.Worker.timeout/1","doc":"Set a job's maximum execution time in milliseconds.\n\nJobs that exceed the time limit are considered a failure and may be retried.\n\nDefaults to `:infinity`.","ref":"Oban.Worker.html#c:timeout/1"},{"type":"function","title":"Oban.Worker.to_string/1","doc":"Return a string representation of a worker module.\n\nThis is particularly useful for normalizing worker names when building custom Ecto queries.","ref":"Oban.Worker.html#to_string/1"},{"type":"function","title":"Examples - Oban.Worker.to_string/1","doc":"iex> Oban.Worker.to_string(MyApp.SomeWorker)\n    \"MyApp.SomeWorker\"\n\n    iex> Oban.Worker.to_string(Elixir.MyApp.SomeWorker)\n    \"MyApp.SomeWorker\"\n\n    iex> Oban.Worker.to_string(\"Elixir.MyApp.SomeWorker\")\n    \"MyApp.SomeWorker\"","ref":"Oban.Worker.html#to_string/1-examples"},{"type":"behaviour","title":"Oban.Plugin","doc":"Defines a shared behaviour for Oban plugins.\n\nIn addition to implementing the Plugin behaviour, all plugins **must** be a `GenServer`, `Agent`, or\nanother OTP compliant module.","ref":"Oban.Plugin.html"},{"type":"behaviour","title":"Example - Oban.Plugin","doc":"Defining a basic plugin that satisfies the minimum behaviour:\n\n    defmodule MyPlugin do\n      @behaviour Oban.Plugin\n\n      use GenServer\n\n      @impl Oban.Plugin\n      def start_link(opts) do\n        GenServer.start_link(__MODULE__, opts, name: opts[:name])\n      end\n\n      @impl Oban.Plugin\n      def validate(opts) do\n        if is_atom(opts[:mode])\n          :ok\n        else\n          {:error, \"expected opts to have a :mode key\"}\n        end\n      end\n\n      @impl GenServer\n      def init(opts) do\n        case validate(opts) do\n          :ok -> {:ok, opts}\n          {:error, reason} -> {:stop, reason}\n        end\n      end\n    end","ref":"Oban.Plugin.html#module-example"},{"type":"callback","title":"Oban.Plugin.format_logger_output/2","doc":"Format telemetry event meta emitted by the for inclusion in the default logger.","ref":"Oban.Plugin.html#c:format_logger_output/2"},{"type":"type","title":"Oban.Plugin.option/0","doc":"","ref":"Oban.Plugin.html#t:option/0"},{"type":"callback","title":"Oban.Plugin.start_link/1","doc":"Starts a Plugin process linked to the current process.\n\nPlugins are typically started as part of an Oban supervision tree and will receive the current\nconfiguration as `:conf`, along with a `:name` and any other provided options.","ref":"Oban.Plugin.html#c:start_link/1"},{"type":"callback","title":"Oban.Plugin.validate/1","doc":"Validate the structure, presence, or values of keyword options.","ref":"Oban.Plugin.html#c:validate/1"},{"type":"module","title":"Oban.Plugins.Cron","doc":"Periodically enqueue jobs through [cron](https://en.wikipedia.org/wiki/Cron)-based scheduling.\n\nThis plugin registers workers a cron-like schedule and enqueues jobs automatically. To know\nmore about periodic jobs in Oban, see the [*Periodic Jobs* guide][perjob].\n\n> #### 🌟 DynamicCron {: .info}\n>\n> This plugin only loads the crontab statically, at boot time. To configure cron scheduling at\n> runtime, globally, across an entire cluster with scheduling guarantees and timezone overrides,\n> see the [`DynamicCron` plugin in Oban Pro](https://oban.pro/docs/pro/Oban.Pro.Plugins.DynamicCron.html).","ref":"Oban.Plugins.Cron.html"},{"type":"module","title":"Usage - Oban.Plugins.Cron","doc":"Periodic jobs are declared as a list of `{cron, worker}` or `{cron, worker, options}` tuples:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    {Oban.Plugins.Cron,\n     crontab: [\n       {\"* * * * *\", MyApp.MinuteWorker},\n       {\"0 * * * *\", MyApp.HourlyWorker, args: %{custom: \"arg\"}},\n       {\"0 0 * * *\", MyApp.DailyWorker, max_attempts: 1},\n       {\"0 12 * * MON\", MyApp.MondayWorker, queue: :scheduled, tags: [\"mondays\"]},\n       {\"@daily\", MyApp.AnotherDailyWorker}\n     ]}\n  ]\n```","ref":"Oban.Plugins.Cron.html#module-usage"},{"type":"module","title":"Identifying Cron Jobs - Oban.Plugins.Cron","doc":"Jobs inserted by the cron plugin are marked with a `cron` flag and the _original_ expression is\nstored as `cron_expr` in the job's `meta` field. For example, the meta for a `@daily` cron job\nwould look like this:\n\n```elixir\n%Oban.Job{meta: %{\"cron\" => true, \"cron_expr\" => \"@daily\", \"cron_tz\" => \"Etc/UTC\"}}\n```","ref":"Oban.Plugins.Cron.html#module-identifying-cron-jobs"},{"type":"module","title":"Options - Oban.Plugins.Cron","doc":"You can pass the following options to this plugin:\n\n* `:crontab` — a list of cron expressions that enqueue jobs on a periodic basis. See [*Periodic\n  Jobs* guide][perjob] for syntax and details.\n\n* `:timezone` — which timezone to use when scheduling cron jobs. To use a timezone other than\n  the default of `Etc/UTC` you *must* have a timezone database like [tz][tz] installed and\n  configured.","ref":"Oban.Plugins.Cron.html#module-options"},{"type":"module","title":"Instrumenting with Telemetry - Oban.Plugins.Cron","doc":"The `Oban.Plugins.Cron` plugin adds the following metadata to the `[:oban, :plugin, :stop]`\nevent (see `Oban.Telemetry`):\n\n* `:jobs` — a list of jobs that were inserted into the database\n\n[tz]: https://hexdocs.pm/tz\n[perjob]: periodic_jobs.html","ref":"Oban.Plugins.Cron.html#module-instrumenting-with-telemetry"},{"type":"type","title":"Oban.Plugins.Cron.cron_input/0","doc":"","ref":"Oban.Plugins.Cron.html#t:cron_input/0"},{"type":"opaque","title":"Oban.Plugins.Cron.expression/0","doc":"","ref":"Oban.Plugins.Cron.html#t:expression/0"},{"type":"type","title":"Oban.Plugins.Cron.option/0","doc":"","ref":"Oban.Plugins.Cron.html#t:option/0"},{"type":"function","title":"Oban.Plugins.Cron.parse/1","doc":"Parse a crontab expression into a cron struct.\n\nThis is provided as a convenience for validating and testing cron expressions. As such, the cron\nstruct itself is opaque and the internals may change at any time.\n\nThe parser can handle common expressions that use minutes, hours, days, months and weekdays,\nalong with ranges and steps. It also supports common extensions, also called nicknames.\n\nReturns `{:error, %ArgumentError{}}` with a detailed error if the expression cannot be parsed.","ref":"Oban.Plugins.Cron.html#parse/1"},{"type":"function","title":"Nicknames - Oban.Plugins.Cron.parse/1","doc":"The following special nicknames are supported in addition to standard cron expressions:\n\n  * `@yearly`—Run once a year, \"0 0 1 1 *\"\n  * `@annually`—Same as `@yearly`\n  * `@monthly`—Run once a month, \"0 0 1 * *\"\n  * `@weekly`—Run once a week, \"0 0 * * 0\"\n  * `@daily`—Run once a day, \"0 0 * * *\"\n  * `@midnight`—Same as `@daily`\n  * `@hourly`—Run once an hour, \"0 * * * *\"\n  * `@reboot`—Run once at boot","ref":"Oban.Plugins.Cron.html#parse/1-nicknames"},{"type":"function","title":"Examples - Oban.Plugins.Cron.parse/1","doc":"iex> Oban.Plugins.Cron.parse(\"@hourly\")\n    {:ok, #Oban.Cron.Expression<...>}\n\n    iex> Oban.Plugins.Cron.parse(\"0 * * * *\")\n    {:ok, #Oban.Cron.Expression<...>}\n\n    iex> Oban.Plugins.Cron.parse(\"60 * * * *\")\n    {:error, %ArgumentError{message: \"expression field 60 is out of range 0..59\"}}","ref":"Oban.Plugins.Cron.html#parse/1-examples"},{"type":"module","title":"Oban.Plugins.Lifeline","doc":"Naively transition jobs stuck `executing` back to `available`.\n\nThe `Lifeline` plugin periodically rescues orphaned jobs, i.e. jobs that are stuck in the\n`executing` state because the node was shut down before the job could finish. Rescuing is\npurely based on time, rather than any heuristic about the job's expected execution time or\nwhether the node is still alive.\n\nIf an executing job has exhausted all attempts, the Lifeline plugin will mark it `discarded`\nrather than `available`.\n\n> #### 🌟 DynamicLifeline {: .info}\n>\n> This plugin may transition jobs that are genuinely `executing` and cause duplicate execution.\n> For more accurate rescuing or to rescue jobs that have exhausted retry attempts see the\n> `DynamicLifeline` plugin in [Oban Pro](https://oban.pro/docs/pro/Oban.Pro.Plugins.DynamicLifeline.html).","ref":"Oban.Plugins.Lifeline.html"},{"type":"module","title":"Using the Plugin - Oban.Plugins.Lifeline","doc":"Rescue orphaned jobs that are still `executing` after the default of 60 minutes:\n\n    config :my_app, Oban,\n      plugins: [Oban.Plugins.Lifeline],\n      ...\n\nOverride the default period to rescue orphans after a more aggressive period of 5 minutes:\n\n    config :my_app, Oban,\n      plugins: [{Oban.Plugins.Lifeline, rescue_after: :timer.minutes(5)}],\n      ...","ref":"Oban.Plugins.Lifeline.html#module-using-the-plugin"},{"type":"module","title":"Options - Oban.Plugins.Lifeline","doc":"* `:interval` — the number of milliseconds between rescue attempts. The default is `60_000ms`.\n\n* `:rescue_after` — the maximum amount of time, in milliseconds, that a job may execute before\nbeing rescued. 60 minutes by default, and rescuing is performed once a minute.","ref":"Oban.Plugins.Lifeline.html#module-options"},{"type":"module","title":"Instrumenting with Telemetry - Oban.Plugins.Lifeline","doc":"The `Oban.Plugins.Lifeline` plugin adds the following metadata to the `[:oban, :plugin, :stop]`\nevent:\n\n* `:rescued_jobs` — a list of jobs transitioned back to `available`\n\n* `:discarded_jobs` — a list of jobs transitioned to `discarded`\n\n_Note: jobs only include `id`, `queue`, `state` fields._","ref":"Oban.Plugins.Lifeline.html#module-instrumenting-with-telemetry"},{"type":"type","title":"Oban.Plugins.Lifeline.option/0","doc":"","ref":"Oban.Plugins.Lifeline.html#t:option/0"},{"type":"module","title":"Oban.Plugins.Pruner","doc":"Periodically delete `completed`, `cancelled`, and `discarded` jobs based on their age.\n\nPruning is critical for maintaining table size and continued responsive job processing. It\nis recommended for all production applications. See also the\n[*Operational Maintenance* guide](operational_maintenance.html).\n\n> #### 🌟 DynamicPruner {: .info}\n>\n> This plugin is limited to a fixed interval and a single `max_age` check for all jobs. To prune\n> on a cron-style schedule, retain jobs by a limit or age, or provide overrides for specific\n> queues, workers, and job states; see Oban Pro's\n> [DynamicPruner](https://oban.pro/docs/pro/Oban.Pro.Plugins.DynamicPruner.html).","ref":"Oban.Plugins.Pruner.html"},{"type":"module","title":"Using the Plugin - Oban.Plugins.Pruner","doc":"The following example demonstrates using the plugin without any configuration, which will prune\njobs older than the default of 60 seconds:\n\n    config :my_app, Oban,\n      plugins: [Oban.Plugins.Pruner],\n      ...\n\nOverride the default options to prune jobs after 5 minutes:\n\n    config :my_app, Oban,\n      plugins: [{Oban.Plugins.Pruner, max_age: 300}],\n      ...","ref":"Oban.Plugins.Pruner.html#module-using-the-plugin"},{"type":"module","title":"Options - Oban.Plugins.Pruner","doc":"* `:interval` — the number of milliseconds between pruning attempts. The default is `30_000ms`.\n\n* `:limit` — the maximum number of jobs to prune at one time. The default is 10,000 to prevent\n  request timeouts. Applications that steadily generate more than 10k jobs a minute should\n  increase this value.\n\n* `:max_age` — the number of seconds after which a job may be pruned. Defaults to 60s.","ref":"Oban.Plugins.Pruner.html#module-options"},{"type":"module","title":"Instrumenting with Telemetry - Oban.Plugins.Pruner","doc":"The `Oban.Plugins.Pruner` plugin adds the following metadata to the `[:oban, :plugin, :stop]` event:\n\n* `:pruned_jobs` - the jobs that were deleted from the database\n\n_Note: jobs only include `id`, `queue`, `state` fields._","ref":"Oban.Plugins.Pruner.html#module-instrumenting-with-telemetry"},{"type":"type","title":"Oban.Plugins.Pruner.option/0","doc":"","ref":"Oban.Plugins.Pruner.html#t:option/0"},{"type":"module","title":"Oban.Plugins.Reindexer","doc":"Periodically rebuild indexes to minimize database bloat.\n\nOver time various Oban indexes may grow without `VACUUM` cleaning them up properly. When this\nhappens, rebuilding the indexes will release bloat.\n\nThe plugin uses `REINDEX` with the `CONCURRENTLY` option to rebuild without taking any locks\nthat prevent concurrent inserts, updates, or deletes on the table.\n\nNote: This plugin requires the `CONCURRENTLY` option, which is only available in Postgres 12 and\nabove.","ref":"Oban.Plugins.Reindexer.html"},{"type":"module","title":"Using the Plugin - Oban.Plugins.Reindexer","doc":"By default, the plugin will reindex once a day, at midnight UTC:\n\n    config :my_app, Oban,\n      plugins: [Oban.Plugins.Reindexer],\n      ...\n\nTo run on a different schedule you can provide a cron expression. For example, you could use the\n`\"@weekly\"` shorthand to run once a week on Sunday:\n\n    config :my_app, Oban,\n      plugins: [{Oban.Plugins.Reindexer, schedule: \"@weekly\"}],\n      ...","ref":"Oban.Plugins.Reindexer.html#module-using-the-plugin"},{"type":"module","title":"Options - Oban.Plugins.Reindexer","doc":"* `:indexes` — a list of indexes to reindex on the `oban_jobs` table. Defaults to only the\n    `oban_jobs_args_index` and `oban_jobs_meta_index`.\n\n  * `:schedule` — a cron expression that controls when to reindex. Defaults to `\"@midnight\"`.\n\n  * `:timeout` - time in milliseconds to wait for each query call to finish. Defaults to 15 seconds.\n\n  * `:timezone` — which timezone to use when evaluating the schedule. To use a timezone other than\n    the default of \"Etc/UTC\" you *must* have a timezone database like [tz][tz] installed and\n    configured.\n\n[tz]: https://hexdocs.pm/tz","ref":"Oban.Plugins.Reindexer.html#module-options"},{"type":"type","title":"Oban.Plugins.Reindexer.option/0","doc":"","ref":"Oban.Plugins.Reindexer.html#t:option/0"},{"type":"module","title":"Oban.Config","doc":"The Config struct validates and encapsulates Oban instance state.\n\nTypically, you won't use the Config module directly. Oban automatically creates a Config struct\non initialization and passes it through to all supervised children with the `:conf` key.\n\nTo fetch a running Oban supervisor's config, see `Oban.config/1`.","ref":"Oban.Config.html"},{"type":"function","title":"Oban.Config.new/1","doc":"Generate a Config struct after normalizing and verifying Oban options.\n\nSee `Oban.start_link/1` for a comprehensive description of available options.","ref":"Oban.Config.html#new/1"},{"type":"function","title":"Example - Oban.Config.new/1","doc":"Generate a minimal config with only a `:repo`:\n\n    Oban.Config.new(repo: Oban.Test.Repo)","ref":"Oban.Config.html#new/1-example"},{"type":"type","title":"Oban.Config.t/0","doc":"","ref":"Oban.Config.html#t:t/0"},{"type":"function","title":"Oban.Config.validate/1","doc":"Verify configuration options.\n\nThis helper is used by `new/1`, and therefore by `Oban.start_link/1`, to verify configuration\noptions when an Oban supervisor starts. It is provided publicly to aid in configuration testing,\nas `test` config may differ from `prod` config.\n\n# Example\n\nValidating top level options:\n\n    iex> Oban.Config.validate(name: Oban)\n    :ok\n\n    iex> Oban.Config.validate(name: Oban, log: false)\n    :ok\n\n    iex> Oban.Config.validate(node: {:not, :binary})\n    {:error, \"expected :node to be a binary, got: {:not, :binary}\"}\n\n    iex> Oban.Config.validate(plugins: true)\n    {:error, \"invalid value for :plugins, expected :plugins to be a list, got: true\"}\n\nValidating plugin options:\n\n    iex> Oban.Config.validate(plugins: [{Oban.Plugins.Pruner, max_age: 60}])\n    :ok\n\n    iex> Oban.Config.validate(plugins: [{Oban.Plugins.Pruner, max_age: 0}])\n    {:error, \"invalid value for :plugins, expected :max_age to be a positive integer, got: 0\"}","ref":"Oban.Config.html#validate/1"},{"type":"module","title":"Oban.Registry","doc":"Local process storage for Oban instances.","ref":"Oban.Registry.html"},{"type":"function","title":"Oban.Registry.config/1","doc":"Fetch the config for an Oban supervisor instance.","ref":"Oban.Registry.html#config/1"},{"type":"function","title":"Example - Oban.Registry.config/1","doc":"Get the default instance config:\n\n    Oban.Registry.config(Oban)\n\nGet config for a custom named instance:\n\n    Oban.Registry.config(MyApp.Oban)","ref":"Oban.Registry.html#config/1-example"},{"type":"type","title":"Oban.Registry.key/0","doc":"","ref":"Oban.Registry.html#t:key/0"},{"type":"function","title":"Oban.Registry.lookup/2","doc":"Find the `{pid, value}` pair for a registered Oban process.","ref":"Oban.Registry.html#lookup/2"},{"type":"function","title":"Example - Oban.Registry.lookup/2","doc":"Get the default instance config:\n\n    Oban.Registry.lookup(Oban)\n\nGet a supervised module's pid:\n\n    Oban.Registry.lookup(Oban, Oban.Notifier)","ref":"Oban.Registry.html#lookup/2-example"},{"type":"type","title":"Oban.Registry.role/0","doc":"","ref":"Oban.Registry.html#t:role/0"},{"type":"function","title":"Oban.Registry.select/1","doc":"Select details of registered Oban processes using a full match spec.","ref":"Oban.Registry.html#select/1"},{"type":"function","title":"Example - Oban.Registry.select/1","doc":"Get a list of all running Oban instances:\n\n    Oban.Registry.select([{{:\"$1\", :_, :_}, [{:is_atom, :\"$1\"}], [:\"$1\"]}])","ref":"Oban.Registry.html#select/1-example"},{"type":"type","title":"Oban.Registry.value/0","doc":"","ref":"Oban.Registry.html#t:value/0"},{"type":"function","title":"Oban.Registry.via/3","doc":"Build a via tuple suitable for calls to a supervised Oban process.","ref":"Oban.Registry.html#via/3"},{"type":"function","title":"Example - Oban.Registry.via/3","doc":"For an Oban supervisor:\n\n    Oban.Registry.via(Oban)\n\nFor a supervised module:\n\n    Oban.Registry.via(Oban, Oban.Notifier)\n\nFor a plugin:\n\n    Oban.Registry.via(Oban, {:plugin, Oban.Plugins.Cron})","ref":"Oban.Registry.html#via/3-example"},{"type":"function","title":"Oban.Registry.whereis/2","doc":"Returns the pid of a supervised Oban process, or `nil` if the process can't be found.","ref":"Oban.Registry.html#whereis/2"},{"type":"function","title":"Example - Oban.Registry.whereis/2","doc":"Get the Oban supervisor's pid:\n\n    Oban.Registry.whereis(Oban)\n\nGet a supervised module's pid:\n\n    Oban.Registry.whereis(Oban, Oban.Notifier)\n\nGet the pid for a plugin:\n\n    Oban.Registry.whereis(Oban, {:plugin, MyApp.Oban.Plugin})\n\nGet the pid for a queue's producer:\n\n    Oban.Registry.whereis(Oban, {:producer, \"default\"})","ref":"Oban.Registry.html#whereis/2-example"},{"type":"module","title":"Oban.Repo","doc":"Wrappers around `Ecto.Repo` and `Ecto.Adapters.SQL` callbacks.\n\nEach function resolves the correct repo instance and sets options such as `prefix` and `log`\naccording to `Oban.Config`.\n\n> #### Meant for Extending Oban {: .warning}\n>\n> These functions should only be used when working with a repo inside engines, plugins, or other\n> extensions for Oban. Favor using your application's repo directly when querying `Oban.Job`\n> from your workers.","ref":"Oban.Repo.html"},{"type":"module","title":"Examples - Oban.Repo","doc":"The first argument for every function must be an `Oban.Config` struct. Many functions pass\nconfiguration around as a `conf` key, and it can always be fetched with `Oban.config/1`. This\ndemonstrates fetching the default instance config and querying all jobs:\n\n    Oban\n    |> Oban.config()\n    |> Oban.Repo.all(Oban.Job)","ref":"Oban.Repo.html#module-examples"},{"type":"function","title":"Oban.Repo.aggregate/4","doc":"Wraps `c:Ecto.Repo.aggregate/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#aggregate/4"},{"type":"function","title":"Oban.Repo.all/3","doc":"Wraps `c:Ecto.Repo.all/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#all/3"},{"type":"function","title":"Oban.Repo.checkout/3","doc":"Wraps `c:Ecto.Repo.checkout/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#checkout/3"},{"type":"function","title":"Oban.Repo.config/1","doc":"Wraps `c:Ecto.Repo.config/0` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#config/1"},{"type":"function","title":"Oban.Repo.default_options/1","doc":"The default values extracted from `Oban.Config` for use in all queries with options.","ref":"Oban.Repo.html#default_options/1"},{"type":"function","title":"Oban.Repo.default_options/2","doc":"Wraps `c:Ecto.Repo.default_options/1` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#default_options/2"},{"type":"function","title":"Oban.Repo.delete/3","doc":"Wraps `c:Ecto.Repo.delete/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#delete/3"},{"type":"function","title":"Oban.Repo.delete!/3","doc":"Wraps `c:Ecto.Repo.delete!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#delete!/3"},{"type":"function","title":"Oban.Repo.delete_all/3","doc":"Wraps `c:Ecto.Repo.delete_all/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#delete_all/3"},{"type":"function","title":"Oban.Repo.exists?/3","doc":"Wraps `c:Ecto.Repo.exists?/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#exists?/3"},{"type":"function","title":"Oban.Repo.get/4","doc":"Wraps `c:Ecto.Repo.get/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get/4"},{"type":"function","title":"Oban.Repo.get!/4","doc":"Wraps `c:Ecto.Repo.get!/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get!/4"},{"type":"function","title":"Oban.Repo.get_by/4","doc":"Wraps `c:Ecto.Repo.get_by/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get_by/4"},{"type":"function","title":"Oban.Repo.get_by!/4","doc":"Wraps `c:Ecto.Repo.get_by!/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get_by!/4"},{"type":"function","title":"Oban.Repo.get_dynamic_repo/1","doc":"Wraps `c:Ecto.Repo.get_dynamic_repo/0` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get_dynamic_repo/1"},{"type":"function","title":"Oban.Repo.in_transaction?/1","doc":"Wraps `c:Ecto.Repo.in_transaction?/0` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#in_transaction?/1"},{"type":"function","title":"Oban.Repo.insert/3","doc":"Wraps `c:Ecto.Repo.insert/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert/3"},{"type":"function","title":"Oban.Repo.insert!/3","doc":"Wraps `c:Ecto.Repo.insert!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert!/3"},{"type":"function","title":"Oban.Repo.insert_all/4","doc":"Wraps `c:Ecto.Repo.insert_all/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert_all/4"},{"type":"function","title":"Oban.Repo.insert_or_update/3","doc":"Wraps `c:Ecto.Repo.insert_or_update/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert_or_update/3"},{"type":"function","title":"Oban.Repo.insert_or_update!/3","doc":"Wraps `c:Ecto.Repo.insert_or_update!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert_or_update!/3"},{"type":"function","title":"Oban.Repo.load/3","doc":"Wraps `c:Ecto.Repo.load/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#load/3"},{"type":"function","title":"Oban.Repo.one/3","doc":"Wraps `c:Ecto.Repo.one/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#one/3"},{"type":"function","title":"Oban.Repo.one!/3","doc":"Wraps `c:Ecto.Repo.one!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#one!/3"},{"type":"function","title":"Oban.Repo.preload/4","doc":"Wraps `c:Ecto.Repo.preload/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#preload/4"},{"type":"function","title":"Oban.Repo.put_dynamic_repo/2","doc":"Wraps `c:Ecto.Repo.put_dynamic_repo/1` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#put_dynamic_repo/2"},{"type":"function","title":"Oban.Repo.query/4","doc":"Wraps `Ecto.Adapters.SQL.Repo.query/4` with an added `Oban.Config` argument.","ref":"Oban.Repo.html#query/4"},{"type":"function","title":"Oban.Repo.query!/4","doc":"Wraps `Ecto.Adapters.SQL.Repo.query!/4` with an added `Oban.Config` argument.","ref":"Oban.Repo.html#query!/4"},{"type":"function","title":"Oban.Repo.reload/3","doc":"Wraps `c:Ecto.Repo.reload/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#reload/3"},{"type":"function","title":"Oban.Repo.reload!/3","doc":"Wraps `c:Ecto.Repo.reload!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#reload!/3"},{"type":"function","title":"Oban.Repo.rollback/2","doc":"Wraps `c:Ecto.Repo.rollback/1` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#rollback/2"},{"type":"function","title":"Oban.Repo.stream/3","doc":"Wraps `c:Ecto.Repo.stream/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#stream/3"},{"type":"function","title":"Oban.Repo.to_sql/3","doc":"Wraps `Ecto.Adapters.SQL.Repo.to_sql/2` with an added `Oban.Config` argument.","ref":"Oban.Repo.html#to_sql/3"},{"type":"function","title":"Oban.Repo.transaction/3","doc":"Wraps `c:Ecto.Repo.transaction/2` with an additional `Oban.Config` argument and automatic\nretries with backoff.","ref":"Oban.Repo.html#transaction/3"},{"type":"function","title":"Options - Oban.Repo.transaction/3","doc":"Backoff helpers, in addition to the standard transaction options:\n\n* `delay` — the time to sleep between retries, defaults to `500ms`\n* `retry` — the number of retries for unexpected errors, defaults to `5`\n* `expected_delay` — the time to sleep between expected errors, e.g. `serialization` or\n  `lock_not_available`, defaults to `10ms`\n* `expected_retry` — the number of retries for expected errors, defaults to `20`","ref":"Oban.Repo.html#transaction/3-options"},{"type":"function","title":"Oban.Repo.update/3","doc":"Wraps `c:Ecto.Repo.update/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#update/3"},{"type":"function","title":"Oban.Repo.update!/3","doc":"Wraps `c:Ecto.Repo.update!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#update!/3"},{"type":"function","title":"Oban.Repo.update_all/4","doc":"Wraps `c:Ecto.Repo.update_all/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#update_all/4"},{"type":"module","title":"Oban.Telemetry","doc":"Telemetry integration for event metrics, logging and error reporting.","ref":"Oban.Telemetry.html"},{"type":"module","title":"Initialization Events - Oban.Telemetry","doc":"Oban emits the following telemetry event when an Oban supervisor is started:\n\n* `[:oban, :supervisor, :init]` - when the Oban supervisor is started this will execute\n\nThe initialization event contains the following measurements:\n\n* `:system_time` - The system's time when Oban was started\n\nThe initialization event contains the following metadata:\n\n* `:conf` - The configuration used for the Oban supervisor instance\n* `:pid` - The PID of the supervisor instance","ref":"Oban.Telemetry.html#module-initialization-events"},{"type":"module","title":"Job Events - Oban.Telemetry","doc":"Oban emits the following telemetry events for each job:\n\n* `[:oban, :job, :start]` — at the point a job is fetched from the database and will execute\n* `[:oban, :job, :stop]` — after a job succeeds and the success is recorded in the database\n* `[:oban, :job, :exception]` — after a job fails and the failure is recorded in the database\n\nAll job events share the same details about the job that was executed. In addition, failed jobs\nprovide the error type, the error itself, and the stacktrace. The following chart shows which\nmetadata you can expect for each event:\n\n| event        | measures                                             | metadata                                                                |\n| ------------ | ---------------------------------------------------- | ----------------------------------------------------------------------- |\n| `:start`     | `:system_time`                                       | `:conf`, `:job`                                                         |\n| `:stop`      | `:duration`, `:memory`, `:queue_time`, `:reductions` | `:conf`, `:job`, `:state`, `:result`                                    |\n| `:exception` | `:duration`, `:memory`, `:queue_time`, `:reductions` | `:conf`, `:job`, `:state`, `:kind`, `:reason`, `:result`, `:stacktrace` |\n\n#### Metadata\n\n* `:conf` — the executing Oban instance's config\n* `:job` — the executing `Oban.Job`\n* `:result` — the `perform/1` return value, always `nil` for an exception or crash\n* `:state` — one of `:success`, `:failure`, `:cancelled`, `:discard` or `:snoozed`\n\nFor `:exception` events the metadata also includes details about what caused the failure.\n\n* `:kind` — describes how an error occurred. Here are the possible kinds:\n  - `:error` — from an `{:error, error}` return value. Some Erlang functions may also throw an\n    `:error` tuple, which will be reported as `:error`.\n  - `:exit` — from a caught process exit\n  - `:throw` — from a caught value, this doesn't necessarily mean that an error occurred and the\n    error value is unpredictable\n\n* `:reason` — a raised exception, wrapped crash, or wrapped error that caused the job to fail.\n  Raised exceptions are passes as is, crashes are wrapped in an `Oban.CrashError`, timeouts in\n  `Oban.TimeoutError`, and all other errors are normalized into an `Oban.PerformError`.\n\n* `:stacktrace` — the `t:Exception.stacktrace/0` for crashes or raised exceptions. Failures from \n  manual error returns won't contain any application code entries and may have an empty\n  stacktrace.","ref":"Oban.Telemetry.html#module-job-events"},{"type":"module","title":"Engine Events - Oban.Telemetry","doc":"Oban emits telemetry span events for the following Engine operations:\n\n* `[:oban, :engine, :init, :start | :stop | :exception]`\n* `[:oban, :engine, :refresh, :start | :stop | :exception]`\n* `[:oban, :engine, :put_meta, :start | :stop | :exception]`\n* `[:oban, :engine, :check_available, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                              |\n| ------------ | -------------- | ----------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:engine`                                    |\n| `:stop`      | `:duration`    | `:conf`, `:engine`                                    |\n| `:exception` | `:duration`    | `:conf`, `:engine`, `:kind`, `:reason`, `:stacktrace` |\n\nEvents for bulk operations also include `:jobs` for the `:stop` event:\n\n* `[:oban, :engine, :cancel_all_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :delete_all_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :fetch_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :insert_all_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :prune_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :rescue_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :retry_all_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :stage_jobs, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                              |\n| ------------ | -------------- | ----------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:engine`                                    |\n| `:stop`      | `:duration`    | `:conf`, `:engine`, `:jobs`                                    |\n| `:exception` | `:duration`    | `:conf`, `:engine`, `:kind`, `:reason`, `:stacktrace` |\n\nEvents for job-level Engine operations also include the `job`, with the exception of\n`:insert_job, :start`, because the `job` isn't available yet.\n\n* `[:oban, :engine, :cancel_job, :start | :stop | :exception]`\n* `[:oban, :engine, :complete_job, :start | :stop | :exception]`\n* `[:oban, :engine, :delete_job, :start | :stop | :exception]`\n* `[:oban, :engine, :discard_job, :start | :stop | :exception]`\n* `[:oban, :engine, :error_job, :start | :stop | :exception]`\n* `[:oban, :engine, :insert_job, :start | :stop | :exception]`\n* `[:oban, :engine, :retry_job, :start | :stop | :exception]`\n* `[:oban, :engine, :snooze_job, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                                      |\n| ------------ | -------------- | ------------------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:engine`, `:job`                                    |\n| `:stop`      | `:duration`    | `:conf`, `:engine`, `:job`                                    |\n| `:exception` | `:duration`    | `:conf`, `:engine`, `:job`, `:kind`, `:reason`, `:stacktrace` |\n\n#### Metadata\n\n* `:conf` — the Oban supervisor's config\n* `:engine` — the module of the engine used\n* `:job` - the `Oban.Job` in question\n* `:jobs` — zero or more maps with the `queue`, `state` for each modified job\n* `:kind`, `:reason`, `:stacktrace` — see the explanation in job metadata above","ref":"Oban.Telemetry.html#module-engine-events"},{"type":"module","title":"Notifier Events - Oban.Telemetry","doc":"Oban emits a telemetry span event each time the Notifier is triggered:\n\n* `[:oban, :notifier, :notify, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                                            |\n| ------------ | -------------- | ------------------------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:channel`, `:payload`                                     |\n| `:stop`      | `:duration`    | `:conf`, `:channel`, `:payload`                                     |\n| `:exception` | `:duration`    | `:conf`, `:channel`, `:payload`, `:kind`, `:reason`, `:stacktrace`  |\n\n#### Metadata\n\n* `:conf` — the Oban supervisor's config\n* `:channel` — the channel on which the notification was sent\n* `:payload` - the decoded payload that was sent\n* `:kind`, `:reason`, `:stacktrace` — see the explanation in job metadata above\n\n\nOban also emits an event when the notifier's sonar, responsible for tracking notifier\nconnectivity, switches connectivity status:\n\n* `[:oban, :notifier, :switch]`\n\n| event        | measures  | metadata           |\n| ------------ | --------- | ------------------ |\n| `:switch`    |           | `:conf`, `:status` |\n\n#### Metadata\n\n* `:conf` — see the explanation in metadata above\n* `:status` — one of `:isolated`, `:solitary`, or `:clustered`, see\n  `Oban.Notifier.status/1` for details","ref":"Oban.Telemetry.html#module-notifier-events"},{"type":"module","title":"Plugin Events - Oban.Telemetry","doc":"All the Oban plugins emit telemetry events under the `[:oban, :plugin, *]` pattern (where `*` is\neither `:init`, `:start`, `:stop`, or `:exception`). You can filter out for plugin events by\nlooking into the metadata of the event and checking the value of `:plugin`. The `:plugin` field\nis the plugin module that emitted the event. For example, to get `Oban.Plugins.Cron` specific\nevents, you can filter for telemetry events with a metadata key/value of `plugin:\nOban.Plugins.Cron`.\n\nOban emits the following telemetry event whenever a plugin executes (be sure to check the\ndocumentation for each plugin as each plugin can also add additional metadata specific to\nthe plugin):\n\n* `[:oban, :plugin, :init]` — when the plugin first initializes\n* `[:oban, :plugin, :start]` — when the plugin beings performing its work\n* `[:oban, :plugin, :stop]` —  after the plugin completes its work\n* `[:oban, :plugin, :exception]` — when the plugin encounters an error\n\nThe following chart shows which metadata you can expect for each event:\n\n| event        | measures        | metadata                                              |\n| ------------ | --------------- | ----------------------------------------------------- |\n| `:init`      |                 | `:conf`, `:plugin`                                    |\n| `:start`     | `:system_time`  | `:conf`, `:plugin`                                    |\n| `:stop`      | `:duration`     | `:conf`, `:plugin`                                    |\n| `:exception` | `:duration`     | `:conf`, `:plugin`, `:kind`, `:reason`, `:stacktrace` |","ref":"Oban.Telemetry.html#module-plugin-events"},{"type":"module","title":"Peer Events - Oban.Telemetry","doc":"Oban emits a telemetry span event each time an Oban Peer election occurs:\n\n* `[:oban, :peer, :election, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                                       |\n| ------------ | -------------- | -------------------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:leader`, `:peer`,                                   |\n| `:stop`      | `:duration`    | `:conf`, `:leader`, `:peer`, `:was_leader`                    |\n| `:exception` | `:duration`    | `:conf`, `:leader`, `:peer`, `:kind`, `:reason`, `:stacktrace` |\n\n#### Metadata\n\n* `:conf`, `:kind`, `:reason`, `:stacktrace` — see the explanation in notifier metadata above\n* `:leader` — whether the peer is the current leader\n* `:was_leader` — whether the peer was the leader before the election occurred\n* `:peer` — the module used for peering","ref":"Oban.Telemetry.html#module-peer-events"},{"type":"module","title":"Queue Events - Oban.Telemetry","doc":"Oban emits an event when a queue shuts down cleanly, e.g. without being brutally killed. Event\nemission isn't guaranteed because it is emitted as part of a `terminate/1` callback.\n\n* `[:oban, :queue, :shutdown]`\n\n| event       | measures   | metadata                       |\n| ----------- | -----------| ------------------------------ |\n| `:shutdown` | `:elapsed` | `:conf`, `:orphaned`, `:queue` |\n\n#### Metadata\n\n* `:conf` — see the explanation in metadata above\n* `:orphaned` — a list of job id's left in an `executing` state because they couldn't finish\n* `:queue` — the stringified queue name","ref":"Oban.Telemetry.html#module-queue-events"},{"type":"module","title":"Stager Events - Oban.Telemetry","doc":"Oban emits an event any time the Stager switches between `local` and `global` modes:\n\n* `[:oban, :stager, :switch]`\n\n| event        | measures  | metadata         |\n| ------------ | --------- | ---------------- |\n| `:switch`    |           | `:conf`, `:mode` |\n\n#### Metadata\n\n* `:conf` — see the explanation in metadata above\n* `:mode` — either `local` for polling mode or `global` in the more efficient pub-sub mode","ref":"Oban.Telemetry.html#module-stager-events"},{"type":"module","title":"Default Logger - Oban.Telemetry","doc":"A default log handler that emits structured JSON is provided, see `attach_default_logger/0` for\nusage. Otherwise, if you would prefer more control over logging or would like to instrument\nevents you can write your own handler.\n\nHere is an example of the JSON output for the `job:stop` event:\n\n```json\n{\n  \"args\":{\"action\":\"OK\",\"ref\":1},\n  \"attempt\":1,\n  \"duration\":4327295,\n  \"event\":\"job:stop\",\n  \"id\":123,\n  \"max_attempts\":20,\n  \"meta\":{},\n  \"queue\":\"alpha\",\n  \"queue_time\":3127905,\n  \"source\":\"oban\",\n  \"state\":\"success\",\n  \"tags\":[],\n  \"worker\":\"Oban.Integration.Worker\"\n}\n```\n\nAll timing measurements are recorded as native time units but logged in microseconds.","ref":"Oban.Telemetry.html#module-default-logger"},{"type":"module","title":"Examples - Oban.Telemetry","doc":"A handler that only logs a few details about failed jobs:\n\n```elixir\ndefmodule MicroLogger do\n  require Logger\n\n  def handle_event([:oban, :job, :exception], %{duration: duration}, meta, nil) do\n    Logger.warning(\"[#{meta.queue}] #{meta.worker} failed in #{duration}\")\n  end\nend\n\n:telemetry.attach(\"oban-logger\", [:oban, :job, :exception], &MicroLogger.handle_event/4, nil)\n```\n\nAnother great use of execution data is error reporting. Here is an example of integrating with\n[Honeybadger][honey], but only reporting jobs that have failed 3 times or more:\n\n```elixir\ndefmodule ErrorReporter do\n  def handle_event([:oban, :job, :exception], _, %{attempt: attempt} = meta, _) do\n    if attempt >= 3 do\n      context = Map.take(meta, [:id, :args, :queue, :worker])\n\n      Honeybadger.notify(meta.reason, metadata: context, stacktrace: meta.stacktrace)\n    end\n  end\nend\n\n:telemetry.attach(\"oban-errors\", [:oban, :job, :exception], &ErrorReporter.handle_event/4, [])\n```\n\n[honey]: https://honeybadger.io","ref":"Oban.Telemetry.html#module-examples"},{"type":"function","title":"Oban.Telemetry.attach_default_logger/1","doc":"Attaches a default structured JSON Telemetry handler for logging.\n\nThis function attaches a handler that outputs logs with `message` and `source` fields, along\nwith some event specific fields.\n\n#### Job Events\n\n* `args` — a map of the job's raw arguments\n* `attempt` — the job's execution atttempt\n* `duration` — the job's runtime duration in microseconds\n* `event` — `job:start`, `job:stop`, `job:exception` depending on reporting telemetry event\n* `error` — a formatted error banner, without the extended stacktrace\n* `id` — the job's id\n* `meta` — a map of the job's raw metadata\n* `queue` — the job's queue\n* `state` — the execution state, one of \"success\", \"failure\", \"cancelled\", \"discard\", or\n  \"snoozed\"\n* `system_time` — when the job started, in microseconds\n* `tags` — the job's tags\n* `worker` — the job's worker module\n\n#### Queue Events\n\n* `elapsed` — the amount of time the queue waited for shutdown, in milliseconds\n* `event` — always `queue:shutdown`\n* `orphaned` — a list of any job id's left in an `executing` state\n* `queue` — the queue name\n\n#### Notifier Events\n\n* `event` — always `notifier:switch`\n* `message` — information about the status switch\n* `status` — either `\"isolated\"`, `\"solitary\"`, or `\"clustered\"`\n\n#### Peer Events\n\n* `event` — always `peer:election`\n* `leader` — boolean indicating whether the peer is the leader\n* `message` — information about the peers role in an election\n* `node` — the name of the node that changed leadership\n* `was_leader` — boolean indicating whether the peer was leader before the election\n\n#### Plugin Events\n\n* `event` — `plugin:stop` or `plugin:exception`\n* `plugin` — the plugin module\n* `duration` — the runtime duration in microseconds\n\nOther values may be included depending on the plugin.\n\n#### Stager Events\n\n* `event` — always `stager:switch`\n* `message` — information about the mode switch\n* `mode` — either `\"local\"` or `\"global\"`","ref":"Oban.Telemetry.html#attach_default_logger/1"},{"type":"function","title":"Options - Oban.Telemetry.attach_default_logger/1","doc":"* `:encode` — Whether to encode log output as JSON rather than using structured logging,\n  defaults to `true`\n\n* `:events` — Which event categories to log, where the categories include `:job`, `:notifier`,\n  `:plugin`, `:peer`, `:queue`, `:stager`, or `:all`. Defaults to `:all`.\n\n* `:level` — The log level to use for logging output, defaults to `:info`","ref":"Oban.Telemetry.html#attach_default_logger/1-options"},{"type":"function","title":"Examples - Oban.Telemetry.attach_default_logger/1","doc":"Attach a logger at the default `:info` level with JSON encoding:\n\n    Oban.Telemetry.attach_default_logger()\n\nAttach a logger at the `:debug` level:\n\n    Oban.Telemetry.attach_default_logger(level: :debug)\n\nAttach a logger with JSON logging disabled:\n\n    Oban.Telemetry.attach_default_logger(encode: false)\n\nExplicitly attach a logger for all supported events:\n\n    Oban.Telemetry.attach_default_logger(events: :all)\n\nAttach a logger with only `:notifier`, `:peer`, and `:stager` events logged:\n\n    Oban.Telemetry.attach_default_logger(events: ~w(notifier peer stager)a)","ref":"Oban.Telemetry.html#attach_default_logger/1-examples"},{"type":"function","title":"Oban.Telemetry.default_handler_id/0","doc":"The unique id used to attach telemetry logging.\n\nThis is the constant `\"oban-default-logger\"` and exposed for testing purposes.","ref":"Oban.Telemetry.html#default_handler_id/0"},{"type":"function","title":"Oban.Telemetry.detach_default_logger/0","doc":"Undoes `Oban.Telemetry.attach_default_logger/1` by detaching the attached logger.","ref":"Oban.Telemetry.html#detach_default_logger/0"},{"type":"function","title":"Examples - Oban.Telemetry.detach_default_logger/0","doc":"Detach a previously attached logger:\n\n    :ok = Oban.Telemetry.attach_default_logger()\n    :ok = Oban.Telemetry.detach_default_logger()\n\nAttempt to detach when a logger wasn't attached:\n\n    {:error, :not_found} = Oban.Telemetry.detach_default_logger()","ref":"Oban.Telemetry.html#detach_default_logger/0-examples"},{"type":"type","title":"Oban.Telemetry.event_types/0","doc":"The types of telemetry events, essentially the second element of each event list. For example,\nin the event `[:oban, :job, :start]`, the \"type\" is `:job`.","ref":"Oban.Telemetry.html#t:event_types/0"},{"type":"type","title":"Oban.Telemetry.logger_opts/0","doc":"Available logging options.","ref":"Oban.Telemetry.html#t:logger_opts/0"},{"type":"behaviour","title":"Oban.Notifier","doc":"The `Notifier` coordinates listening for and publishing notifications for events in predefined\nchannels.\n\nOban functions such as `pause_queue`, `scale_queue`, and `cancel_job` all **require a connected\nnotifier to operate**. Use `status/1` to check the notifier's connectivity status and diagnose\nissues.","ref":"Oban.Notifier.html"},{"type":"behaviour","title":"Notifiers - Oban.Notifier","doc":"Every Oban supervision tree contains a notifier process, registered as `Oban.Notifier`, which is\nan implementation of the `Oban.Notifier` behaviour. \n\nChoosing a notifer comes with some tradeoffs; see each module for details.\n\n* `Oban.Notifiers.Postgres` — A Postgres notifier that uses `LISTEN/NOTIFY` to broadcast\n  messages.\n\n* `Oban.Notifiers.PG` — A process groups notifier that relies on Distributed Erlang to broadcast\n  messages.\n\n* [`Oban.Notifiers.Phoenix`](https://github.com/oban-bg/oban_notifiers_phoenix) — A notifier\n  that uses `Phoenix.PubSub` to broadcast messages. In addition to centralizing PubSub\n  communications, it opens up the possible transports to all PubSub adapters.","ref":"Oban.Notifier.html#module-notifiers"},{"type":"behaviour","title":"Channels - Oban.Notifier","doc":"All incoming notifications are relayed through the notifier to any processes listening on a\ngiven channel. Internally, Oban uses a variety of predefined channels with distinct\nresponsibilities:\n\n* `insert` — as jobs are inserted an event is published on the `insert` channel. Processes such\n  as queue producers use this as a signal to dispatch new jobs.\n\n* `leader` — messages regarding node leadership exchanged between peers\n\n* `signal` — instructions to take action, such as scale a queue or kill a running job, are sent\n  through the `signal` channel\n\n* `sonar` — periodic notification checks to monitor pubsub health and determine connectivity","ref":"Oban.Notifier.html#module-channels"},{"type":"behaviour","title":"Examples - Oban.Notifier","doc":"Broadcasting after a job is completed:\n\n    defmodule MyApp.Worker do\n      use Oban.Worker\n\n      @impl Oban.Worker\n      def perform(job) do\n        :ok = MyApp.do_work(job.args)\n\n        Oban.Notifier.notify(Oban, :my_app_jobs, %{complete: job.id})\n\n        :ok\n      end\n    end\n\nListening for job complete events from another process:\n\n    def insert_and_listen(args) do\n      :ok = Oban.Notifier.listen([:my_app_jobs])\n\n      {:ok, %{id: job_id} = job} =\n        args\n        |> MyApp.Worker.new()\n        |> Oban.insert()\n\n      receive do\n        {:notification, :my_app_jobs, %{\"complete\" => ^job_id}} ->\n          IO.puts(\"Other job complete!\")\n      after\n        30_000 ->\n          IO.puts(\"Other job didn't finish in 30 seconds!\")\n      end\n    end","ref":"Oban.Notifier.html#module-examples"},{"type":"type","title":"Oban.Notifier.channel/0","doc":"","ref":"Oban.Notifier.html#t:channel/0"},{"type":"function","title":"Oban.Notifier.listen/2","doc":"Register the current process to receive relayed messages for the provided channels.\n\nAll messages are received as `JSON` and decoded _before_ they are relayed to registered\nprocesses. Each registered process receives a three element notification tuple in the following\nformat:\n\n    {:notification, channel :: channel(), decoded :: map()}","ref":"Oban.Notifier.html#listen/2"},{"type":"function","title":"Example - Oban.Notifier.listen/2","doc":"Register to listen for all `:gossip` channel messages:\n\n    Oban.Notifier.listen(:gossip)\n\nListen for messages on multiple channels:\n\n    Oban.Notifier.listen([:gossip, :insert, :leader, :signal])\n\nListen for messages when using a custom Oban name:\n\n    Oban.Notifier.listen(MyApp.MyOban, [:gossip, :signal])","ref":"Oban.Notifier.html#listen/2-example"},{"type":"callback","title":"Oban.Notifier.listen/2","doc":"Register the current process to receive messages from one or more channels.","ref":"Oban.Notifier.html#c:listen/2"},{"type":"type","title":"Oban.Notifier.name_or_conf/0","doc":"","ref":"Oban.Notifier.html#t:name_or_conf/0"},{"type":"function","title":"Oban.Notifier.notify/3","doc":"Broadcast a notification to listeners on all nodes.\n\nNotifications are scoped to the configured `prefix`. For example, if there are instances running\nwith the `public` and `private` prefixes, a notification published in the `public` prefix won't\nbe picked up by processes listening with the `private` prefix.","ref":"Oban.Notifier.html#notify/3"},{"type":"function","title":"Example - Oban.Notifier.notify/3","doc":"Broadcast a gossip message:\n\n    Oban.Notifier.notify(:my_channel, %{message: \"hi!\"})\n\nBroadcast multiple messages at once:\n\n    Oban.Notifier.notify(:my_channel, [%{message: \"hi!\"}, %{message: \"there\"}])\n\nBroadcast using a custom instance name:\n\n    Oban.Notifier.notify(MyOban, :my_channel, %{message: \"hi!\"})","ref":"Oban.Notifier.html#notify/3-example"},{"type":"callback","title":"Oban.Notifier.notify/3","doc":"Broadcast a notification to all subscribers of a channel.","ref":"Oban.Notifier.html#c:notify/3"},{"type":"type","title":"Oban.Notifier.payload/0","doc":"","ref":"Oban.Notifier.html#t:payload/0"},{"type":"type","title":"Oban.Notifier.pubsub_status/0","doc":"","ref":"Oban.Notifier.html#t:pubsub_status/0"},{"type":"callback","title":"Oban.Notifier.start_link/1","doc":"Starts a notifier instance.","ref":"Oban.Notifier.html#c:start_link/1"},{"type":"function","title":"Oban.Notifier.status/1","doc":"Check a notifier's connectivity level to see whether it's able to publish or receive messages\nfrom other nodes.\n\nOban functions such as `pause_queue`, `scale_queue`, and `cancel_job` all require a connected\nnotifier to operate. Each Oban instance runs a persistent process to monitor connectivity,\nwhich is exposed by this function.","ref":"Oban.Notifier.html#status/1"},{"type":"function","title":"Statuses - Oban.Notifier.status/1","doc":"* `:unknown` — This is the default state on start before the notifier has time to determine the\n  appropriate status.\n\n* `:isolated` — The notifier isn't receiving any messages.\n\n  The notifier may be connected to a database but `:isolated` and unable to receive other\n  message and unable to receive outside messages. Typically, this is the case for the default\n  `Postgres` notifier while testing or behind a connection pooler.\n\n* `:solitary` — The notifier is only receiving messages from itself. This may be the case for\n  the `PG` notifier when Distributed Erlang nodes aren't connected, in development, or in\n  production deployments that only run a single node. If you're running multiple nodes in production\n  and the status is `:solitary`, there's a connectivity issue.\n\n* `:clustered` — The notifier is connected and able to receive messages from other nodes. The\n  `Postgres` notifier is considered clustered if it can receive notifications, while the PG\n  notifier requires a functional Distributed Erlang cluster.","ref":"Oban.Notifier.html#status/1-statuses"},{"type":"function","title":"Examples - Oban.Notifier.status/1","doc":"Check the notifier's pubsub status:\n\n    Oban.Notifier.status()\n\nCheck the status for a custom instance:\n\n    Oban.Notifier.status(MyOban)","ref":"Oban.Notifier.html#status/1-examples"},{"type":"function","title":"Oban.Notifier.unlisten/2","doc":"Unregister the current process from receiving relayed messages on provided channels.","ref":"Oban.Notifier.html#unlisten/2"},{"type":"function","title":"Example - Oban.Notifier.unlisten/2","doc":"Stop listening for messages on the `:gossip` channel:\n\n    Oban.Notifier.unlisten(:gossip)\n\nStop listening for messages on multiple channels:\n\n    Oban.Notifier.unlisten([:insert, :gossip])\n\nStop listening for messages when using a custom Oban name:\n\n    Oban.Notifier.unlisten(MyApp.MyOban, [:gossip])","ref":"Oban.Notifier.html#unlisten/2-example"},{"type":"callback","title":"Oban.Notifier.unlisten/2","doc":"Unregister current process from channels.","ref":"Oban.Notifier.html#c:unlisten/2"},{"type":"module","title":"Oban.Notifiers.PG","doc":"A [PG (Process Groups)][pg] based notifier implementation that runs with Distributed Erlang.\nThis notifier scales much better than `Oban.Notifiers.Postgres` but lacks its transactional\nguarantees.\n\n> #### Distributed Erlang {: .info}\n>\n> PG requires a functional [Distributed Erlang][de] cluster to broadcast between nodes. If your\n> application isn't clustered, then you should consider an alternative notifier such as\n> `Oban.Notifiers.Postgres`","ref":"Oban.Notifiers.PG.html"},{"type":"module","title":"Usage - Oban.Notifiers.PG","doc":"Specify the `PG` notifier in your Oban configuration:\n\n```elixir\nconfig :my_app, Oban,\n  notifier: Oban.Notifiers.PG,\n  ...\n```\n\nBy default, all Oban instances using the same `prefix` option will receive notifications from\neach other. You can use the `namespace` option to separate instances that are in the same\ncluster _without_ changing the prefix:\n\n```elixir\nconfig :my_app, Oban,\n  notifier: {Oban.Notifiers.PG, namespace: :custom_namespace}\n  ...\n```\n\nThe namespace may be any term.\n\n[pg]: https://www.erlang.org/doc/man/pg.html\n[de]: https://elixir-lang.org/getting-started/mix-otp/distributed-tasks.html#our-first-distributed-code","ref":"Oban.Notifiers.PG.html#module-usage"},{"type":"function","title":"Oban.Notifiers.PG.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Oban.Notifiers.PG.html#child_spec/1"},{"type":"module","title":"Oban.Notifiers.Postgres","doc":"A Postgres LISTEN/NOTIFY based Notifier.\n\n> #### Connection Pooling {: .info}\n>\n> Postgres PubSub is fine for most applications, but it doesn't work with connection poolers\n> like [PgBouncer][pgb] when configured in _transaction_ or _statement_ mode, which is\n> standard. Notifications are required for some core Oban functionality, and you should\n> consider using an alternative notifier such as `Oban.Notifiers.PG`.","ref":"Oban.Notifiers.Postgres.html"},{"type":"module","title":"Usage - Oban.Notifiers.Postgres","doc":"Specify the `Postgres` notifier in your Oban configuration:\n\n    config :my_app, Oban,\n      notifier: Oban.Notifiers.Postgres,\n      ...","ref":"Oban.Notifiers.Postgres.html#module-usage"},{"type":"module","title":"Transactions and Testing - Oban.Notifiers.Postgres","doc":"The notifications system is built on PostgreSQL's `LISTEN/NOTIFY` functionality. Notifications\nare only delivered **after a transaction completes** and are de-duplicated before publishing.\nThis means that notifications sent during a transaction will not be sent if the transaction is\nrolled back, providing consistency; this is the only notifer which provides that guarantee.\nHowever, it is not as scalable as other notifiers because because each notification requires a\nseparate query and notifications can't exceed 8kb.\n\nTypically, applications run Ecto in sandbox mode while testing, but sandbox mode wraps each test\nin a separate transaction that's rolled back after the test completes. That means the\ntransaction is never committed, which prevents delivering any notifications.\n\nTo test using notifications you must run Ecto without sandbox mode enabled, or use\n`Oban.Notifiers.PG` instead.\n\n[pgb]: https://www.pgbouncer.org/","ref":"Oban.Notifiers.Postgres.html#module-transactions-and-testing"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_call/3","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_call/3"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_connect/1","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_connect/1"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_disconnect/1","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_disconnect/1"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_info/2","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_info/2"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_result/2","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_result/2"},{"type":"function","title":"Oban.Notifiers.Postgres.init/1","doc":"","ref":"Oban.Notifiers.Postgres.html#init/1"},{"type":"behaviour","title":"Oban.Peer","doc":"The `Peer` module maintains leadership for a particular Oban instance within a cluster.\n\nLeadership is used by plugins, primarily, to prevent duplicate work across nodes. For example,\nonly the leader's `Cron` plugin will try inserting new jobs. You can use peer leadership to\nextend Oban with custom plugins, or even within your own application.\n\nNote a few important details about how peer leadership operates:\n\n* Each peer checks for leadership at a 30 second interval. When the leader exits it broadcasts a\n  message to all other peers to encourage another one to assume leadership.\n\n* Each Oban instance supervises a distinct `Oban.Peer` instance. That means that with multiple\n  Oban instances on the same node one instance may be the leader, while the others aren't.\n\n* Without leadership, global plugins (Cron, Lifeline, Stager, etc.), will not run on any node.","ref":"Oban.Peer.html"},{"type":"behaviour","title":"Available Peer Implementations - Oban.Peer","doc":"There are two built-in peering modules:\n\n* `Oban.Peers.Database` — uses table-based leadership through the `oban_peers` table and works\n  in any environment, with or without clustering. Only one node (per instance name) will have a\n  row in the peers table, that node is the leader. This is the default.\n\n* `Oban.Peers.Global` — coordinates global locks through distributed Erlang, requires\n  distributed Erlang.\n\nYou can specify the peering module to use in your Oban configuration:\n\n    config :my_app, Oban,\n      peer: Oban.Peers.Database, # default value\n      ...\n\nIf in doubt, you can call `Oban.config()` to see which module is being used.","ref":"Oban.Peer.html#module-available-peer-implementations"},{"type":"behaviour","title":"Examples - Oban.Peer","doc":"Check leadership for the default Oban instance:\n\n    Oban.Peer.leader?()\n    # => true\n\nThat is identical to using the name `Oban`:\n\n    Oban.Peer.leader?(Oban)\n    # => true\n\nCheck leadership for a couple of instances:\n\n    Oban.Peer.leader?(Oban.A)\n    # => true\n\n    Oban.Peer.leader?(Oban.B)\n    # => false","ref":"Oban.Peer.html#module-examples"},{"type":"type","title":"Oban.Peer.conf_or_name/0","doc":"","ref":"Oban.Peer.html#t:conf_or_name/0"},{"type":"callback","title":"Oban.Peer.get_leader/1","doc":"Check which node's peer instance currently leads the cluster.","ref":"Oban.Peer.html#c:get_leader/1"},{"type":"function","title":"Oban.Peer.get_leader/2","doc":"Get the name and node of the instance that currently leads the cluster.","ref":"Oban.Peer.html#get_leader/2"},{"type":"function","title":"Example - Oban.Peer.get_leader/2","doc":"Get the leader node for the default Oban instance:\n\n    Oban.Peer.get_leader()\n    \"web.1\"\n\nGet the leader node for an alternate instance named `Oban.Private`\n\n    Oban.Peer.get_leader(Oban.Private)\n    \"web.1\"","ref":"Oban.Peer.html#get_leader/2-example"},{"type":"callback","title":"Oban.Peer.leader?/1","doc":"Check whether the current peer instance leads the cluster.","ref":"Oban.Peer.html#c:leader?/1"},{"type":"function","title":"Oban.Peer.leader?/2","doc":"Check whether the current instance leads the cluster.","ref":"Oban.Peer.html#leader?/2"},{"type":"function","title":"Example - Oban.Peer.leader?/2","doc":"Check leadership for the default Oban instance:\n\n    Oban.Peer.leader?()\n    # => true\n\nCheck leadership for an alternate instance named `Oban.Private`:\n\n    Oban.Peer.leader?(Oban.Private)\n    # => true","ref":"Oban.Peer.html#leader?/2-example"},{"type":"type","title":"Oban.Peer.option/0","doc":"","ref":"Oban.Peer.html#t:option/0"},{"type":"callback","title":"Oban.Peer.start_link/1","doc":"Starts a peer instance.","ref":"Oban.Peer.html#c:start_link/1"},{"type":"module","title":"Oban.Peers.Database","doc":"A peer that coordinates centrally through a database table.\n\nDatabase peers don't require clustering through distributed Erlang or any other\ninterconnectivity between nodes. Leadership is coordinated through the `oban_peers` table in\nyour database. With a standard Oban config the `oban_peers` table will only have one row, and\nthat node is the leader.\n\nApplications that run multiple Oban instances will have one row per instance. For example, an\numbrella application that runs `Oban.A` and `Oban.B` will have two rows in `oban_peers`.","ref":"Oban.Peers.Database.html"},{"type":"module","title":"Usage - Oban.Peers.Database","doc":"This is the default peer for the `Basic` and `Dolphin` engines and no configuration is\nnecessary. However, to be explicit, specify the `Database` peer in your configuration.\n\n    config :my_app, Oban,\n      peer: Oban.Peers.Database,\n      ...","ref":"Oban.Peers.Database.html#module-usage"},{"type":"module","title":"Oban.Peers.Global","doc":"A cluster based peer that coordinates through a distributed registry.\n\nLeadership is coordinated through global locks. It requires a functional distributed Erlang\ncluster, without one global plugins (Cron, Lifeline, etc.) will not function correctly.","ref":"Oban.Peers.Global.html"},{"type":"module","title":"Usage - Oban.Peers.Global","doc":"Specify the `Global` peer in your Oban configuration.\n\n    config :my_app, Oban,\n      peer: Oban.Peers.Global,\n      ...","ref":"Oban.Peers.Global.html#module-usage"},{"type":"behaviour","title":"Oban.Engine","doc":"Defines an Engine for job orchestration.\n\nEngines are responsible for all non-plugin database interaction, from inserting through\nexecuting jobs.\n\nOban ships with three Engine implementations:\n\n1. `Basic` — The default engine for development, production, and manual testing mode.\n2. `Inline` — Designed specifically for testing, it executes jobs immediately, in-memory, as\n   they are inserted.\n3. `Lite` - The engine for running Oban using SQLite3.\n\n> #### 🌟 SmartEngine {: .info}\n>\n> The Basic engine lacks advanced functionality such as global limits, rate limits, and\n> unique bulk insert. For those features and more, see the [`Smart` engine in Oban\n> Pro](https://oban.pro/docs/pro/Oban.Pro.Engines.Smart.html).","ref":"Oban.Engine.html"},{"type":"callback","title":"Oban.Engine.cancel_all_jobs/2","doc":"Mark many `executing`, `available`, `scheduled` or `retryable` job as `cancelled` to prevent them\nfrom running.","ref":"Oban.Engine.html#c:cancel_all_jobs/2"},{"type":"callback","title":"Oban.Engine.cancel_job/2","doc":"Mark an `executing`, `available`, `scheduled` or `retryable` job as `cancelled` to prevent it\nfrom running.","ref":"Oban.Engine.html#c:cancel_job/2"},{"type":"callback","title":"Oban.Engine.check_available/1","doc":"Check for a list of queues with available jobs.","ref":"Oban.Engine.html#c:check_available/1"},{"type":"callback","title":"Oban.Engine.check_meta/3","doc":"Format engine meta in a digestible format for queue inspection.","ref":"Oban.Engine.html#c:check_meta/3"},{"type":"callback","title":"Oban.Engine.complete_job/2","doc":"Record that a job completed successfully.","ref":"Oban.Engine.html#c:complete_job/2"},{"type":"type","title":"Oban.Engine.conf/0","doc":"","ref":"Oban.Engine.html#t:conf/0"},{"type":"callback","title":"Oban.Engine.delete_all_jobs/2","doc":"Delete many jobs that aren't currently `executing`.","ref":"Oban.Engine.html#c:delete_all_jobs/2"},{"type":"callback","title":"Oban.Engine.delete_job/2","doc":"Delete a job that isn't currently `executing`.","ref":"Oban.Engine.html#c:delete_job/2"},{"type":"callback","title":"Oban.Engine.discard_job/2","doc":"Transition a job to `discarded` and record an optional reason that it shouldn't be ran again.","ref":"Oban.Engine.html#c:discard_job/2"},{"type":"callback","title":"Oban.Engine.error_job/3","doc":"Record an executing job's errors and either retry or discard it, depending on whether it has\nexhausted its available attempts.","ref":"Oban.Engine.html#c:error_job/3"},{"type":"callback","title":"Oban.Engine.fetch_jobs/3","doc":"Fetch available jobs for the given queue, up to configured limits.","ref":"Oban.Engine.html#c:fetch_jobs/3"},{"type":"callback","title":"Oban.Engine.init/2","doc":"Initialize metadata for a queue engine.\n\nQueue metadata is used to identify and track subsequent actions such as fetching or staging\njobs.","ref":"Oban.Engine.html#c:init/2"},{"type":"callback","title":"Oban.Engine.insert_all_jobs/3","doc":"Insert multiple jobs into the database.","ref":"Oban.Engine.html#c:insert_all_jobs/3"},{"type":"callback","title":"Oban.Engine.insert_all_jobs/5","doc":"","ref":"Oban.Engine.html#c:insert_all_jobs/5"},{"type":"callback","title":"Oban.Engine.insert_job/3","doc":"Insert a job into the database.","ref":"Oban.Engine.html#c:insert_job/3"},{"type":"callback","title":"Oban.Engine.insert_job/5","doc":"","ref":"Oban.Engine.html#c:insert_job/5"},{"type":"type","title":"Oban.Engine.job/0","doc":"","ref":"Oban.Engine.html#t:job/0"},{"type":"type","title":"Oban.Engine.meta/0","doc":"","ref":"Oban.Engine.html#t:meta/0"},{"type":"type","title":"Oban.Engine.opts/0","doc":"","ref":"Oban.Engine.html#t:opts/0"},{"type":"callback","title":"Oban.Engine.prune_jobs/3","doc":"Delete completed, cancelled and discarded jobs.","ref":"Oban.Engine.html#c:prune_jobs/3"},{"type":"callback","title":"Oban.Engine.put_meta/4","doc":"Store the given key/value pair in the engine meta.","ref":"Oban.Engine.html#c:put_meta/4"},{"type":"type","title":"Oban.Engine.queryable/0","doc":"","ref":"Oban.Engine.html#t:queryable/0"},{"type":"callback","title":"Oban.Engine.refresh/2","doc":"Refresh a queue to indicate that it is still alive.","ref":"Oban.Engine.html#c:refresh/2"},{"type":"callback","title":"Oban.Engine.rescue_jobs/3","doc":"Transition stuck jobs to `available` or `discarded` state based on attempts.","ref":"Oban.Engine.html#c:rescue_jobs/3"},{"type":"callback","title":"Oban.Engine.retry_all_jobs/2","doc":"Mark many jobs as `available`, adding attempts if already maxed out. Any jobs currently\n`available`, `executing` or `scheduled` should be ignored.","ref":"Oban.Engine.html#c:retry_all_jobs/2"},{"type":"callback","title":"Oban.Engine.retry_job/2","doc":"Mark a job as `available`, adding attempts if already maxed out. If the job is currently\n`available`, `executing` or `scheduled` it should be ignored.","ref":"Oban.Engine.html#c:retry_job/2"},{"type":"type","title":"Oban.Engine.running/0","doc":"","ref":"Oban.Engine.html#t:running/0"},{"type":"type","title":"Oban.Engine.seconds/0","doc":"","ref":"Oban.Engine.html#t:seconds/0"},{"type":"callback","title":"Oban.Engine.shutdown/2","doc":"Prepare a queue engine for shutdown.\n\nThe queue process is expected to stop processing new jobs after shutdown starts, though it may\ncontinue executing jobs that are already running.","ref":"Oban.Engine.html#c:shutdown/2"},{"type":"callback","title":"Oban.Engine.snooze_job/3","doc":"Reschedule an executing job to run some number of seconds in the future.","ref":"Oban.Engine.html#c:snooze_job/3"},{"type":"callback","title":"Oban.Engine.stage_jobs/3","doc":"Transition scheduled or retryable jobs to available prior to execution.","ref":"Oban.Engine.html#c:stage_jobs/3"},{"type":"module","title":"Oban.Engines.Basic","doc":"The default engine for use with Postgres databases.","ref":"Oban.Engines.Basic.html"},{"type":"module","title":"Usage - Oban.Engines.Basic","doc":"This is the default engine, no additional configuration is necessary:\n\n    Oban.start_link(repo: MyApp.Repo, queues: [default: 10])","ref":"Oban.Engines.Basic.html#module-usage"},{"type":"module","title":"Oban.Engines.Dolphin","doc":"An engine for running Oban with MySQL (via the MyXQL adapter).","ref":"Oban.Engines.Dolphin.html"},{"type":"module","title":"Usage - Oban.Engines.Dolphin","doc":"Start an `Oban` instance using the `Dolphin` engine:\n\n    Oban.start_link(\n      engine: Oban.Engines.Dolphin,\n      queues: [default: 10],\n      repo: MyApp.Repo\n    )","ref":"Oban.Engines.Dolphin.html#module-usage"},{"type":"module","title":"Oban.Engines.Inline","doc":"A testing-specific engine that's used when Oban is started with `testing: :inline`.","ref":"Oban.Engines.Inline.html"},{"type":"module","title":"Usage - Oban.Engines.Inline","doc":"This is meant for testing and shouldn't be configured directly:\n\n    Oban.start_link(repo: MyApp.Repo, testing: :inline)","ref":"Oban.Engines.Inline.html#module-usage"},{"type":"module","title":"Oban.Engines.Lite","doc":"An engine for running Oban with SQLite3.","ref":"Oban.Engines.Lite.html"},{"type":"module","title":"Usage - Oban.Engines.Lite","doc":"Start an `Oban` instance using the `Lite` engine:\n\n    Oban.start_link(\n      engine: Oban.Engines.Lite,\n      queues: [default: 10],\n      repo: MyApp.Repo\n    )","ref":"Oban.Engines.Lite.html#module-usage"},{"type":"exception","title":"Oban.CrashError","doc":"Wraps unhandled exits and throws that occur during job execution.","ref":"Oban.CrashError.html"},{"type":"exception","title":"Oban.PerformError","doc":"Wraps the reason returned by `{:error, reason}`, `{:cancel, reason}`, or `{:discard, reason}` in\na proper exception.\n\nThe original return tuple is available in the `:reason` key.","ref":"Oban.PerformError.html"},{"type":"exception","title":"Oban.TimeoutError","doc":"Returned when a job is terminated early due to a custom timeout.","ref":"Oban.TimeoutError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog for Oban v2.19\n\n_🌟 Looking for changes to [Oban Pro][pro]? Check the [Oban.Pro Changelog][opc] 🌟_\n\nThe minimum Elixir version is now v1.15. The official policy is to only support the three latest\nversions of Elixir.","ref":"changelog.html"},{"type":"extras","title":"🐬 MySQL Support - Changelog","doc":"Oban officially supports MySQL with the new `Dolphin` engine. Oban supports modern (read \"with\nfull JSON support\") MySQL [versions from 8.4][m84] on, and has been tested on the highly scalable\n[Planetscale][pla] database.\n\nRunning on MySQL is as simple as specifying the `Dolphin` engine in your configuration:\n\n```elixir\nconfig :my_app, Oban,\n  engine: Oban.Engines.Dolphin,\n  queues: [default: 10],\n  repo: MyApp.Repo\n```\n\nWith this addition, Oban can run in [estimated 10% more][myx] Elixir applications!\n\n[m84]: https://dev.mysql.com/doc/relnotes/mysql/8.4/en/\n[pla]: https://planetscale.com/\n[myx]: https://hex.pm/packages/myxql","ref":"changelog.html#mysql-support"},{"type":"extras","title":"⚗️ Automated Installer - Changelog","doc":"Installing Oban into a new application is simplified with a new [igniter][ign] powered `mix` task.\nThe new `oban.install` task handles installing and configuring a standard Oban installation, and\nit will deduce the correct `engine` and `notifier` automatically based on the database adapter.\n\n```bash\nmix igniter.install oban\n```\n\nThis `oban.install` task is currently the [recommended way to install][ins] Oban. As a bonus, the\ntask composes together with other igniter installers, making it possible to install `phoenix`,\n`ash`, `oban`, and other packages with a single command:\n\n```bash\nmix igniter.install phoenix ash_phoenix ash_postgres ash_oban\n```\n\nLook at the [`Mix.Oban.Install`][mio] docs for full usage and options.\n\n[ign]: https://hexdocs.pm/igniter/readme.html\n[ins]: installation.html\n[mio]: Mix.Tasks.Oban.Install.html","ref":"changelog.html#automated-installer"},{"type":"extras","title":"📔 Logging Enhancements - Changelog","doc":"Logging in a busy system may be noisy due to job events, but there are other events that are\nparticularly useful for diagnosing issues. A new `events` option for `attach_default_logger/1`\nallows selective event logging, so it's possible to receive important notices such as notifier\nconnectivity issues, without logging all job activity:\n\n```elixir\nOban.Telemetry.attach_default_logger(events: ~w(notifier peer stager)a)\n```\n\nAlong with filtering, there are new events to make diagnosing operational problems easier.\n\nA `peer:election` events logs leadership changes to indicate when nodes gain or lose leadership.\nLeadership issues are rare, but insidious, and make diagnosing production problems especially\ntricky.\n\n```elixir\n[\n  message: \"peer became leader\",\n  source: \"oban\",\n  event: \"peer:election\",\n  node: \"worker.1\",\n  leader: true,\n  was_leader: false\n]\n```\n\nHelpfully, `plugin:stop` events are now logged for all core plugins via an optional callback, and\n`plugin:exception` events are logged for all plugins regardless of whether they implement the\ncallback. Runtime information is logged for `Cron`, `Lifeline`, `Pruner`, `Stager`, and\n`Reindexer` plugins.\n\nFor example, every time `Cron` runs successfully it will output details about the execution time\nand all of the inserted job ids:\n\n```elixir\n[\n  source: \"oban\",\n  duration: 103,\n  event: \"plugin:stop\",\n  plugin: \"Oban.Plugins.Cron\",\n  jobs: [1, 2, 3]\n]\n```","ref":"changelog.html#logging-enhancements"},{"type":"extras","title":"⛵️ Official JSON - Changelog","doc":"Oban will default to using the official `JSON` module built into Elixir v1.18+ when available.\n\nA new `Oban.JSON` module detects whether the official Elixir `JSON` module is available at compile\ntime. If it isn't available, then it falls back to `Jason`, and if `Jason` isn't available (which\nis extremely rare) then it warns about a missing module.\n\nThis approach was chosen over a config option for backward compatibility because Oban will only\nsupport the JSON module once the minimum supported Elixir version is v1.18.","ref":"changelog.html#official-json"},{"type":"extras","title":"v2.19.1 — 2025-01-27 - Changelog","doc":"","ref":"changelog.html#v2-19-1-2025-01-27"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Mix] Improve igniter installer idempotency and compatibility.\n\n  The installer now uses `on_exists: :skip` when generating a migration, so it composes safely\n  with other igniter installers. It also removes unnecessary `add_dep` calls that would overwrite\n  a previously specified Oban version with `~> 2.18`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.19.0 — 2025-01-16 - Changelog","doc":"","ref":"changelog.html#v2-19-0-2025-01-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Oban] Start all queues in parallel on initialization.\n\n  The midwife now starts queues using an async stream to parallelize startup and minimize boot\n  time for applications with many queues.\n\n- [Oban] Safely return `nil` from `check_queue/2` when checking queues that aren't running.\n\n  Checking on a queue that wasn't currently running on the local node now returns `nil` rather\n  than causing a crash. This makes it safer to check the whether a queue is running at all without\n  a `try/catch` clause.\n\n- [Oban] Add `check_all_queues/1` to gather all queue status in a single function.\n\n  This new helper gathers the \"check\" details from all running queues on the local node. While it\n  was previously possible to pull the queues list from config and call `check_queue/2` on each\n  entry, this more accurately pulls from the registry and checks each producer concurrently.\n\n- [Oban] Add `delete_job/2` and `delete_all_jobs/2` operations.\n\n  This adds `Oban.delete_job/2`, `Oban.delete_all_jobs/2`, Engine callbacks, and associated\n  operations for all native engines. Deleting jobs is now easier and safer, due to automatic state\n  protections.\n\n- [Engine] Record when a queue starts shutting down\n\n  Queue producer metadata now includes a `shutdown_started_at` field to indicate that a queue\n  isn't just paused, but is actually shutting down as well.\n\n- [Engine] Add `rescue_jobs/3` callback for all engines.\n\n  The `Lifeline` plugin formerly used two queries to rescue jobs—one to mark jobs with remaining\n  attempts as `available` and another that `discarded` the remaining stuck jobs. Those are now\n  combined into a single callback, with the base definition in the `Basic` engine.\n\n  MySQL won't accept a select in an update statement. The Dolphin implementation of\n  `rescue_jobs/3` uses multiple queries to return the relevant telemetry data and make multiple\n  updates.\n\n- [Cron] Introduce `Oban.Cron` with `schedule_interval/4`\n\n  The new `Cron` module allows processes, namely plugins, to get cron-like scheduled functionality\n  with a single function call. This will allow plugins to removes boilerplate around parsing,\n  scheduling, and evaluating for cron behavior.\n\n- [Registry] Add `select/1 ` to simplify querying for registered modules.\n\n- [Testing] Add `build_job/3` helper for easier testing.\n\n  Extract the mechanism for verifying and building jobs out of `perform_job/3` so that it's usable\n  in isolation. This also introduces `perform_job/2` for executing built jobs.\n\n- [Telemetry] Add information on leadership changes to `oban.peer.election` event.\n\n  An additional `was_leader?` field is included in `[:oban, :peer, :election | _]` event metadata\n  to make hooking into leadership change events simpler.\n\n- [Telemetry] Add callback powered logging for plugin events.\n\n  Events are now logged for plugins that implement the a new optional callback, and exceptions are\n  logged for all plugins regardless of whether they implement the callback.\n\n  This adds logging for `Cron`, `Lifeline`, `Pruner`, `Stager`, and `Reindexer`.\n\n- [Telemetry] Add peer election logging to default logger.\n\n  The default logger now includes leadership events to make identifying the leader, and leadership\n  changes between nodes, easier.\n\n- [Telemetry] Add option to restrict logging to certain events.\n\n  Logging in a busy system may be noisy due to job events, but there are other events that are\n  particularly useful for diagnosing issues. This adds an `events` option to\n  `attach_default_logger/1` to allow selective event logging.\n\n- [Telemetry] Expose `default_handler_id/0` for telemetry testing.\n\n  Simplifies testing whether the default logger is attached or detached in application code.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Chores - Changelog","doc":"- [Peer] The default database-backed peer was renamed from `Postgres` to `Database` because it is\n  also used for MySQL databases.","ref":"changelog.html#chores"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Oban] Allow overwriting all `insert/*` functions arities after `use Oban`.\n\n- [Node] Correctly handle `:node` option for `scale_queue/2`\n\n  Scoping `scale_queue/2` calls to a single node didn't work as advertised due to some extra\n  validation for producer meta compatibility.\n\n- [Migration] Fix version query for databases with non-unique `oid`\n\n  Use `pg_catalog.obj_description(object_oid, catalog_name)`, introduced in PostgreSQL 7.2, to\n  specify the `pg_class` catalog so only the `oban_jobs` description is returned.\n\n- [Pruner] Use state specific fields when querying for prunable jobs.\n\n  Using `scheduled_at` is not correct in all situations. Depending on job state,  one of\n  `cancelled_at`, `discarded_at`, or `scheduled_at` should be used.\n\n- [Peer] Conditionally return the current node as leader for isolated peers.\n\n  Prevents returning the current node name when leadership is disabled.\n\n- [Testing] Retain time as microseconds for `scheduled_at` tests.\n\n  Include microseconds in the `begin` and `until` times used for scheduled_at tests with a delta.\n  The prior version would truncate, which rounded the `until` down and broke microsecond level\n  checks.\n\n- [Telemetry] Correct spelling of \"elapsed\" in `oban.queue.shutdown` metadata.\n\n[pro]: https://oban.pro\n[opc]: https://oban.pro/docs/pro/changelog.html","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Installation","doc":"# Installation\n\nBefore starting ensure your application has Ecto configured to use [Postgrex][postgrex] for\nPostgres, [EctoSQLite3][ecto_sqlite3] for SQLite3, or [MyXQL][myxql] for use with MySQL.\n\nThere are three installation mechanisms available:\n\n* [Semi-Automatic Installation](#semi-automatic-installation) using an igniter powered mix task\n* [Igniter Installation](#igniter-installation) fully automatic installation using igniter\n* [Manual Installation](#manual-installation) add oban and handle all steps manually","ref":"installation.html"},{"type":"extras","title":"Semi-Automatic Installation - Installation","doc":"It's possible to use the `oban.install` task without the `igniter.install` escript available.\nFirst, add `oban` and `igniter` to your deps in `mix.exs`:\n\n```elixir\n{:oban, \"~> 2.19\"},\n{:igniter, \"~> 0.5\", only: [:dev]},\n```\n\nRun `mix deps.get` to fetch `oban`, then run the install task:\n\n```bash\nmix oban.install\n```\n\nThat will automate all of the manual steps listed below!","ref":"installation.html#semi-automatic-installation"},{"type":"extras","title":"Igniter Installation - Installation","doc":"For projects that have [igniter][igniter] available, Oban may be installed and configured with a\nsingle command:\n\n```bash\nmix igniter.install oban\n```\n\nThat will add the latest version of `oban` to your dependencies before running the installer.\nInstallation will use the application's default Ecto repo, select the corresponding engine, and\nset the pubsub notifier accordingly.\n\nUse the `--repo` flag to specify an alternate repo manually:\n\n```bash\nmix igniter.install oban --repo MyApp.LiteRepo\n```","ref":"installation.html#igniter-installation"},{"type":"extras","title":"Manual Installation - Installation","doc":"Add `:oban` to your list of deps in `mix.exs`:\n\n```elixir\n{:oban, \"~> 2.19\"}\n```\n\nThen run `mix deps.get` to install Oban and its dependencies. After the packages are installed you\nmust create a database migration to add the `oban_jobs` table to your database:\n\n```bash\nmix ecto.gen.migration add_oban_jobs_table\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`Oban.Migration`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddObanJobsTable do\n  use Ecto.Migration\n\n  def up do\n    Oban.Migration.up(version: 12)\n  end\n\n  # We specify `version: 1` in `down`, ensuring that we'll roll all the way back down if\n  # necessary, regardless of which version we've migrated `up` to.\n  def down do\n    Oban.Migration.down(version: 1)\n  end\nend\n```\n\nThis will run all of Oban's versioned migrations for your database. Migrations between versions\nare idempotent and rarely change after a release. As new versions are released you may need to run\nadditional migrations.\n\nNow, run the migration to create the table:\n\n```bash\nmix ecto.migrate\n```\n\nBefore you can run an Oban instance you must provide some base configuration:\n\n<!-- tabs-open -->","ref":"installation.html#manual-installation"},{"type":"extras","title":"Postgres - Installation","doc":"Running with Postgres requires using the `Oban.Engines.Basic` engine:\n\n```elixir\n# config/config.exs\nconfig :my_app, Oban,\n  engine: Oban.Engines.Basic,\n  queues: [default: 10],\n  repo: MyApp.Repo\n```","ref":"installation.html#postgres"},{"type":"extras","title":"SQLite3 - Installation","doc":"Running with SQLite3 requires using the `Oban.Engines.Lite` engine:\n \n```elixir\n# config/config.exs\nconfig :my_app, Oban,\n  engine: Oban.Engines.Lite,\n  queues: [default: 10],\n  repo: MyApp.Repo\n```","ref":"installation.html#sqlite3"},{"type":"extras","title":"MySQL - Installation","doc":"Running with MySQL requires using the `Oban.Engines.Dolphin` engine:\n\n```elixir\n# config/config.exs\nconfig :my_app, Oban,\n  engine: Oban.Engines.Dolphin,\n  queues: [default: 10],\n  repo: MyApp.Repo\n```\n\n<!-- tabs-close -->\n\nTo prevent Oban from running jobs and plugins during test runs, enable `:testing` mode in\n`test.exs`:\n\n```elixir\n# config/test.exs\nconfig :my_app, Oban, testing: :manual\n```\n\nOban instances are isolated supervision trees and must be included in your application's\nsupervisor to run. Use the application configuration you've just set and include Oban in the list\nof supervised children:\n\n```elixir\n# lib/my_app/application.ex\ndef start(_type, _args) do\n  children = [\n    MyApp.Repo,\n    {Oban, Application.fetch_env!(:my_app, Oban)}\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\nend\n```\n\nFinally, verify that Oban is configured and running properly. Within a new `iex -S mix` session:\n\n```elixir\niex(1)> Oban.config()\n#=> %Oban.Config{repo: MyApp.Repo}\n```\n\nYou're all set! Add the [Oban Web][web] dashboard for monitoring, get started creating jobs and\nconfiguring queues in [Usage][use], or head to the [testing guide][test] to learn how to test with\nOban.\n\n[use]: Oban.html#Usage\n[web]: https://github.com/oban-bg/oban_web\n[igniter]: https://hex.pm/packages/igniter\n[test]: testing.md\n[postgrex]: https://hex.pm/packages/postgrex\n[ecto_sqlite3]: https://hex.pm/packages/ecto_sqlite3\n[myxql]: https://hex.pm/packages/myxql","ref":"installation.html#mysql"},{"type":"extras","title":"Ready for Production","doc":"# Ready for Production\n\nThere are a few additional bits of configuration to consider before you're ready to run Oban in\nproduction. In development and test environments, job data is short lived and there's no scale to\ncontend with. Now we'll dig into enabling introspection, external observability, and maintaining\ndatabase health.","ref":"ready_for_production.html"},{"type":"extras","title":"Logging - Ready for Production","doc":"Oban heavily utilizes Telemetry for instrumentation at every level. From job execution, plugin\nactivity, through to every database call there's a telemetry event to hook into.\n\nThe simplest way to leverage Oban's telemetry usage is through the default logger, available with\n`Oban.Telemetry.attach_default_logger/1`. Attach the logger in your `application.ex`:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  @impl Application\n  def start(_type, _args) do\n    Oban.Telemetry.attach_default_logger()\n\n    children = [\n      ...\n    ]\n  end\nend\n```\n\nBy default, the logger emits JSON encoded logs at the `:info` level. You can disable encoding and\nfall back to structured logging with `encode: false`, or change the log level with the `:level`\noption.\n\nFor example, to log without encoding at the `:debug` level:\n\n```elixir\nOban.Telemetry.attach_default_logger(encode: false, level: :debug)\n```","ref":"ready_for_production.html#logging"},{"type":"extras","title":"Pruning Jobs - Ready for Production","doc":"Job introspection and uniqueness relies on keeping job rows in the database after they have\nexecuted. To prevent the `oban_jobs` table from growing indefinitely, the `Oban.Plugins.Pruner`\nplugin provides out-of-band deletion of `completed`, `cancelled` and `discarded` jobs.\n\nRetaining jobs for 7 days is a good starting point, but depending on throughput, you may wish to\nkeep jobs for even longer. Include `Pruner` in the list of plugins and configure it to retain jobs\nfor 7 days, specified in seconds:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    {Oban.Plugins.Pruner, max_age: 60 * 60 * 24 * 7},\n  ...\n```","ref":"ready_for_production.html#pruning-jobs"},{"type":"extras","title":"Rescuing Jobs - Ready for Production","doc":"During deployment or unexpected node restarts jobs may be left in an executing state indefinitely.\nWe call these jobs \"orphans\", but orphaning isn't a bad thing. It means that the job wasn't lost\nand it may be retried again when the system comes back online.\n\nThere are two mechanisms to mitigate orphans:\n\n1. Use the `Oban.Plugins.Lifeline` plugin to automatically move those jobs back to available so\n   they can run again.\n2. Increase the `shutdown_grace_period` to allow the system more time to finish executing before\n   shutdown.\n\nEven with a higher `shutdown_grace_period` it's possible to have orphans if there is an unexpected\ncrash or extra long running jobs.\n\nConsider adding the `Lifeline` plugin and configure it to rescue after a generous period of time,\nlike 30 minutes:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    {Oban.Plugins.Lifeline, rescue_after: :timer.minutes(30)},\n  ...\n```","ref":"ready_for_production.html#rescuing-jobs"},{"type":"extras","title":"Error Handling - Ready for Production","doc":"Telemetry events can be used to report issues externally to services like Sentry or AppSignal.\nWrite a handler that sends error notifications to a third party (use a mock, or something that\nsends a message back to the test process).\n\nYou can use exception events to send error reports to Honeybadger, Rollbar, AppSignal, ErrorTracker \nor any other application monitoring platform.\n\nSome libraries like AppSignal, ErrorTracker or Sentry automatically handle these events without \nrequiring any extra code on your application. You can take a look at ErrorTracker's \n[Oban integration](https://github.com/elixir-error-tracker/error-tracker/blob/main/lib/error_tracker/integrations/oban.ex) \nas an example on how to attach to and use `Oban.Telemetry` events.\n\nIf you need a custom integration you can add a reporter module that fit your needs. \nHere's an example reporter module for [Sentry](https://hex.pm/packages/sentry):\n\n```elixir\ndefmodule MyApp.ObanReporter do\n  def attach do\n    :telemetry.attach(\"oban-errors\", [:oban, :job, :exception], &__MODULE__.handle_event/4, [])\n  end\n\n  def handle_event([:oban, :job, :exception], measure, meta, _) do\n    extra =\n      meta.job\n      |> Map.take([:id, :args, :meta, :queue, :worker])\n      |> Map.merge(measure)\n\n    Sentry.capture_exception(meta.reason, stacktrace: meta.stacktrace, extra: extra)\n  end\nend\n```\n\nAttach the handler when your application boots:\n\n```elixir\n# application.ex\n@impl Application\ndef start(_type, _args) do\n  MyApp.ObanReporter.attach()\nend\n```","ref":"ready_for_production.html#error-handling"},{"type":"extras","title":"Ship It! - Ready for Production","doc":"Now you're ready to ship to production with essential logging, error reporting, and baseline\njob maintenance.\n\nFor additional observability and introspection, consider integrating with one of these external\ntools built on `Oban.Telemetry`:\n\n* [Oban Web](https://hex.pm/packages/oban_web)—an official Oban package, it's a view of jobs,\n  queues, and metrics that you host directly within your application. Powered by Phoenix LiveView\n  and Oban Metrics, it is extremely lightweight and continuously updated.\n\n* [PromEx](https://hex.pm/packages/prom_ex)—Prometheus metrics and Grafana dashboards based on\n  metrics from job events, producer events, and also from internal polling jobs to monitor queue\n  sizes.\n\n* [AppSignal](https://docs.appsignal.com/elixir/integrations/oban.html)—The AppSignal for Elixir\n  package instruments jobs performed by Oban workers, and collects metrics about your jobs'\n  performance.\n\n* [ErrorTracker](https://hex.pm/packages/error_tracker)—an Elixir-based open source error tracking\n  solution that automatically integrates with Oban. It allows you to store and view exceptions on\n  your app without external services. It's powered by Telemetry, Ecto and Phoenix LiveView.","ref":"ready_for_production.html#ship-it"},{"type":"extras","title":"Defining Queues","doc":"# Defining Queues\n\nQueues are defined as a keyword list where the key is the name of the queue and the value is the\nmaximum number of concurrent jobs. The following configuration would start four queues with\nconcurrency ranging from 5 to 50:\n\n```elixir\nconfig :my_app, Oban,\n  queues: [default: 10, mailers: 20, events: 50, media: 5],\n  repo: MyApp.Repo\n```\n\nYou may also use an expanded form to configure queues with individual overrides:\n\n```elixir\nqueues: [\n  default: 10,\n  events: [limit: 50, paused: true]\n]\n```\n\nThe `events` queue will now start in a paused state, which means it won't process anything until\n`Oban.resume_queue/2` is called to start it.\n\nThere isn't a limit to the number of queues or how many jobs may execute\nconcurrently in each queue. Some additional guidelines:\n\n  * Each queue will run as many jobs as possible concurrently, up to the configured limit. Make\n  sure your system has enough resources (such as *database connections*) to handle the concurrent\n  load.\n\n  * Queue limits are **local** (per-node), not global (per-cluster). For example, running a queue\n  with a local limit of `2` on three separate nodes is effectively a global limit of *six\n  concurrent jobs*. If you require a global limit, you must restrict the number of nodes running a\n  particular queue or consider Oban Pro's [Smart Engine][smart], which can manage global\n  concurrency *automatically*!\n  * Only jobs in the configured queues will execute. Jobs in any other queue will\n  stay in the database untouched.\n\n  * Pay attention to the number of concurrent jobs making expensive system calls (such as calls to\n  resource-intensive tools like [FFMpeg][ffmpeg] or [ImageMagick][imagemagick]). The BEAM ensures\n  that the system stays responsive under load, but those guarantees don't apply when using ports\n  or shelling out commands.\n\n[ffmpeg]: https://www.ffmpeg.org\n[imagemagick]: https://imagemagick.org/index.php\n[smart]: https://oban.pro/docs/pro/Oban.Pro.Engines.Smart.html","ref":"defining_queues.html"},{"type":"extras","title":"Scheduling Jobs","doc":"# Scheduling Jobs\n\nYou can **schedule** jobs down to the second any time in the future:\n\n```elixir\n%{id: 1}\n|> MyApp.Business.new(schedule_in: _seconds = 5)\n|> Oban.insert()\n```\n\nJobs may also be scheduled at a *specific timestamp* in the future:\n\n```elixir\n%{id: 1}\n|> MyApp.Business.new(scheduled_at: ~U[2020-12-25 19:00:56.0Z])\n|> Oban.insert()\n```\n\nScheduling is *always* in UTC. You'll have to shift timestamps in other zones to UTC before\nscheduling:\n\n```elixir\n%{id: 1}\n|> MyApp.Business.new(scheduled_at: DateTime.shift_zone!(datetime, \"Etc/UTC\"))\n|> Oban.insert()\n```\n\nScheduling works across a cluster of nodes. See the [*Clustering* guide](clustering.html) for more\ninformation.","ref":"scheduling_jobs.html"},{"type":"extras","title":"Periodic Jobs","doc":"# Periodic Jobs\n\nOban's [cron plugin](`Oban.Plugins.Cron`) registers workers that run on a cron-like schedule and enqueues jobs automatically.\n\nPeriodic jobs are declared as a list of `{cron, worker}` or `{cron, worker, options}` tuples:\n\n```elixir\nconfig :my_app, Oban,\n  repo: MyApp.Repo,\n  plugins: [\n    {Oban.Plugins.Cron,\n     crontab: [\n       {\"* * * * *\", MyApp.MinuteWorker},\n       {\"0 * * * *\", MyApp.HourlyWorker, args: %{custom: \"arg\"}},\n       {\"0 0 * * *\", MyApp.DailyWorker, max_attempts: 1},\n       {\"0 12 * * MON\", MyApp.MondayWorker, queue: :scheduled, tags: [\"mondays\"]},\n       {\"@daily\", MyApp.AnotherDailyWorker}\n     ]}\n  ]\n```\n\nThe crontab would insert jobs as follows:\n\n  * `MyApp.MinuteWorker` — Inserted once every minute\n  * `MyApp.HourlyWorker` — Inserted at the first minute of every hour with custom arguments\n  * `MyApp.DailyWorker` — Inserted at midnight every day with no retries\n  * `MyApp.MondayWorker` — Inserted at noon every Monday in the \"scheduled\" queue\n  * `MyApp.AnotherDailyWorker` — Inserted at midnight every day with no retries\n\nThe crontab format respects all [standard rules][cron] and has one minute resolution. Jobs are\nalways inserted at the top by the leader node, which prevents duplicate jobs regardless of\ncluster size, restarts, or crashes.\n\nLike other jobs, recurring jobs will use the `:queue` specified by the worker module (or\n`:default` if one is not specified).","ref":"periodic_jobs.html"},{"type":"extras","title":"Cron Expressions - Periodic Jobs","doc":"> #### Crontab Guru {: .tip}\n>\n> Cron syntax can be hard to write and read. We recommend using a tool\n> like [Crontab Guru][guru] to make sense of cron expressions and write\n> new ones.\n\nStandard cron expressions are composed of rules specifying the minutes, hours, days, months and\nweekdays. Rules for each field are comprised of literal values, wildcards, step values or ranges:\n\n  * `*` — Wildcard, matches any value (0, 1, 2, …)\n  * `0` — Literal, matches only itself (only 0)\n  * `*/15` — Step, matches any value that is a multiple (0, 15, 30, 45)\n  * `0-5` — Range, matches any value within the range (0, 1, 2, 3, 4, 5)\n  * `0-9/2` — Step values can be used in conjunction with ranges (0, 2, 4, 6, 8)\n\nEach part may have multiple rules, where rules are separated by a comma. The allowed values for\neach field are as follows:\n\n| Field      | Allowed Values                               |\n| ---------- | -------------------------------------------- |\n| `minute`   | 0-59                                         |\n| `hour`     | 0-23                                         |\n| `days`     | 1-31                                         |\n| `month`    | 1-12 (or aliases, `JAN`, `FEB`, `MAR`, etc.) |\n| `weekdays` | 0-6 (or aliases, `SUN`, `MON`, `TUE`, etc.)  |","ref":"periodic_jobs.html#cron-expressions"},{"type":"extras","title":"Cron Extensions - Periodic Jobs","doc":"Oban supports these common cron extensions:\n\n| Expression                 | Translates To                                   |\n| -------------------------- | ----------------------------------------------- |\n| `@hourly`                  | `0 * * * *`                                     |\n| `@daily` (or `@midnight`)  | `0 0 * * *`                                     |\n| `@weekly`                  | `0 0 * * 0`                                     |\n| `@monthly`                 | `0 0 1 * *`                                     |\n| `@yearly` (or `@annually`) | `0 0 1 1 *`                                     |\n| `@reboot`                  | Run once at boot time across the entire cluster |","ref":"periodic_jobs.html#cron-extensions"},{"type":"extras","title":"Examples - Periodic Jobs","doc":"Here are some specific examples that demonstrate the full range of expressions:\n\n  * `0 * * * *` — The first minute of every hour\n  * `*/15 9-17 * * *` — Every fifteen minutes during standard business hours\n  * `0 0 * DEC *` — Once a day at midnight during December\n  * `0 7-9,4-6 13 * FRI` — Once an hour during both rush hours on Friday the 13th\n\nFor more in depth information see the man documentation for `cron` and `crontab` in your system.\nAlternatively you can experiment with various expressions online at [Crontab Guru][guru].","ref":"periodic_jobs.html#examples"},{"type":"extras","title":"Caveats & Guidelines - Periodic Jobs","doc":"* All schedules are evaluated as UTC unless a different timezone is provided. See\n  `Oban.Plugins.Cron` for information about configuring a timezone.\n\n  * Workers can be used for regular _and_ scheduled jobs so long as they accept different arguments.\n\n  * Long-running jobs may execute simultaneously if the scheduling interval is shorter than it takes\n  to execute the job.\n\n[cron]: https://en.wikipedia.org/wiki/Cron\n[guru]: https://crontab.guru","ref":"periodic_jobs.html#caveats-guidelines"},{"type":"extras","title":"Unique Jobs","doc":"# Unique Jobs\n\nThe *uniqueness* of a job is a somewhat complex topic. This guide is here to help you understand its complexities!\n\nThe unique jobs feature allows you to specify constraints to prevent *enqueuing* duplicate jobs.\nThese constraints only apply when jobs are inserted. Uniqueness has no bearing on whether jobs\nare *executed* concurrently.\nUniqueness is based on a combination of job attributes based on the following options:\n\n  * `:period` — The number of seconds until a job is no longer considered duplicate. You should\n    always specify a period, otherwise Oban will default to 60 seconds. `:infinity` can be used to\n    indicate the job be considered a duplicate as long as jobs are retained (see\n    `Oban.Plugins.Pruner`).\n\n  * `:fields` — The fields to compare when evaluating uniqueness. The available fields are\n    `:args`, `:queue`, `:worker`, and `:meta`. `:fields` defaults to `[:worker, :queue, :args]`.\n    It's recommended that you leave the default `:fields`, otherwise you risk unexpected conflicts\n    between unrelated jobs.\n\n  * `:keys` — A specific subset of the `:args` or `:meta` to consider when comparing against\n    historic jobs. This allows a job with multiple key/value pairs in its arguments to be compared\n    using only a subset of them.\n\n  * `:states` — The job states that are checked for duplicates. The available states are\n    described in `t:Oban.Job.unique_state/0`. By default, Oban checks all states except for\n    `:discarded` and `:cancelled`, which prevents duplicates even if the previous job has been\n    completed.\n\n  * `:timestamp` — Which job timestamp to check the period against. The available timestamps are\n    `:inserted_at` or `:scheduled_at`. Defaults to `:inserted_at` for legacy reasons.\n\nThe simplest form of uniqueness will configure uniqueness for as long as a matching job exists in\nthe database, regardless of state:\n\n```elixir\nuse Oban.Worker, unique: true\n```\n\nHere's a more complex example which uses multiple options:\n\n```elixir\nuse Oban.Worker,\n  unique: [\n    # Jobs should be unique for 2 minutes...\n    period: {2, :minutes},\n    # ...after being scheduled, not inserted\n    timestamp: :scheduled_at,\n    # Don't consider the whole :args field, but just the :url field within :args\n    keys: [:url],\n    # Consider a job unique across all states, including :cancelled/:discarded\n    states: Oban.Job.states(),\n    # Consider a job unique across queues; only compare the :url key within\n    # the :args, as per the :keys configuration above\n    fields: [:worker, :args]\n  ]\n```","ref":"unique_jobs.html"},{"type":"extras","title":"Detecting Unique Conflicts - Unique Jobs","doc":"When unique settings match an existing job, the return value of `Oban.insert/2` is still `{:ok,\njob}`. However, you can detect a unique conflict by checking the job's `:conflict?` field. If\nthere was an existing job, the field is `true`; otherwise it is `false`.\n\nYou can use the `:conflict?` field to customize responses after insert:\n\n```elixir\ncase Oban.insert(changeset) do\n  {:ok, %Job{id: nil, conflict?: true}} ->\n    {:error, :failed_to_acquire_lock}\n\n  {:ok, %Job{conflict?: true}} ->\n    {:error, :job_already_exists}\n\n  result ->\n    result\nend\n```\n\n> #### Caveat with `insert_all` {: .warning}\n>\n> Unless you are using Oban Pro's [Smart Engine][pro-smart-engine], Oban only detects conflicts\n> for jobs enqueued through [`Oban.insert/2,3`](`Oban.insert/2`). When using the [Basic\n> Engine](`Oban.Engines.Basic`), jobs enqueued through `Oban.insert_all/2` *do not* use per-job\n> unique configuration.","ref":"unique_jobs.html#detecting-unique-conflicts"},{"type":"extras","title":"Replacing Values - Unique Jobs","doc":"In addition to detecting unique conflicts, passing options to `:replace` can update any job field\nwhen there is a conflict. Any of the following fields can be replaced per *state*:\n\n  * `:args`\n  * `:max_attempts`\n  * `:meta`\n  * `:priority`\n  * `:queue`\n  * `:scheduled_at`\n  * `:tags`\n  * `:worker`\n\nFor example, to change the `:priority` and increase `:max_attempts` when there is a conflict with\na job in a `:scheduled` state:\n\n```elixir\nBusinessWorker.new(\n  args,\n  max_attempts: 5,\n  priority: 0,\n  replace: [scheduled: [:max_attempts, :priority]]\n)\n```\n\nAnother example is bumping the scheduled time on conflict. Either `:scheduled_at` or\n`:schedule_in` values will work, but the replace option is always `:scheduled_at`.\n\n```elixir\nUrgentWorker.new(args, schedule_in: 1, replace: [scheduled: [:scheduled_at]])\n```\n\n> #### Jobs in the `:executing` State {: .error}\n>\n> If you use this feature to replace a field (such as `:args`) in the `:executing` state by doing\n> something like\n>\n> ```elixir\n> UniqueWorker.new(new_args, replace: [executing: [:args]])\n> ```\n>\n> then Oban will update `:args`, but the job will continue executing with the original value.","ref":"unique_jobs.html#replacing-values"},{"type":"extras","title":"Unique Guarantees - Unique Jobs","doc":"Oban **strives** for uniqueness of jobs through transactional locks and database queries.\nUniqueness *does not* rely on unique constraints in the database, which leaves it prone to race\nconditions in some circumstances.\n\n[pro-smart-engine]: https://oban.pro/docs/pro/Oban.Pro.Engines.Smart.html","ref":"unique_jobs.html#unique-guarantees"},{"type":"extras","title":"Operational Maintenance","doc":"# Operational Maintenance\n\nThis guide walks you through *maintaining* a production Oban setup from an operational\nperspective.","ref":"operational_maintenance.html"},{"type":"extras","title":"Pruning Historic Jobs - Operational Maintenance","doc":"Job stats and queue introspection are built on *keeping job rows in the database* after they have\ncompleted. This allows administrators to review completed jobs and build informative aggregates,\nat the expense of storage and an unbounded table size. To prevent the `oban_jobs` table from\ngrowing indefinitely, Oban actively prunes `completed`, `cancelled`, and `discarded` jobs.\n\nBy default, the [pruner plugin](`Oban.Plugins.Pruner`) retains jobs for 60 seconds. You can\nconfigure a longer retention period by providing a `:max_age` in seconds to the pruner plugin.\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [{Oban.Plugins.Pruner, max_age: _5_minutes_in_seconds = 300}],\n  # ...\n```\n\n> #### Caveats & Guidelines {: .info}\n>\n> * Pruning is best-effort and performed out-of-band. This means that all limits are soft; jobs\n> beyond a specified age may not be pruned immediately after jobs complete.\n>\n> * Pruning is only applied to jobs that are `completed`, `cancelled`, or `discarded`. It'll never\n> delete a new job, a scheduled job, or a job that failed and will be retried.","ref":"operational_maintenance.html#pruning-historic-jobs"},{"type":"extras","title":"Instrumentation and Logging","doc":"# Instrumentation and Logging\n\nOban provides integration with [Telemetry][telemetry], a dispatching library for metrics and\ninstrumentation. It is easy to report Oban metrics to any backend by attaching to Telemetry events\nprefixed with `:oban`.","ref":"instrumentation.html"},{"type":"extras","title":"Default Logger - Instrumentation and Logging","doc":"The `Oban.Telemetry` module provides a robust structured logger that handles all of Oban's\ntelemetry events. As in the example above, attach it within your application module:\n\n```elixir\n:ok = Oban.Telemetry.attach_default_logger()\n```\n\nFor more details on the default structured logger and information on event metadata see docs for\nthe `Oban.Telemetry` module.","ref":"instrumentation.html#default-logger"},{"type":"extras","title":"Custom Handlers - Instrumentation and Logging","doc":"Here is an example of an unstructured log handler:\n\n```elixir\ndefmodule MyApp.ObanLogger do\n  require Logger\n\n  def handle_event([:oban, :job, :start], measure, meta, _) do\n    Logger.warning(\"[Oban] :started #{meta.worker} at #{measure.system_time}\")\n  end\n\n  def handle_event([:oban, :job, event], measure, meta, _) do\n    Logger.warning(\"[Oban] #{event} #{meta.worker} ran in #{measure.duration}\")\n  end\nend\n```\n\nAttach the handler to success and failure events in your application's `c:Application.start/2`\ncallback (usually in `lib/my_app/application.ex`):\n\n```elixir\ndef start(_type, _args) do\n  events = [\n    [:oban, :job, :start],\n    [:oban, :job, :stop],\n    [:oban, :job, :exception]\n  ]\n\n  :telemetry.attach_many(\"oban-logger\", events, &MyApp.ObanLogger.handle_event/4, [])\n\n  Supervisor.start_link(...)\nend\n```\n\n[telemetry]: https://github.com/beam-telemetry/telemetry","ref":"instrumentation.html#custom-handlers"},{"type":"extras","title":"Error Handling","doc":"# Error Handling\n\nThis page guides you through handling and reporting errors in Oban.\n\nJobs can fail in expected or unexpected ways. To mark a job as failed, you can return `{:error,\nreason}` from a worker's [`perform/1` callback](`c:Oban.Worker.perform/1`), as documented in the\n`t:Oban.Worker.result/0` type. A job can also fail because of unexpected raised errors or exits.\n\nIn any case, when a job fails the details of the failure are recorded in the `errors` array on the\n`Oban.Job` struct.","ref":"error_handling.html"},{"type":"extras","title":"Error Details - Error Handling","doc":"Oban stores execution errors as a list of maps (`t:Oban.Job.errors/0`). Each error contains the\nfollowing keys:\n\n  * `:at` — The UTC timestamp when the error occurred at\n  * `:attempt` — The attempt number when the error occurred\n  * `:error` — A *formatted* error message and stacktrace\n\nSee the [Instrumentation docs](instrumentation.html) for an example of integrating with external\nerror reporting systems.","ref":"error_handling.html#error-details"},{"type":"extras","title":"Retries - Error Handling","doc":"When a job fails and the number of execution attempts is below the configured `max_attempts` limit\nfor that job, the job will automatically be retried in the future. If the number of failures\nreaches `max_attempts`, the job gets **discarded**.\n\nThe retry delay has an *exponential backoff with jitter*. This means that the delay between\nattempts grows exponentially (8s, 16s, and so on), and a randomized \"jitter\" is introduced for\neach attempt, so that chances of jobs overlapping when being retried go down. So, a job could be\nretried after 7.3s, then 17.1s, and so on.\n\nSee the `Oban.Worker` documentation on \"Customizing Backoff\" for alternative backoff strategies.","ref":"error_handling.html#retries"},{"type":"extras","title":"Limiting Retries - Error Handling","doc":"By default, jobs are retried up to 20 times. The number of retries is controlled by the\n`:max_attempts` value, which can be set at the **worker** or **job** level. For example, to\ninstruct a worker to discard jobs after three failures:\n\n```elixir\nuse Oban.Worker, queue: :limited, max_attempts: 3\n```","ref":"error_handling.html#limiting-retries"},{"type":"extras","title":"Reporting Errors - Error Handling","doc":"Another great use of execution data and instrumentation is error reporting. Here is an example of\nan event handler module that integrates with [Honeybadger][honeybadger] to report job failures:\n\n```elixir\ndefmodule MyApp.ErrorReporter do\n  def attach do\n    :telemetry.attach(\n      \"oban-errors\",\n      [:oban, :job, :exception],\n      &__MODULE__.handle_event/4,\n      []\n    )\n  end\n\n  def handle_event([:oban, :job, :exception], measure, meta, _) do\n    Honeybadger.notify(meta.reason, stacktrace: meta.stacktrace)\n  end\nend\n\n# Attach it with:\nMyApp.ErrorReporter.attach()\n```\n\nYou can use exception events to send error reports to Sentry, AppSignal, Honeybadger, Rollbar, or\nany other application monitoring platform.","ref":"error_handling.html#reporting-errors"},{"type":"extras","title":"Built-in Reporting - Error Handling","doc":"Some error-reporting and application-monitoring services support reporting Oban errors out of the\nbox:\n\n  - Sentry — [Oban integration documentation][sentry-integration]\n  - AppSignal — [Oban integration documentation][appsignal-integration]\n\n[honeybadger]: https://www.honeybadger.io\n[sentry-integration]: https://docs.sentry.io/platforms/elixir/integrations/oban\n[appsignal-integration]: https://docs.appsignal.com/elixir/integrations/oban.html","ref":"error_handling.html#built-in-reporting"},{"type":"extras","title":"Clustering","doc":"# Clustering\n\nOban supports running in clusters of nodes. It supports both nodes that are connected to each\nother (via *distributed Erlang*), as well as nodes that are not connected to each other but that\ncommunicate via the database's pubsub mechanism.\n\nUsually, scheduled job management operates in **global mode** and notifies queues of available\njobs via pub/sub to minimize database load. However, when pubsub isn't available, staging\nswitches to a **local mode** where each queue polls independently.\n\nLocal mode is less efficient and will only happen if you're running in an environment where\nneither PostgreSQL nor PG notifications work. That situation should be rare and limited to the\nfollowing conditions:\n\n  1. Running with a connection pooler, like [pg_bouncer], in transaction mode.\n  2. Running without clustering, that is, without *distributed Erlang*.\n\nIf **both** of those criteria apply and pubsub notifications won't work, then staging will switch\nto polling in local mode.\n\n[pg_bouncer]: http://www.pgbouncer.org","ref":"clustering.html"},{"type":"extras","title":"Instance and Database Isolation","doc":"# Instance and Database Isolation\n\nThis guide will walk you through options for isolating Oban instances as well as Oban database\ntables.","ref":"isolation.html"},{"type":"extras","title":"Running Multiple Oban Instances - Instance and Database Isolation","doc":"You can run multiple Oban instances with different prefixes on the same system and have them\nentirely isolated, provided you give each Oban supervisor a distinct name. You can do this in one\nof two ways: explicit names or **facades**.","ref":"isolation.html#running-multiple-oban-instances"},{"type":"extras","title":"Facades - Instance and Database Isolation","doc":"You can create an [Oban **facade**](`Oban.__using__/1`) by defining a module that calls `use\nOban`:\n\n```elixir\ndefmodule MyApp.ObanA do\n  use Oban, otp_app: :my_app\nend\n\ndefmodule MyApp.ObanB do\n  use Oban, otp_app: :my_app\nend\n```\n\nConfigure facades through the application environment, for example in `config/config.exs`:\n\n```elixir\nconfig :my_app, MyApp.ObanA, repo: MyAppo.Repo, prefix: \"special\"\nconfig :my_app, MyApp.ObanB, repo: MyAppo.Repo, prefix: \"private\"\n```\n\nYou can then start these facades in your application's supervision tree:\n\n```elixir\n@impl true\ndef start(_type, _args) do\n  children = [\n    MyApp.Repo,\n    MyApp.ObanA,\n    MyApp.ObanB\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\nend\n```\n\nOban facades define all the functions that the `Oban` module defines, so use the facade in place\nof `Oban`:\n\n```elixir\nMyApp.ObanA.insert(MyApp.Worker.new(%{}))\n```","ref":"isolation.html#facades"},{"type":"extras","title":"Isolated Instances Via Names - Instance and Database Isolation","doc":"Here we configure our application to start three Oban supervisors using the `\"public\"` (default),\n`\"special\"`, and `\"private\"` prefixes, respectively:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    MyApp.Repo,\n    {Oban, name: ObanA, repo: MyApp.Repo},\n    {Oban, name: ObanB, repo: MyApp.Repo, prefix: \"special\"},\n    {Oban, name: ObanC, repo: MyApp.Repo, prefix: \"private\"}\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\nend\n```\n\nWhen you do this, you'll have to use the correct Oban supervisor name when performing Oban-related\noperations. You'll see that most functions in the `Oban` module, for example, take an optional\nfirst argument which represents the name of the Oban supervisor. By default, that's `Oban`, which\nis why this works if you don't explicitly start an Oban supervisor in your application:\n\n```elixir\nOban.insert(MyApp.Worker.new(%{}))\n```\n\nIn the example above, with `ObanA`/`ObanB`/`ObanC`, you can specify which Oban instance you want\nto use for scheduling by passing its name in:\n\n```elixir\nOban.insert(ObanB, MyApp.Worker.new(%{}))\n```","ref":"isolation.html#isolated-instances-via-names"},{"type":"extras","title":"Umbrella Apps - Instance and Database Isolation","doc":"If you need to run Oban from an umbrella application where more than one of the child apps need to\ninteract with Oban, you may need to set the `:name` for each child application that configures\nOban.\n\nFor example, your umbrella contains two apps: `MyAppA` and `MyAppB`. `MyAppA` is responsible for\ninserting jobs, while only `MyAppB` actually runs any queues.\n\nConfigure Oban with a custom name for `MyAppA`:\n\n```elixir\nconfig :my_app_a, Oban,\n  name: MyAppA.Oban,\n  repo: MyApp.Repo\n```\n\nThen configure Oban for `MyAppB` with a different name and different options:\n\n```elixir\nconfig :my_app_b, Oban,\n  name: MyAppB.Oban,\n  repo: MyApp.Repo,\n  queues: [default: 10]\n```\n\nNow, use the configured name when calling functions like `Oban.insert/2`, `Oban.insert_all/2`,\n`Oban.drain_queue/2`, and so on, to reference the correct Oban process for the current\napplication.\n\n```elixir\nOban.insert(MyAppA.Oban, MyWorker.new(%{}))\nOban.insert_all(MyAppB.Oban, multi, :multiname, [MyWorker.new(%{})])\nOban.drain_queue(MyAppB.Oban, queue: :default)\n```","ref":"isolation.html#umbrella-apps"},{"type":"extras","title":"Database Isolation - Instance and Database Isolation","doc":"Let's look at a few options for isolating or scoping Oban database queries.","ref":"isolation.html#database-isolation"},{"type":"extras","title":"Database Prefixes - Instance and Database Isolation","doc":"Oban supports namespacing through **PostgreSQL schemas**, also called \"prefixes\" in Ecto. With\nprefixes, your job table can reside outside of your primary schema (usually `public`) and you can\nhave multiple separate job tables.\n\nTo use a prefix you first have to specify it within your migration:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedObanJobsTable do\n  use Ecto.Migration\n\n  def up do\n    Oban.Migrations.up(prefix: \"private\")\n  end\n\n  def down do\n    Oban.Migrations.down(prefix: \"private\")\n  end\nend\n```\n\nThe migration will create the `private` schema and all Oban-related tables within that schema.\nWith the database migrated, you'll then specify the prefix in your configuration:\n\n```elixir\nconfig :my_app, Oban,\n  prefix: \"private\",\n  repo: MyApp.Repo,\n  queues: [default: 10]\n```\n\nNow all jobs are inserted and executed using the `private.oban_jobs` table. Note that while\n`Oban.insert/2,4` will write jobs in the `private.oban_jobs` table automatically, you'll need to\nspecify a prefix *manually* if you insert jobs directly through a repo.\n\nNot only is the `oban_jobs` table isolated within the schema, but all notification events are also\nisolated. That means that insert/update events will only dispatch new jobs for their prefix.","ref":"isolation.html#database-prefixes"},{"type":"extras","title":"Dynamic Repositories - Instance and Database Isolation","doc":"Oban supports [Ecto dynamic repositories][dynamic] through the `:get_dynamic_repo` option. To make\nthis work, you need to run a separate Oban instance for each dynamic repo instance. Most often\nit's worth bundling each Oban and repo instance under the same supervisor:\n\n```elixir\ndef start_repo_and_oban(instance_id) do\n  children = [\n    {MyDynamicRepo, name: nil, url: repo_url(instance_id)},\n    {Oban, name: instance_id, get_dynamic_repo: fn -> repo_pid(instance_id) end}\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one)\nend\n```\n\nThe function `repo_pid/1` in this example must return the PID of the repo for the given instance.\nYou can use `Registry` to register the repo (for example in the repo's `init/2` callback) and\ndiscover it.\n\nIf your application exclusively uses dynamic repositories and doesn't specify all credentials\nupfront, you must implement a `init/1` callback in your Ecto repo. Doing so provides the Postgres\nnotifier with the correct credentials on initialization, allowing jobs to process as expected.","ref":"isolation.html#dynamic-repositories"},{"type":"extras","title":"Ecto Multi-tenancy - Instance and Database Isolation","doc":"If you followed the Ecto guide on setting up multi-tenancy with foreign keys, you need to add an\nexception for queries originating from Oban. All of Oban's queries have the custom option `oban:\ntrue` to help you identify them in `prepare_query/3` or other instrumentation:\n\n```elixir\n# Sample code, only relevant if you followed the Ecto guide on multi tenancy with foreign keys.\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app\n\n  require Ecto.Query\n\n  @impl true\n  def prepare_query(_operation, query, opts) do\n    cond do\n      opts[:skip_org_id] || opts[:schema_migration] || opts[:oban] ->\n        {query, opts}\n\n      org_id = opts[:org_id] ->\n        {Ecto.Query.where(query, org_id: ^org_id), opts}\n\n      true ->\n        raise \"expected org_id or skip_org_id to be set\"\n    end\n  end\nend\n```\n\n[dynamic]: https://hexdocs.pm/ecto/replicas-and-dynamic-repositories.html#dynamic-repositories","ref":"isolation.html#ecto-multi-tenancy"},{"type":"extras","title":"Writing Plugins","doc":"# Writing Plugins\n\nOban supports the use of plugins to extend its base functionality. A plugin is\nany module that begins a process and exposes a `start_link/1` function. That\nmeans a plugin may be a `GenServer`, an `Agent`, a `Task`, or any other `OTP`\nbehaviour that manages a process. Realistically you'll want a long lived process\nto complement Oban's behaviour, which makes a `GenServer` or `GenStateMachine`\nideal.\n\nUpon startup, Oban dynamically injects each plugin into its supervision tree and\npasses a few base options along with any custom configuration for the plugin.\n\nEvery plugin receives these base options:\n\n* `:conf` — An `Oban.Config` struct, which contains all of the options provided\n  to `Oban.start_link/1` in a validated and normalized form.\n* `:name` — A unique via tuple scoped to the current supervision tree. The name\n  may be used later to support helper/interface functions.","ref":"writing_plugins.html"},{"type":"extras","title":"Example Plugin - Writing Plugins","doc":"Let's look at a tiny example plugin to get a feel for how options are passed in\nand how they run. Our plugin will periodically generate a table of counts for\neach `queue` / `state` combination and then print it out. It isn't an amazingly\nuseful plugin, but it demonstrates how to handle options, work with the\n`Oban.Config` struct, and periodically interact with the `oban_jobs` table.\n\n```elixir\ndefmodule MyApp.Plugins.Breakdown do\n  @behaviour Oban.Plugin\n\n  use GenServer\n\n  import Ecto.Query, only: [group_by: 3, select: 3]\n\n  @impl Oban.Plugin\n  def start_link(opts) do\n    name = Keyword.get(opts, :name, __MODULE__)\n\n    GenServer.start_link(__MODULE__, opts, name: name)\n  end\n\n  @impl Oban.Plugin\n  def validate(opts) do\n    Oban.Validation.validate_schema(opts,\n      conf: :any,\n      name: :any,\n      interval: :pos_integer\n    )\n  end\n\n  @impl GenServer\n  def init(opts) do\n    state = Map.new(opts)\n\n    {:ok, schedule_poll(state)}\n  end\n\n  @impl GenServer\n  def handle_info(:poll, %{conf: conf} = state) do\n    breakdown =\n      Oban.Repo.all(\n        conf,\n        Oban.Job\n        |> group_by([j], [j.queue, j.state])\n        |> select([j], {j.queue, j.state, count(j.id)})\n      )\n\n    IO.inspect(breakdown)\n\n    {:noreply, schedule_poll(state)}\n  end\n\n  defp schedule_poll(%{interval: interval} = state) do\n    Process.send_after(self(), :poll, interval)\n\n    state\n  end\nend\n```\n\nThe plugin's `start_link/1` function expects a keyword list with `:name`,\n`:conf`, and `:interval` values. After extracting the `:name` for process\nregistration, it passes the options through to `init/1`. The `init` function\nconverts the keyword list of options into a map for easier access and then\nbegins a polling loop.\n\nEach iteration of the loop will query the `oban_jobs` table and print out a list\nof `{queue, state, count}` tuples like this:\n\n```\n[\n  {\"default\", \"executing\", 8},\n  {\"default\", \"retryable\", 1},\n  {\"default\", \"completed\", 3114},\n  {\"default\", \"discarded\", 1},\n  {\"events\", \"scheduled\", 3},\n  {\"events\", \"executing\", 21},\n  {\"events\", \"retryable\", 2},\n  {\"events\", \"completed\", 1783},\n  {\"events\", \"discarded\", 1},\n]\n```\n\nNow all that is left is to adding the `Breakdown` module to Oban's plugin list:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    {MyApp.Plugins.Breakdown, interval: :timer.seconds(10)}\n  ]\n  ...\n```\n\nIn the configuration we're only providing the `:interval` value. Oban injects\nthe `:name` and `:conf` automatically.","ref":"writing_plugins.html#example-plugin"},{"type":"extras","title":"Calling Interface Functions - Writing Plugins","doc":"Plugins are named dynamically using `via` tuples, which is an effective way to\nmanage process registration for multiple unique Oban instances. However, it\nmakes writing interface functions for plugins a little more complicated. The\nsolution is to make use of the `Oban.Registry` for process discovery.\n\nImagine adding a `pause` interface function to the `Breakdown` plugin we built\nabove:\n\n```elixir\nalias Oban.Registry\n\ndef pause(oban_name \\\\ Oban) do\n  oban_name\n  |> Registry.whereis({:plugin, __MODULE__})\n  |> GenServer.call(:pause)\nend\n```\n\nThe function accepts an `oban_name` argument with a default of `Oban`, which is\nthe default name for an Oban supervision tree. It then calls `whereis/2` with a\n`{:plugin, plugin_name}` tuple and uses the returned `pid` to call the plugin\nprocess.\n\nYou can then call `pause/1` from elsewhere in the application:\n\n```elixir\nMyApp.Plugins.Breakdown.pause()\n\n# or\n\nMyApp.Plugins.Breakdown.pause(MyApp.OtherOban)\n```","ref":"writing_plugins.html#calling-interface-functions"},{"type":"extras","title":"Caveats - Writing Plugins","doc":"Plugins run directly within Oban's supervision tree. A badly behaving plugin,\ne.g. a plugin that crashes repeatedly, may bring down the entire supervision\ntree. Be sure that your plugin has safety mechanisms in place to prevent\nrepeated crashes during startup.","ref":"writing_plugins.html#caveats"},{"type":"extras","title":"Release Configuration","doc":"# Release Configuration\n\nWhile having the same Oban configuration for every environment might be fine,\nthere are certainly times you might want to make changes for a specific\nenvironment. For example, you may want to increase or decrease a queue's\nconcurrency.","ref":"release_configuration.html"},{"type":"extras","title":"Using Config Providers - Release Configuration","doc":"If you are using Elixir Releases, this is straight forward to do using [Config \nProviders][mcp]:\n\n\n```elixir\ndefmodule MyApp.ConfigProvider do\n  @moduledoc \"\"\"\n  Provide release configuration for Oban Queue Concurrency\n  \"\"\"\n\n  @behaviour Config.Provider\n\n  def init(path) when is_binary(path), do: path\n\n  def load(config, path) do\n    case parse_json(path) do\n      nil ->\n        config\n\n      queues ->\n        Config.Reader.merge(config, ingestion: [{Oban, [queues: queues]}])\n    end\n  end\n\n  defp parse_json(path) do\n    if File.exists?(path) do\n      path\n      |> File.read!()\n      |> JSON.decode!()\n      |> Map.fetch!(\"queues\")\n      |> Keyword.new(fn {key, value} -> {String.to_atom(key), value} end)\n    end\n  end\nend\n```\n\nOur config provider ensures that the `Jason` app is loaded so that we can parse\na `JSON` configuration file. Once the JSON is loaded we must extract the\n`queues` map and convert it to a keyword list where all of the keys are atoms.\nThe use of `String.to_atom/1` is safe because all of our queue names are\nalready defined.\n\nThen you include this in your `mix.exs` file, where your release is configured:\n\n```elixir\nreleases: [\n  umbrella_app: [\n    version: \"0.0.1\",\n    applications: [\n      child_app: :permanent\n    ],\n    config_providers: [{Path.To.ConfigProvider, \"/etc/config.json\"}]\n  ]\n]\n```\n\nThen when you release your app, you ensure that you have a JSON file mounted at\nwhatever path you specified above and that it contains all of your desired queues:\n\n```json\n{\"queues\": {\"special\": 1, \"default\": 10, \"events\": 20}}\n```\n\n[mcp]: https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-config-providers","ref":"release_configuration.html#using-config-providers"},{"type":"extras","title":"Scaling Applications","doc":"# Scaling Applications","ref":"scaling.html"},{"type":"extras","title":"Notifications - Scaling Applications","doc":"Oban uses PubSub notifications for communication between nodes, like job inserts, pausing queues,\nresuming queues, and metrics for Web. The default notifier is `Oban.Notifiers.Postgres`, which\nsends all messages through the database. This provides transactional consistency, but Postgres'\nnotifications adds up at scale because each one requires a separate query.\n\nIf you're clustered, consider switching to an alternative notifier like `Oban.Notifiers.PG`. That\nkeeps notifications out of the db, reduces total queries, and allows larger messages, with the\ntradeoff that notifications from within a database transaction may be sent even if the transaction\nis rolled back. As long as you have a functional Distributed Erlang cluster, switching is a\nsingle line change to your Oban config.\n\n```diff\n config :my_app, Oban,\n+  notifier: Oban.Notifiers.PG,\n```\n\nIf you're not clustered, consider using [`Oban.Notifiers.Phoenix`][onp] to send notifications\nthrough an alternative service like Redis.\n\n[onp]: https://github.com/sorentwo/oban_notifiers_phoenix","ref":"scaling.html#notifications"},{"type":"extras","title":"Triggers - Scaling Applications","doc":"Inserting jobs emits a trigger notification to let queues know there are jobs to process\nimmediately, without waiting up to 1s for the next polling interval. Triggers may create many\nnotifications for active queues.\n\nEvaluate if you need sub-second job dispatch. Without it, jobs may wait up to 1s before running,\nbut that’s not a concern for busy queues since they’re constantly fetching and dispatching.\n\nDisable triggers in your Oban configuration:\n\n```diff\n config :my_app, Oban,\n+  insert_trigger: false,\n```","ref":"scaling.html#triggers"},{"type":"extras","title":"Uniqueness - Scaling Applications","doc":"Frequently, people set uniqueness for jobs that don’t really need it. Not you, of course.\nBefore setting uniqueness, ensure the following, in a very checklist type fashion:\n\n1. Evaluate whether it’s necessary for your workload\n2. Always set a `keys` option so that uniqueness isn’t based on the full `args` or `meta`\n3. Avoid setting a `period` at all if possible, use `period: :infinity` instead\n\nIf you're still committed to setting uniquness for your jobs, consider tweaking your\nconfiguration as follows:\n\n```diff\nuse Oban.Worker, unique: [\n-   period: {1, :hour},\n+   period: :infinity,\n+   keys: [:some_key]\n```\n\n> #### 🌟 Pro Uniqueness {: .tip}\n>\n> Oban Pro uses an [alternative mechanism for unique jobs][uniq] that works for bulk inserts, and\n> is designed for speed, correctness, scalability, and simplicity. Uniqueness is enforced and makes insertion entirely safe between processes and nodes, without the load added\n> by multiple queries.\n\n[uniq]: https://oban.pro/docs/pro/1.5.0-rc.4/Oban.Pro.Engines.Smart.html#module-enhanced-unique","ref":"scaling.html#uniqueness"},{"type":"extras","title":"Reindexing - Scaling Applications","doc":"To stop oban_jobs indexes from taking up so much space on disk, use the\n`Oban.Plugins.Reindexer` plugin to rebuild indexes periodically. The Postgres transactional\nmodel applies to indexes as well as tables. That leaves bloat from inserting, updating, and\ndeleting jobs that auto-vacuuming won’t always fix.\n\nThe reindexer rebuilds key indexes on a fixed schedule, concurrently. Concurrent rebuilds are low\nimpact, they don’t lock the table, and they free up space while optimizing indexes.\n\nThe `Oban.Plugins.Reindexer` plugin is part of OSS Oban. It runs every day at midnight by\ndefault, but it accepts a cron-style schedule and you can tweak it to run less frequently.\n\n```diff\nconfig :my_app, Oban,\n   plugins: [\n+   {Oban.Plugins.Reindexer, schedule: \"@weekly\"},\n    …\n   ]\n```","ref":"scaling.html#reindexing"},{"type":"extras","title":"Pruning - Scaling Applications","doc":"Ensuring you are using the `Pruner` plugin, and that you prune _aggressively_. Pruning\nperiodically deletes `completed`, `cancelled`, and `discarded` jobs. Your application\nand database will benefit from keeping the jobs table small. Aim to retain as few jobs\nas necessary for uniqueness and historic introspection.\n\nFor example, to limit historic jobs to 1 day:\n\n```diff\n config :my_app, Oban,\n  plugins: [\n+    {Oban.Plugins.Pruner, max_age: 1_day_in_seconds}\n     …\n   ]\n```\n\nThe default auto vacuum settings are conservative and may fall behind on active tables. Dead\ntuples accumulate until autovacuum proc comes to mark them as cleanable.\n\nLike indexes, the MVCC system only flags rows for deletion later. Then, those rows are deleted\nwhen the auto-vacuum runs. Autovacuum can be tweaked for the oban_jobs table alone.Tune autovacuum\nfor the oban_jobs table.\n\nThe exact scale factor tuning will vary based on total rows, table size, and database load.\n\nBelow is an example of the possible scale factor and threshold:\n\n```diff\nALTER TABLE oban_jobs SET (\n  autovacuum_vacuum_scale_factor = 0,\n  autovacuum_vacuum_threshold = 100\n)\n```\n\n> #### 🌟 Partitioning {: .tip}\n>\n> For _extreme_ load (tens of millions of jobs a day), Oban Pro’s [DynamicPartitioner][dynp] may\n> help. It manages partitioned tables to drop older jobs without any bloat. Dropping tables\n> entirely is instantaneous and leaves zero bloat. Autovacuuming each partition is faster as well.\n\n[dynp]: https://oban.pro/docs/pro/1.5.0-rc.4/Oban.Pro.Plugins.DynamicPartitioner.html","ref":"scaling.html#pruning"},{"type":"extras","title":"Pooling - Scaling Applications","doc":"Oban uses connections from your application Repo’s pool to talk to the database. When that pool\nis busy, it can starve Oban of connections and you’ll see timeout errors. Likewise, if Oban is\nextremely busy (as it should be), it can starve your application of connections. A good solution\nfor this is to set up another pool that’s exclusively for Oban’s internal use. The dedicated\npool isolates Oban’s queries from the rest of the application.\n\nStart by defining a new `ObanRepo`:\n\n```elixir\ndefmodule MyApp.ObanRepo do\n   use Ecto.Repo,\n     adapter: Ecto.Adapters.Postgres,\n     otp_app: :my_app\nend\n```\n\nThen add a function to your main Repo\n\n```diff\ndefmodule MyApp.Repo do\n  use Ecto.Repo,\n     adapter: Ecto.Adapters.Postgres,\n     otp_app: :my_app\n  \n+ def oban_repo do\n+   if in_transaction?() do\n+     MyApp.Repo\n+   else\n+     MyApp.ObanRepo\n+   end\n+ end\nend\n```\n\nThen switch the configured `repo`, and use `get_dynamic_repo` to ensure the same repo is used\nwithin a transaction:\n\n```diff\n config :my_app, Oban,\n-  repo: MyApp.Repo,\n+  repo: MyApp.ObanRepo,\n+  get_dynamic_repo: {MyApp.Repo, :oban_repo, []}\n   ...\n```","ref":"scaling.html#pooling"},{"type":"extras","title":"High Concurrency - Scaling Applications","doc":"In a busy system with high concurrency all of the record keeping after jobs run causes pool\ncontention, despite the individual queries being very quick. Fetching jobs uses a single query\nper queue. However, acking when a job finishes takes a single connection for each job.\n\nImprove the ratio between executing jobs and available connections by scaling up your Ecto\n`pool_size` and minimizing concurrency between all queues.\n\n```diff\nconfig :my_app, Repo,\n-  pool_size: 10,\n+  pool_size: 50,\n   …\n\n config :my_app, Oban,\n   queues: [\n-    events: 200,\n+    events: 50,\n-    emails: 100,\n+    emails: 25,\n   …\n```\n\nUsing a dedicated pool with a known number of constant connections can also help the ratio. It’s\nnot necessary for most applications, but a dedicated database can help maintain predictable\nperformance.","ref":"scaling.html#high-concurrency"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting","ref":"troubleshooting.html"},{"type":"extras","title":"Jobs Stuck Executing Forever - Troubleshooting","doc":"During deployment or unexpected node restarts, jobs may be left in an `executing` state\nindefinitely. We call these jobs \"orphans\", but orphaning isn't a bad thing. It means that the job\nwasn't lost and it may be retried again when the system comes back online.\n\nThere are two mechanisms to mitigate orphans:\n\n1. Increase the [shutdown_grace_period](Oban.html#start_link/1-twiddly-options) to allow the\n   system more time to finish executing before shutdown. During shutdown each queue stops\n   fetching more jobs, but executing jobs have up to the grace period to complete. The default\n   value is `15000ms`, or 15 seconds.\n\n2. Use the [Lifeline plugin](Oban.Plugins.Lifeline.html) to automatically move those jobs back to\n   available so they can run again.\n\n```elixir\nconfig :my_app, Oban,\n    plugins: [Oban.Plugins.Lifeline],\n    shutdown_grace_period: :timer.seconds(60),\n    ...\n```","ref":"troubleshooting.html#jobs-stuck-executing-forever"},{"type":"extras","title":"Jobs or Plugins aren't Running - Troubleshooting","doc":"Sometimes `Cron` or `Pruner` plugins appear to stop working unexpectedly. Typically, this happens in\nsystems with multi-node setups where \"web\" nodes only enqueue jobs while \"worker\" nodes are\nconfigured to run queues and plugins. Most plugins require leadership to function, so when a \"web\"\nnode becomes leader the plugins go dormant.\n\nThe solution is to disable leadership with `peer: false` on any node that doesn't run plugins:\n\n```elixir\nconfig :my_app, Oban, peer: false, ...\n```","ref":"troubleshooting.html#jobs-or-plugins-aren-t-running"},{"type":"extras","title":"No Notifications with PgBouncer - Troubleshooting","doc":"Using PgBouncer's \"Transaction Pooling\" setup disables all of PostgreSQL's `LISTEN` and `NOTIFY`\nactivity. Some functionality, such as triggering job execution, scaling queues, canceling jobs,\netc. rely on those notifications.\n\nThere are several options available to ensure functional notifications:\n\n1. Switch to the `Oban.Notifiers.PG` notifier. This alternative notifier relies on Distributed\n   Erlang and exchanges messages within a cluster. The only drawback to the PG notifier is that it\n   doesn't trigger job insertion events.\n\n2. Switch `PgBouncer` to \"Session Pooling\". Session pooling isn't as resource efficient as\n   transaction pooling, but it retains all Postgres functionality.\n\n3. Use a dedicated Repo that connects directly to the database, bypassing `PgBouncer`.\n\nIf none of those options work, the [Stager][stag] will switch to `local` polling mode to ensure\nthat queues keep processing jobs.\n\n[stag]: Oban.Plugins.Stager.html","ref":"troubleshooting.html#no-notifications-with-pgbouncer"},{"type":"extras","title":"Unexpectedly Re-running All Migrations - Troubleshooting","doc":"Without a version comment on the `oban_jobs` table, it will rerun all of the migrations. This can\nhappen when comments are stripped when restoring from a backup, most commonly during a transition\nfrom one database to another.\n\nThe fix is to set the latest migrated version as a comment. To start, search through your previous\nmigrations and find the last time you ran an Oban migration. Once you've found the latest version,\ne.g. `version: 10`, then you can set that as a comment on the `oban_jobs` table:\n\n```sql\nCOMMENT ON TABLE public.oban_jobs IS '10'\"\n```\n\nOnce the comment is in place only the migrations from that version onward will\nrun.","ref":"troubleshooting.html#unexpectedly-re-running-all-migrations"},{"type":"extras","title":"Recursive Jobs","doc":"# Recursive Jobs\n\nRecursive jobs, like recursive functions, call themselves after they have\nexecuted. Except unlike recursive functions, where recursion happens in a tight\nloop, a recursive job enqueues a new version of itself and may add a slight\ndelay to alleviate pressure on the queue.\n\nRecursive jobs are a great way to backfill large amounts of data where a\ndatabase migration or a mix task may not be suitable. Here are a few reasons\nthat a recursive job may be better suited for backfilling data:\n\n* Data can't be backfilled with a database migration, it may require talking to\n  an external service\n* A task may fail partway through execution; resuming the task would mean\n  starting over again, or tracking progress manually to resume where the failure\n  occurred\n* A task may be computationally intensive or put heavy pressure on the database\n* A task may run for too long and would be interrupted by code releases or other\n  node restarts\n* A task may interface with an external service and require some rate limiting\n* A job can be used directly for new records _and_ to backfill existing records\n\nLet's explore recursive jobs with a use case that builds on several of those\nreasons.","ref":"recursive-jobs.html"},{"type":"extras","title":"Use Case: Backfilling Timezone Data - Recursive Jobs","doc":"Consider a worker that queries an external service to determine what timezone a\nuser resides in. The external service has a rate limit and the response time is\nunpredictable. We have a lot of users in our database missing timezone\ninformation, and we need to backfill.\n\nOur application has an existing `TimezoneWorker` that accepts a user's `id`,\nmakes an external request and then updates the user's timezone. We can modify\nthe worker to handle backfilling by adding a new clause to `perform/1`. The new\nclause explicitly checks for a `backfill` argument and will enqueue the next job\nafter it executes:\n\n```elixir\ndefmodule MyApp.Workers.TimezoneWorker do\n  use Oban.Worker\n\n  import Ecto.Query\n\n  alias MyApp.{Repo, User}\n\n  @backfill_delay 1\n\n  @impl true\n  def perform(%{args: %{\"id\" => id, \"backfill\" => true}}) do\n    with :ok <- perform(%{args: %{\"id\" => id}}) do\n      case fetch_next(id) do\n        next_id when is_integer(next_id) ->\n          %{id: next_id, backfill: true}\n          |> new(schedule_in: @backfill_delay)\n          |> Oban.insert()\n\n        nil ->\n          :ok\n      end\n    end\n  end\n\n  def perform(%{args: %{\"id\" => id}}) do\n    update_timezone(id)\n  end\n\n  defp fetch_next(current_id) do\n    User\n    |> where([u], is_nil(u.timezone))\n    |> where([u], u.id > ^current_id)\n    |> order_by(asc: :id)\n    |> limit(1)\n    |> select([u], u.id)\n    |> Repo.one()\n  end\n\n  defp update_timezone(_id), do: Enum.random([:ok, {:error, :reason}])\nend\n```\n\nThere is a lot happening in the worker module, so let's unpack it a little bit.\n\n1. There are two clauses for `perform/1`, the first only matches when a job is\n   marked as `\"backfill\" => true`, the second does the actual work of updating the\n   timezone.\n2. The backfill clause checks that the timezone update succeeds and then uses\n   `fetch_next/1` to look for the id of the next user without a timezone.\n3. When another user needing a backfill is available it enqueues a new backfill\n   job with a one second delay.\n\nWith the new `perform/1` clause in place and our code deployed we can kick off\nthe recursive backfill. Assuming the `id` of the first user is `1`, you can\nstart the job from an `iex` console:\n\n```elixir\niex> %{id: 1, backfill: true} |> MyApp.Workers.TimezoneWorker.new() |> Oban.insert()\n```\n\nNow the jobs will chug along at a steady rate of one per second until the\nbackfill is complete (or something fails). If there are any errors the backfill\nwill pause until the failing job completes: especially useful for jobs relying\non flaky external services. Finally, when there aren't any more user's without a\ntimezone, the backfill is complete and recursion will stop.","ref":"recursive-jobs.html#use-case-backfilling-timezone-data"},{"type":"extras","title":"Building On Recursive Jobs - Recursive Jobs","doc":"This was a relatively simple example, and hopefully it illustrates the power and\nflexibility of recursive jobs. Recursive jobs are a general pattern and aren't\nspecific to Oban. In fact, aside from the `use Oban.Worker` directive there\nisn't anything specific to Oban in the recipe!","ref":"recursive-jobs.html#building-on-recursive-jobs"},{"type":"extras","title":"Reliable Scheduled Jobs","doc":"# Reliable Scheduled Jobs\n\nA common variant of recursive jobs are \"scheduled jobs\", where the goal is for a\njob to repeat indefinitely with a fixed amount of time between executions. The\npart that makes it \"reliable\" is the guarantee that we'll keep retrying the\njob's business logic when the job retries, but we'll **only schedule the next\noccurrence once**. In order to achieve this guarantee we'll make use of the\n`perform` function to receive a complete `Oban.Job` struct.\n\nTime for illustrative example!","ref":"reliable-scheduling.html"},{"type":"extras","title":"Use Case: Delivering Daily Digest Emails - Reliable Scheduled Jobs","doc":"When a new user signs up to use our site we need to start sending them daily\ndigest emails. We want to deliver the emails around the same time a user signed\nup, repeating every 24 hours. It is important that we don't spam them with\nduplicate emails, so we ensure that the next email is only scheduled on our\nfirst attempt.\n\n```elixir\ndefmodule MyApp.Workers.ScheduledWorker do\n  use Oban.Worker, queue: :scheduled, max_attempts: 10\n\n  alias MyApp.Mailer\n\n  @one_day 60 * 60 * 24\n\n  @impl true\n  def perform(%{args: %{\"email\" => email} = args, attempt: 1}) do\n    args\n    |> new(schedule_in: @one_day)\n    |> Oban.insert!()\n\n    Mailer.deliver_email(email)\n  end\n\n  def perform(%{args: %{\"email\" => email}}) do\n    Mailer.deliver_email(email)\n  end\nend\n```\n\nYou'll notice that the first `perform/1` clause only matches a job struct on the\nfirst attempt. When it matches, the first clause schedules the next iteration\nimmediately, _before_ attempting to deliver the email. Any subsequent retries\nfall through to the second `perform/1` clause, which only attempts to deliver\nthe email again. Combined, the clauses get us close to **at-most-once semantics\nfor scheduling**, and **at-least-once semantics for delivery**.","ref":"reliable-scheduling.html#use-case-delivering-daily-digest-emails"},{"type":"extras","title":"More Flexible Than CRON Scheduling - Reliable Scheduled Jobs","doc":"Delivering around the same time using cron-style scheduling would need extra\nbook-keeping to check when a user signed up, and then only deliver to those\nusers that signed up within that window of time. The recursive scheduling\napproach is more accurate and entirely self contained—when and if the digest\ninterval changes the scheduling will pick it up automatically once our code\ndeploys.\n\n_An [extensive discussion][oi27] on the Oban issue tracker prompted this example\nalong with the underlying feature that made it possible._\n\n[oi27]: https://github.com/oban-bg/oban/issues/27","ref":"reliable-scheduling.html#more-flexible-than-cron-scheduling"},{"type":"extras","title":"Considerations for Scheduling Jobs in the Very-Near-Future - Reliable Scheduled Jobs","doc":"If you use the `schedule_in` or `scheduled_at` options with a value that will\nresolve to the very-near-future, for example:\n\n```elixir\n# 1 second from now\n%{}\n|> new(schedule_in: 1)\n|> Oban.insert()\n\n# 500 milliseconds from now\nvery_soon = DateTime.utc_now() |> DateTime.add(500, :millisecond)\n\n%{}\n|> new(scheduled_at: very_soon)\n|> Oban.insert()\n```\n\nyour workers may not be aware of/attempt to perform the job until the next tick\nas specific by the [Oban Stager][stag] `:interval` option. By default this is\nset to `1_000ms`.\n\n**Be aware:** Configuring the `:interval` option below the recommended default\ncan have a considerable impact on database performance! It is not advised to\nlower this value and should only be done as a last resort after considering\nother ways to achieve your desired outcome.\n\n[stag]: Oban.Plugins.Stager.html","ref":"reliable-scheduling.html#considerations-for-scheduling-jobs-in-the-very-near-future"},{"type":"extras","title":"Reporting Job Progress","doc":"# Reporting Job Progress\n\nMost applications provide some way to generate an artifact—something that may\ntake the server a long time to accomplish. If it takes several minutes to render\na video, crunch some numbers or generate an export, users may be left wondering\nwhether your application is working. Providing periodic updates to end users\nassures them that the work is being done and keeps the application feeling\nresponsive.\n\nReporting progress is something that any background job processor with\n_unlimited execution time_ can do! Naturally, we'll look at an example built on\nOban.","ref":"reporting-progress.html"},{"type":"extras","title":"Use Case: Exporting a Large Zip File - Reporting Job Progress","doc":"Users of our site can export a zip of all the files they have uploaded. A _zip_\nfile (no, not a tar, our users don't have neck-beards) is generated on the fly,\nwhen the user requests it. Lazily generating archives is great for our server's\nutilization, but it means that users may wait a while when there are many files.\nFortunately, we know how many files will be included in the zip and we can use\nthat information to send progress reports! We will compute the archive's percent\ncomplete as each file is added and push a message to the user.","ref":"reporting-progress.html#use-case-exporting-a-large-zip-file"},{"type":"extras","title":"Before We Start - Reporting Job Progress","doc":"In the [forum question that prompted this guide][guide] the work was done\nexternally by a port process. Working with ports is well outside the scope of\nthis guide, so I've modified it for the sake of simplicity. The result is\nslightly contrived as it puts both processes within the same module, which isn't\nnecessary if the only goal is to broadcast progress. This guide is ultimately\nabout coordinating processes to report progress from a background job, so that's\nwhat we'll focus on (everything else will be rather [hand-wavy][wavy]).","ref":"reporting-progress.html#before-we-start"},{"type":"extras","title":"Coordinating Processes - Reporting Job Progress","doc":"Our worker, the creatively titled `ZippingWorker`, handles both building the\narchive and reporting progress to the client. Showing the entire module at once\nfelt distracting, so we'll start with only the module definition and the\n`perform/1` function:\n\n```elixir\ndefmodule MyApp.Workers.ZippingWorker do\n  use Oban.Worker, queue: :exports, max_attempts: 1\n\n  alias MyApp.{Endpoint, Zipper}\n\n  def perform(%_{args: %{\"channel\" => channel, \"paths\" => paths}}) do\n    build_zip(paths)\n    await_zip(channel)\n  end\n\n  # ...\nend\n```\n\nThe function accepts an Oban Job with a channel name and a list of file paths,\nwhich it immediately passes on to the private `build_zip/1`:\n\n```elixir\n  defp build_zip(paths) do\n    job_pid = self()\n\n    Task.async(fn ->\n      zip_path = Zipper.new()\n\n      paths\n      |> Enum.with_index(1)\n      |> Enum.each(fn {path, index} ->\n        :ok = Zipper.add_file(zip_path, path)\n        send(job_pid, {:progress, trunc(index / length(paths) * 100)})\n      end)\n\n      send(job_pid, {:complete, zip_path})\n    end)\n  end\n```\n\nThe function grabs the current pid, which belongs to the job, and kicks off an\nasynchronous task to handle the zipping. With a few calls to a fictional\n`Zipper` module the task works through each file path, adding it to the zip.\nAfter adding a file the task sends a `:progress` message with the percent\ncomplete back to the job. Finally, when the zip finishes, the task sends a\n`:complete` message with a path to the archive.\n\nThe asynchronous call spawns a separate process and returns immediately. In\norder for the task to finish building the zip we need to wait on it. Typically\nwe'd use `Task.await/2`, but we'll use a custom receive loop to track the task's\nprogress:\n\n```elixir\n  defp await_zip(channel) do\n    receive do\n      {:progress, percent} ->\n        Endpoint.broadcast(channel, \"zip:progress\", percent)\n        await_zip(channel)\n\n      {:complete, zip_path} ->\n        Endpoint.broadcast(channel, \"zip:complete\", zip_path)\n    after\n      30_000 ->\n        Endpoint.broadcast(channel, \"zip:failed\", \"zipping failed\")\n        raise RuntimeError, \"no progress after 30s\"\n    end\n  end\n```\n\nThe receive loop blocks execution while it waits for `:progress` or `:complete`\nmessages. When a message comes in it broadcasts to the provided channel and the\nclient receives an update (this example uses [Phoenix Channels][chan], but any\nother PubSub type mechanism would work). As a safety mechanism we have an\n`after` clause that will timeout after 30 seconds of inactivity. If the receive\nblock times out we notify the client and raise an error, failing the job.","ref":"reporting-progress.html#coordinating-processes"},{"type":"extras","title":"Made Possible by Unlimited Execution - Reporting Job Progress","doc":"Reporting progress asynchronously works in Oban because anything that blocks a\nworker's `perform/1` function will keep the job executing. Jobs aren't executed\ninside of a transaction, which alleviates any limitations on how long a job can\nrun.\n\nThis technique is suitable for any _single_ long running job where an end user\nis waiting on the results. Consider using [Oban Pro's Batch][batch] jobs if you\nneed to combine _multiple_ jobs into a single output.\n\n[guide]: https://elixirforum.com/t/oban-reliable-and-observable-job-processing/22449/52\n[chan]: https://hexdocs.pm/phoenix/channels.html#content\n[wavy]: https://www.quora.com/When-someone-says-this-explanation-was-hand-wavy-what-does-that-mean\n[batch]: https://oban.pro/docs/pro/Oban.Pro.Batch.html","ref":"reporting-progress.html#made-possible-by-unlimited-execution"},{"type":"extras","title":"Handling Expected Failures","doc":"# Handling Expected Failures\n\nReporting job errors by sending notifications to an external service is\nessential to maintaining application health. While reporting is essential, noisy\nreports for flaky jobs can become a distraction that gets ignored. Sometimes we\n_expect_ that a job will error a few times. That could be because the job relies\non an external service that is flaky, because it is prone to race conditions, or\nbecause the world is a crazy place. Regardless of _why_ a job fails, reporting\nevery failure may be undesirable.","ref":"expected-failures.html"},{"type":"extras","title":"Use Case: Silencing Initial Notifications for Flaky Services - Handling Expected Failures","doc":"One solution for reducing noisy error notifications is to start reporting only\nafter a job has failed several times. Oban uses [Telemetry][tele] to make\nreporting errors and exceptions a simple matter of attaching a handler function.\nIn this example we will extend [Honeybadger][hb] reporting from the\n`Oban.Telemetry` documentation, but account for the number of processing attempts.\n\nTo start, we'll define a `Reportable` [protocol][pro] with a single\n`reportable?/2` function:\n\n```elixir\ndefprotocol MyApp.Reportable do\n  @fallback_to_any true\n  def reportable?(worker, attempt)\nend\n\ndefimpl MyApp.Reportable, for: Any do\n  def reportable?(_worker, _attempt), do: true\nend\n```\n\nThe `Reportable` protocol has a default implementation which always returns\n`true`, meaning it reports all errors. Our application has a `FlakyWorker`\nthat's known to fail a few times before succeeding. We don't want to see a\nreport until after a job has failed three times, so we'll add an implementation\nof `Reportable` within the worker module:\n\n```elixir\ndefmodule MyApp.Workers.FlakyWorker do\n  use Oban.Worker\n\n  defstruct []\n\n  defimpl MyApp.Reportable do\n    @threshold 3\n\n    def reportable?(_worker, attempt), do: attempt > @threshold\n  end\n\n  @impl true\n  def perform(%{args: %{\"email\" => email}}) do\n    MyApp.ExternalService.deliver(email)\n  end\nend\n```\n\nNote that we've also used `defstruct []` to make our worker a viable struct.\nThis is necessary for our protocol to dispatch correctly, as protocols consider\nall modules to be a plain `atom`.\n\nThe final step is to call `reportable?/2` from our application's error reporter,\npassing in the worker module and the attempt number:\n\n```elixir\ndefmodule MyApp.ErrorReporter do\n  alias MyApp.Reportable\n\n  def handle_event(_, _, meta, _) do\n    worker_struct = maybe_get_worker_struct(meta.job.worker)\n\n    if Reportable.reportable?(worker_struct, meta.job.attempt) do\n      context = Map.take(meta.job, [:id, :args, :queue, :worker])\n\n      Honeybadger.notify(meta.reason, context, meta.stacktrace)\n    end\n  end\n\n  def maybe_get_worker_struct(worker) do\n    try do\n      {:ok, module} = Oban.Worker.from_string(worker)\n\n      struct(module)\n    rescue\n      UndefinedFunctionError -> worker\n    end\n  end\nend\n```\n\nAttach the failure handler somewhere in your `application.ex` module:\n\n```elixir\n:telemetry.attach(\"oban-errors\", [:oban, :job, :exception], &ErrorReporter.handle_event/4, nil)\n```\n\nWith the failure handler attached you will start getting error reports **only\nafter the third error**.","ref":"expected-failures.html#use-case-silencing-initial-notifications-for-flaky-services"},{"type":"extras","title":"Giving Time to Recover - Handling Expected Failures","doc":"If a service is especially flaky you may find that Oban's default backoff\nstrategy is too fast. By defining a custom `backoff` function on the\n`FlakyWorker` we can set a linear delay before retries:\n\n```elixir\n# inside of MyApp.Workers.FlakyWorker\n\n@impl true\ndef backoff(%Job{attempt: attempt}) do\n  attempt * 60\nend\n```\n\nNow the first retry is scheduled `60s` later, the second `120s` later, and so on.","ref":"expected-failures.html#giving-time-to-recover"},{"type":"extras","title":"Building Blocks - Handling Expected Failures","doc":"Elixir's powerful primitives of behaviours, protocols and event handling make\nflexible error reporting seamless and extensible. While our `Reportable`\nprotocol only considered the number of attempts, this same mechanism is suitable\nfor filtering by any other `meta` value.\n\n[tele]: https://github.com/beam-telemetry/telemetry\n[hb]: https://www.honeybadger.io/\n[pro]: https://hexdocs.pm/elixir/Protocol.html","ref":"expected-failures.html#building-blocks"},{"type":"extras","title":"Splitting Queues Between Nodes","doc":"# Splitting Queues Between Nodes\n\nRunning every job queue on every node isn't always ideal. Imagine that your\napplication has some CPU intensive jobs that you'd prefer not to run on nodes\nthat serve web requests. Perhaps you start temporary nodes that are only meant\nto _insert_ jobs but should never _execute_ any. Fortunately, we can control\nthis by configuring certain node types, or even single nodes, to **run only a\nsubset of queues**.","ref":"splitting-queues.html"},{"type":"extras","title":"Use Case: Isolating Video Processing Intensive Jobs - Splitting Queues Between Nodes","doc":"One notorious type of CPU intensive work is video processing. When our\napplication is transcoding multiple videos simultaneously it is a _major_ drain\non system resources and may impact response times. To avoid this we can run\ndedicated worker nodes that don't serve any web requests and handle all of the\ntranscoding.\n\nWhile it's possible to separate our system into `web` and `worker` apps within\nan umbrella, that wouldn't allow us to dynamically change queues at runtime.\nLet's look at an **environment variable based method** for dynamically\nconfiguring queues at runtime.\n\nWithin `config.exs` our application is configured to run three queues:\n`default`, `media` and `events`:\n\n```elixir\nconfig :my_app, Oban,\n  repo: MyApp.Repo,\n  queues: [default: 15, media: 10, events: 25]\n```\n\nWe will use an `OBAN_QUEUES` environment variable to override the queues at\nruntime. For illustration purposes the queue parsing all happens within the\napplication module, but it would work equally well in `releases.exs` (or `runtime.exs`).\n\n```elixir\ndefmodule MyApp.Application do\n  @moduledoc false\n\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      MyApp.Repo,\n      MyApp.Endpoint,\n      {Oban, oban_opts()}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n  end\n\n  defp oban_opts do\n    env_queues = System.get_env(\"OBAN_QUEUES\")\n\n    :my_app\n    |> Application.get_env(Oban)\n    |> Keyword.update(:queues, [], &queues(env_queues, &1))\n  end\n\n  defp queues(\"*\", defaults), do: defaults\n  defp queues(nil, defaults), do: defaults\n  defp queues(_, false), do: false\n\n  defp queues(values, _defaults) when is_binary(values) do\n    values\n    |> String.split(\" \", trim: true)\n    |> Enum.map(&String.split(&1, \",\", trim: true))\n    |> Keyword.new(fn [queue, limit] ->\n      {String.to_existing_atom(queue), String.to_integer(limit)}\n    end)\n  end\nend\n```\n\nThe `queues` function's first three clauses ensure that we can fall back to the\nqueues specified in our configuration (or `false`, for testing). The fourth\nclause is much more involved, and that is where the environment parsing happens.\nIt expects the `OBAN_QUEUES` value to be a string formatted as `queue,limit`\npairs and separated by spaces. For example, to run only the `default` and\n`media` queues with a limit of 5 and 10 respectively, you would pass the string\n`default,5 media,10`.\n\nNote that the parsing clause has a couple of safety mechanisms to ensure that\nonly real queues are specified:\n\n1. It automatically trims while splitting values, so extra whitespace won't\n   break parsing (i.e. ` default,3 `)\n2. It only converts the `queue` string to _an existing atom_, hopefully\n   preventing typos that would start a random queue (i.e. `default`)","ref":"splitting-queues.html#use-case-isolating-video-processing-intensive-jobs"},{"type":"extras","title":"Usage Examples - Splitting Queues Between Nodes","doc":"In development (or when using `mix` rather than releases) we can specify the\nenvironment variable inline:\n\n```bash\nOBAN_QUEUES=\"default,10 media,5\" mix phx.server # default: 10, media: 5\n```\n\nWe can also explicitly opt in to running all of the configured queues:\n\n```bash\nOBAN_QUEUES=\"*\" mix phx.server # default: 15, media: 10, events: 25\n```\n\nFinally, without `OBAN_QUEUES` set at all it will implicitly fall back to the\nconfigured queues:\n\n```bash\nmix phx.server # default: 15, media: 10, events: 25\n```","ref":"splitting-queues.html#usage-examples"},{"type":"extras","title":"Flexible Across all Environments - Splitting Queues Between Nodes","doc":"This environment variable based solution is more flexible than running separate\numbrella apps because we can reconfigure at any time. In a limited environment,\nlike staging, we can run _all_ the queues on a single node using the exact same\ncode we use in production. In the future, if other workers start to utilize too\nmuch CPU or RAM we can shift them to the worker node **without any code\nchanges**.\n\n_This guide was [prompted by an inquiry][oi82] on the Oban issue tracker._\n\n[oi82]: https://github.com/oban-bg/oban/issues/82","ref":"splitting-queues.html#flexible-across-all-environments"},{"type":"extras","title":"Migrating from Other Languages","doc":"# Migrating from Other Languages\n\nMigrating background jobs to Elixir is easy with Oban because everything lives in your PostgreSQL\ndatabase. Oban relies on a structured `oban_jobs` table as its job queue, and purposefully uses\nJSON as a portable data structures for serialization. That makes enqueueing jobs into Oban simple for\nany language with a PostgreSQL adapter—no Oban client necessary.","ref":"migrating-from-other-languages.html"},{"type":"extras","title":"Use Case: Inserting Jobs from Rails - Migrating from Other Languages","doc":"It's no secret that Ruby to Elixir is a common migration path for developers and existing\napplications alike. Let's explore how to write an adapter for inserting Oban jobs from a Rails\napplication.\n\nTo start, define a skeletal `ActiveRecord` model with a few conveniences for scheduling jobs:\n\n```ruby\nclass Oban::Job < ApplicationRecord\n  # This column is in use, but not used for the insert workflow.\n  self.ignored_columns = %w[errors]\n\n  # A simple wrapper around `create` that ensures the job is scheduled immediately.\n  def self.insert(worker:, args: {}, queue: \"default\", scheduled_at: nil)\n    create(\n      worker: worker,\n      queue: queue,\n      args: args,\n      scheduled_at: scheduled_at || Time.now.utc,\n      state: scheduled_at ? \"scheduled\" : \"available\"\n    )\n  end\nend\n```\n\nThe `insert` class method is a convenience that uses named arguments to force passing a `worker`\nwhile providing some defaults. The only semi-magical thing within `insert` is determining the\ncorrect state for scheduled jobs. In Oban, jobs that are ready to execute have an `available`\nstate, while jobs slated for the future are `scheduled`.\n\nTo insert a single job using the `insert` class method:\n\n```ruby\nOban::Job.insert(worker: \"MyWorker\", args: {id: 1}, queue: \"default\")\n```\n\nProvided your Elixir application has a worker named `MyWorker` and the `default` queue is running,\nOban will pick up and execute the job immediately. To schedule the job to run a minute in the\nfuture instead, pass a `scheduled_at` timestamp:\n\n```ruby\nOban::Job.insert(worker: \"MyWorker\", args: {id: 1}, scheduled_at: 1.minute.from_now.utc)\n```\n\nNow, if you're using Rails 6+, you can also use `insert_all` to batch insert jobs:\n\n```ruby\nOban::Job.insert_all([\n  {worker: \"MyWorker\", args: {id: 1}, queue: \"default\"},\n  {worker: \"MyWorker\", args: {id: 2}, queue: \"default\"},\n  {worker: \"MyWorker\", args: {id: 3}, queue: \"default\"},\n])\n```","ref":"migrating-from-other-languages.html#use-case-inserting-jobs-from-rails"},{"type":"extras","title":"Safety Guaranteed - Migrating from Other Languages","doc":"Most columns in `oban_jobs` have sensible defaults, so only the `worker` and `args` are typically\nrequired. For integrity, all required columns are marked as `NON NULL`, and several have `CHECK`\nconstraints as well for extra enforcement.\n\nThat's all you need to start migrating background jobs from Rails to Elixir (if you're using Oban,\nthat is). Naturally, the same pattern would work for Python, Node, PHP, or any other language with\na Postgres adapter.","ref":"migrating-from-other-languages.html#safety-guaranteed"},{"type":"extras","title":"Introduction to Testing","doc":"# Introduction to Testing\n\nAutomated testing is an essential component of building reliable, long-lived\napplications. Oban orchestrates your application's background tasks, so naturally,\ntesting Oban is highly recommended.","ref":"testing.html"},{"type":"extras","title":"Setup Application Config - Introduction to Testing","doc":"Ensure your app is configured for testing before you begin running tests.\n\nThere are two testing modes available:\n\n* `:inline`—jobs execute immediately within the calling process and without\n  touching the database. This mode is simple and may not be suitable for apps\n  with complex jobs.\n* `:manual`—jobs are inserted into the database where they can be verified and\n  executed when desired. This mode is more advanced and trades simplicity for\n  flexibility.\n\nIf you're just starting out, `:inline` mode is recommended:\n\n```elixir\nconfig :my_app, Oban, testing: :inline\n```\n\nFor more complex applications, or if you'd like complete control over when jobs\nrun, then use `:manual` mode instead:\n\n```elixir\nconfig :my_app, Oban, testing: :manual\n```\n\nBoth testing modes prevent Oban from running any database queries in the\nbackground. This simultaneously prevents Sandbox errors from plugin queries and\nprevents queues from executing jobs unexpectedly.","ref":"testing.html#setup-application-config"},{"type":"extras","title":"Changing Testing Modes - Introduction to Testing","doc":"Once the application starts in a particular testing mode it can't be changed.\nThat's inconvenient if you're running in `:inline` mode and don't want a\nparticular job to execute inline! `Oban.Testing` provides a helper to\ntemporarily change testing modes within the context of a function.\n\nFor example, to switch to `:manual` mode when Oban is configured for `:inline`\ntesting:\n\n```elixir\nOban.Testing.with_testing_mode(:manual, fn ->\n  Oban.insert(MyWorker.new(%{id: 123}))\n\n  assert_enqueued worker: MyWorker, args: %{id: 123}\nend)\n```\n\nOr vice-versa, switch to `:inline` mode when the application is configured for\n`:manual` mode:\n\n```elixir\nOban.Testing.with_testing_mode(:inline, fn ->\n  {:ok, %Job{state: \"completed\"}} = Oban.insert(MyWorker.new(%{id: 123}))\nend)\n```","ref":"testing.html#changing-testing-modes"},{"type":"extras","title":"Setup Testing Helpers - Introduction to Testing","doc":"Oban provides helpers to facilitate manual testing. These helpers handle the\nboilerplate of making assertions on which jobs are enqueued.\n\nThe most convenient way to use the helpers is to `use` the module within your\ntest case:\n\n```elixir\ndefmodule MyApp.Case do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      use Oban.Testing, repo: MyApp.Repo\n    end\n  end\nend\n```\n\nAlternatively, you can `use` the testing module in individual tests if you'd\nprefer not to include helpers in _every_ test.\n\n```elixir\ndefmodule MyApp.WorkerTest do\n  use MyApp.Case, async: true\n\n  use Oban.Testing, repo: MyApp.Repo\nend\n```\n\nWhichever way you choose, using `use Oban.Testing` requires the `repo` option\nbecause it's injected into many of the generated helpers.\n\nIf you are using isolation with namespaces through PostgreSQL schemas (Ecto\n\"prefixes\"), you can also specify this prefix when using `Oban.Testing`:\n\n```elixir\nuse Oban.Testing, repo: MyApp.Repo, prefix: \"private\"\n```\n\nWith Oban configured for testing and helpers in the appropriate places, you're\nready for testing. Learn about unit testing with [Testing Workers][tw],\nintegration testing with [Testing Queues][tq], or prepare for production with\n[Testing Config][tc].\n\n[tw]: testing_workers.html\n[tq]: testing_queues.html\n[tc]: testing_config.html","ref":"testing.html#setup-testing-helpers"},{"type":"extras","title":"Testing Workers","doc":"# Testing Workers\n\nWorker modules are the primary \"unit\" of an Oban system. You can (and should)\ntest a worker's callback functions locally, in-process, without touching the\ndatabase.\n\nMost worker callback functions take a single argument: an `Oban.Job` struct. A\njob encapsulates arguments, metadata, and other options. Creating jobs, and\nverifying that they're built correctly, requires some boilerplate...that's where\n`Oban.Testing.perform_job/3` comes in!","ref":"testing_workers.html"},{"type":"extras","title":"Testing Perform - Testing Workers","doc":"The `perform_job/3` helper reduces boilerplate when constructing jobs for unit\ntests and checks for common pitfalls. For example, it automatically converts\n`args` to string keys before calling `perform/1`, ensuring that perform clauses\naren't erroneously trying to match on atom keys.\n\nLet's work through test-driving a worker to demonstrate.\n\nStart by defining a test that creates a user and then use `perform_job` to\nmanually call an account activation worker. In this context \"activation\" could\nmean sending an email, notifying administrators, or any number of\nbusiness-critical functions—what's important is how we're testing it.\n\n```elixir\ndefmodule MyApp.ActivationWorkerTest do\n  use MyApp.Case, async: true\n\n  test \"activating a new user\" do\n    user = MyApp.User.create(email: \"parker@example.com\")\n\n    {:ok, _user} = perform_job(MyApp.ActivationWorker, %{id: user.id})\n  end\nend\n```\n\nRunning the test at this point will raise an error that explains the module\ndoesn't implement the `Oban.Worker` behaviour.\n\n```text\n1) test activating a new account (MyApp.ActivationWorkerTest)\n\n   Expected worker to be a module that implements the Oban.Worker behaviour, got:\n\n   MyApp.ActivationWorker\n\n   code: {:ok, user} = perform_job(MyApp.ActivationWorker, %{id: user.id})\n```\n\nTo fix it, define a worker module with the appropriate signature and return\nvalue:\n\n```elixir\ndefmodule MyApp.ActivationWorker do\n  use Oban.Worker\n\n  @impl Worker\n  def perform(%Job{args: %{\"id\" => user_id}}) do\n    MyApp.Account.activate(user_id)\n  end\nend\n```\n\nThe `perform_job/3` helper's errors will guide you through implementing a\ncomplete worker with the following assertions:\n\n* That the worker implements the `Oban.Worker` behaviour\n* That `args` is encodable/decodable to JSON and always has string keys\n* That the options provided build a valid job\n* That the return value is expected, e.g. `:ok`, `{:ok, value}`, `{:error, value}` etc.\n* That a custom `new/1,2` callback works properly\n\nIf all of the assertions pass, then you'll get the result of `perform/1` for you\nto make additional assertions on.","ref":"testing_workers.html#testing-perform"},{"type":"extras","title":"Testing Other Callbacks - Testing Workers","doc":"You may wish to test less-frequently used worker callbacks such as `backoff/1`\nand `timeout/1`, but those callbacks don't have dedicated testing helpers.\nNever fear, it's adequate to build a job struct and test callbacks directly!\n\nHere's a sample test that asserts the backoff value is simply two-times the\njob's `attempt`:\n\n```elixir\ntest \"calculating custom backoff as a multiple of job attempts\" do\n  assert 2 == MyWorker.backoff(%Oban.Job{attempt: 1})\n  assert 4 == MyWorker.backoff(%Oban.Job{attempt: 2})\n  assert 6 == MyWorker.backoff(%Oban.Job{attempt: 3})\nend\n```\n\nSimilarly, here's a sample that verifies a `timeout/1` callback always returns\nsome number of milliseconds:\n\n```elixir\ntest \"allowing a multiple of the attempt as job timeout\" do\n  assert 1000 == MyWorker.timeout(%Oban.Job{attempt: 1})\n  assert 2000 == MyWorker.timeout(%Oban.Job{attempt: 2})\nend\n```\n\nJobs are Ecto schemas, and therefore structs. There isn't anything magical about\nthem! Explore the `Oban.Job` documentation to see all of the types and fields\navailable for testing.","ref":"testing_workers.html#testing-other-callbacks"},{"type":"extras","title":"Testing Queues","doc":"# Testing Queues\n\nWhere workers are the primary \"unit\" of an Oban system, queues are the \"integration\" point between\nthe database and your application. That means to test queues and the jobs within them, your tests\nwill have to interact with the database. To simplify that interaction, reduce boilerplate, and\nmake assertions more expressive `Oban.Testing` provides a variety of helpers.","ref":"testing_queues.html"},{"type":"extras","title":"Asserting Enqueued Jobs - Testing Queues","doc":"During test runs you don't typically want to execute jobs. Rather, you need to verify that the job\nwas enqueued properly. With the recommended test setup queues and plugins are disabled, and jobs\nwon't be inserted into the database at all. Instead, they'll be executed immediately within the\ncalling process. The `Oban.Testing.assert_enqueued/2` and `Oban.Testing.refute_enqueued/2` helpers\nsimplify running queries to check for those `available` or `scheduled` jobs sitting in the\ndatabase.\n\nLet's look at an example where we want to check that an activation job is enqueued after a user\nsigns up:\n\n```elixir\ntest \"scheduling activation upon sign up\" do\n  {:ok, account} = MyApp.Account.sign_up(email: \"parker@example.com\")\n\n  assert_enqueued worker: MyApp.ActivationWorker, args: %{id: account.id}, queue: :default\nend\n```\n\nIt's also possible to assert that job `args` or `meta` have a particular shape, without matching\nexact values:\n\n```elixir\ntest \"enqueued args have a particular key\" do\n  :ok = MyApp.Account.notify_owners(account())\n\n  assert_enqueued queue: :default, args: %{email: _}\nend\n```\n\nYou can also refute that a job was enqueued. The `refute_enqueued` helper takes the same arguments\nas `assert_enqueued`, though you should take care to be as unspecific as possible.\n\nBuilding on the example above, let's refute that a job is enqueued when account sign up fails:\n\n```elixir\ntest \"bypassing activation when sign up fails\" do\n  {:error, _reason} = MyApp.Account.sign_up(email: \"parker@example.com\")\n\n  refute_enqueued worker: MyApp.ActivationWorker\nend\n```","ref":"testing_queues.html#asserting-enqueued-jobs"},{"type":"extras","title":"Asserting Multiple Jobs - Testing Queues","doc":"Asserting and refuting about a single job isn't always enough. Sometimes you need to check for\nmultiple jobs at once, or perform more complex assertions on the jobs themselves. In that\nsituation, you can use `all_enqueued` instead.\n\nThe first example we'll look at asserts that multiple jobs from the same worker are enqueued all\nat once:\n\n```elixir\ntest \"enqueuing one job for each child record\" do\n  :ok = MyApp.Account.notify_owners(account())\n\n  assert jobs = all_enqueued(worker: MyApp.NotificationWorker)\n  assert 3 == length(jobs)\nend\n```\n\nThe `enqueued` helpers all build dynamic queries to check for jobs within the\ndatabase. Dynamic queries don't work for complex objects with nested values or a\npartial set of keys. In that case, you can use `all_enqueued` to pull jobs into\nyour tests and use the full power of pattern matching for assertions.\n\n```elixir\ntest \"enqueued jobs have args that match a particular pattern\" do\n  :ok = MyApp.Account.notify_owners(account())\n\n  for job <- all_enqueued(queue: :default) do\n    assert %{\"email\" => _, \"avatar\" => %{\"url\" => _}} = job.args\n  end\nend\n```","ref":"testing_queues.html#asserting-multiple-jobs"},{"type":"extras","title":"Integration Testing Queues - Testing Queues","doc":"During integration tests it may be necessary to run jobs because they do work essential for the\ntest to complete, i.e. sending an email, processing media, etc. You can execute all available jobs\nin a particular queue by calling `Oban.drain_queue/1,2` directly from your tests.\n\nFor example, to process all pending jobs in the \"mailer\" queue while testing some business logic:\n\n```elixir\ndefmodule MyApp.BusinessTest do\n  use MyApp.DataCase, async: true\n\n  alias MyApp.{Business, Worker}\n\n  test \"we stay in the business of doing business\" do\n    :ok = Business.schedule_a_meeting(%{email: \"monty@brewster.com\"})\n\n    assert %{success: 1, failure: 0} = Oban.drain_queue(queue: :mailer)\n\n    # Now, make an assertion about the email delivery\n  end\nend\n```\n\nSee `Oban.drain_queue/1,2` for a myriad of options and additional details.","ref":"testing_queues.html#integration-testing-queues"},{"type":"extras","title":"Testing Config","doc":"# Testing Config\n\nAn Oban instance's config is built from options passed to `Oban.start_link`\nduring initialization and it determines the supervision tree that Oban builds.\nThe instance config is encapsulated by an `Oban.Config` struct, which is then\nreferenced by plugins, queues, and most public functions.","ref":"testing_config.html"},{"type":"extras","title":"Testing Dynamic Config - Testing Config","doc":"Oban automatically normalizes and validates config options on initialization.\nThat covers any static configuration, but it won't help when config is generated\ndynamically, at runtime.\n\nThe `Oban.Config.validate/1` helper is used internally when the config\ninitializes. Along with each top level option, like the `:engine` or `:repo`, it\nrecursively verifies all queue and plugin options.\n\nYou can use `validate/1` to verify dynamic configuration, e.g. options specified\nin `runtime.exs`:\n\n```elixir\ntest \"production oban config is valid\" do\n  config =\n    \"config/config.exs\"\n    |> Config.Reader.read!(env: :prod)\n    |> get_in([:my_app, Oban])\n\n  assert :ok = Oban.Config.validate(config)\nend\n```\n\nWhen the configuration contains **any** invalid options, like an invalid engine,\nyou'll see the test fail with an error like this:\n\n```elixir\n{:error, \"expected :engine to be an Oban.Queue.Engine, got: MyApp.Repo\"}\n```","ref":"testing_config.html#testing-dynamic-config"},{"type":"extras","title":"Testing Dynamic Plugin Config - Testing Config","doc":"Validation is especially helpful for plugins that have complex configuration,\ne.g. `Cron` or the `Dynamic*` plugins from Oban Pro. As of Oban v2.12.0 all\nplugins implement the `c:Oban.Plugin.validate/1` callback and we can test them\nin isolation as well as through the top level config.\n\nSuppose you have a helper function that returns a crontab config at runtime:\n\n```elixir\ndefmodule MyApp.Oban do\n  def cron_config do\n    [crontab: [{\"0 24 * * *\", MyApp.Worker}]]\n  end\nend\n```\n\nYou can call that function within a test and then assert that it is valid with\n`c:Oban.Plugin.validate/1`:\n\n```elixir\ntest \"testing cron plugin configuration\" do\n  config = MyApp.Oban.cron_config()\n\n  assert :ok = Oban.Plugins.Cron.validate(config)\nend\n```\n\nRunning this test will return an error tuple, showing that the cron expression\nisn't valid.\n\n```elixir\n{:error, %ArgumentError{message: \"expression field 24 is out of range 0..23\"}}\n```\n\nSwitch the expression from `0 24 * * *` to `0 23 * * *`, run the tests again,\nand everything passes!","ref":"testing_config.html#testing-dynamic-plugin-config"},{"type":"extras","title":"Upgrading to v2.0","doc":"# Upgrading to v2.0\n\n> This information is extracted and expanded from the [CHANGELOG](changelog.html).\n\nThis is a big release with a _ton_ of new features and fixes, but also a few\nbreaking changes. This guide will walk you through the upgrade process.","ref":"v2-0.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.0","doc":"Update Oban to the latest version:\n\n```elixir\n  defp deps do\n    [\n      {:oban, \"~> 2.0.0\"}\n      ...\n    ]\n  end\n```\n\nIf you're an Oban Web+Pro user you'll also need to bump Oban.Web and add Oban.Pro:\n\n```elixir\n  defp deps do\n    [\n      {:oban, \"~> 2.0.0\"},\n      {:oban_web, \"~> 2.0.0\", organization: \"oban\"},\n      {:oban_pro, \"~> 0.3.0\", organization: \"oban\"}\n      ...\n    ]\n  end\n```","ref":"v2-0.html#bump-your-deps"},{"type":"extras","title":"Oban.Worker Changes - Upgrading to v2.0","doc":"The `perform/2` callback is replaced with `perform/1`, where the only argument\nis an Oban.Job struct. This unifies the interface for all Oban.Worker callbacks\nand helps to eliminate confusion around pattern matching on arguments.\n\nChange all worker definitions from accepting `args` and a `job` to only accept a\njob and match on the nested `args` key:\n\n```diff\n- def perform(%{\"id\" => id}, _job)\n+ def perform(%Job{args: %{\"id\" => id}})\n```\n\nFor finer control of job backoff the `backoff/1` callback now expects a job\nstruct instead of an integer. Change any workers that expect an attempt number\nto match on the full job struct:\n\n```diff\n- def backoff(attempt)\n+ def backoff(%Job{attempt: attempt})\n```","ref":"v2-0.html#oban-worker-changes"},{"type":"extras","title":"Update Your Config - Upgrading to v2.0","doc":"The `:verbose` setting was renamed to `:log` to better align with values\naccepted by `Ecto.Repo`.\n\n```diff\nconfig :my_app, Oban,\n- verbose: false,\n+ log: false\n  ...\n```\n\nPruning is now handled by the new plugin system. Replace `:prune` in your config\nand pass a `:max_age` value to the plugin:\n\n```diff\nconfig :my_app, Oban,\n- prune: {:max_age, 60},\n- prune_interval: 30_000,\n- prune_limit: 5000,\n+ plugins: [{Oban.Plugins.Pruner, max_age: 60}]\n  ...\n```\n\nIn test mode you can disable pruning by setting `plugins` to false instead:\n\n```diff\nconfig :my_app, Oban,\n-  prune: false,\n+  plugins: false\n  ...\n```\n\n🌟 Oban.Pro users may opt to use the [DynamicPruner](dynamic_pruning.html)\ninstead for finer control. For example, to set per-state retention periods:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [{\n    Oban.Pro.Plugins.DynamicPruner,\n    state_overrides: [\n      completed: {:max_age, {1, :day}},\n      discarded: {:max_age, {1, :month}}\n    ]\n  }]\n  ...\n```\n\nPulse tracking and orphaned job rescue are removed from base Oban. This change\nmeans that you will need to manually rescue any jobs left executing state after\na crash or forced shutdown.\n\nRemove any `:beats_maxage`, `:rescue_after` or `:rescue_interval` settings from\nyour config:\n\n```diff\nconfig :my_app, Oban,\n-  beats_maxage: 10,\n-  rescue_after: 30_000,\n-  rescue_interval: 5_000\n  ...\n```\n\n🌟 Oban.Pro users may use the [Lifeline](lifeline.html) plugin to retain\nautomatic orphaned job rescue with lightweight heartbeat recording:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [Oban.Pro.Plugins.Lifeline]\n  ...\n```","ref":"v2-0.html#update-your-config"},{"type":"extras","title":"Update Your Tests - Upgrading to v2.0","doc":"The new `perform_job` test helper automates validating, normalizing and\nperform jobs in unit tests.\n\nTo update your tests replace any calls to `perform` with the new helper:\n\n```diff\n- assert :ok = MyApp.Worker.perform(%{id: 1}, %Oban.Job{})\n+ assert :ok = perform_job(MyApp.Worker, %{id: 1})\n```\n\nThe `perform_job` helper will verify the worker, the arguments and any\nprovided options. It will then verify that your worker returns a valid result\nand return the value for you to assert on.\n\nWithin integration tests replace `drain_queue/3` with `drain_queue/2`:\n\n```diff\n- Oban.drain_queue(:myqueue, with_safety: false)\n+ Oban.drain_queue(queue: :myqueue, with_safety: false)\n```","ref":"v2-0.html#update-your-tests"},{"type":"extras","title":"Update Telemetry - Upgrading to v2.0","doc":"Telemetry event names have changed, along with some of the metadata and timing\nunits. The new event names are consistent and align with the now-standard\n`telemetry:span` conventions.\n\nUpdate handlers to match on the new event names:\n\n```diff\n- def handle_event([:oban, :failure], measure, meta, _) do\n+ def handle_event([:oban, :job, :exception], measure, meta, _) do\n```\n\nAnd update the `attach` calls:\n\n```diff\n- :telemetry.attach(\"oban\", [[:oban, :failure]], &handle_event/4, %{})\n+ :telemetry.attach(\"oban\", [[:oban, :job, :exception]], &handle_event/4, %{})\n```\n\nFinally, replace references to `meta.stack` with `meta.stacktrace`:\n\n```diff\n- Sentry.capture_exception(meta.error, stacktrace: meta.stack, extra: extra)\n+ Sentry.capture_exception(meta.error, stacktrace: meta.stacktrace, extra: extra)\n```\n\nHere is a conversion chart for any other handlers you may have:\n\n```text\n[:oban, :started] -> [:oban, :job, :start]\n[:oban, :success] -> [:oban, :job, :stop]\n[:oban, :failure] -> [:oban, :job, :exception]\n[:oban, :trip_circuit] -> [:oban, :circuit, :trip]\n[:oban, :open_circuit] -> [:oban, :circuit, :open]\n```","ref":"v2-0.html#update-telemetry"},{"type":"extras","title":"Update Oban.Web (Optional) - Upgrading to v2.0","doc":"Oban.Web is streamlined to share configuration with Oban and use the new plugin\nsystem. Remove any Oban.Web specific configuration:\n\n```diff\n- config :my_app, Oban.Web, repo: MyApp.Repo\n```\n\nNext, add the necessary plugins to the Oban config:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    Oban.Pro.Plugins.DynamicPruner,\n    Oban.Pro.Plugins.Lifeline,\n    Oban.Web.Plugins.Stats\n  ],\n  ...\n```\n\nNow remove ObanWeb from your application's supervision tree:\n\n```diff\nchildren = [\n  MyApp.Repo,\n  MyApp.Endpoint,\n  {Oban, oban_opts},\n- {ObanWeb, oban_web_opts}\n]\n```\n\nAnd delete any existing `ObanWeb` migrations:\n\n```diff\n- defdelegate up, to: ObanWeb.Migrations\n- defdelegate down, to: ObanWeb.Migrations\n```\n\nFinally, use the `oban_dashboard` macro to easily mount in your router:\n\n```diff\n+ import Oban.Web.Router\n\nscope \"/\" do\n  pipe_through :browser\n\n-  live \"/oban\", ObanWeb.DashboardLive, layout: {ObanWeb.LayoutView, \"app.html\"}\n+  oban_dashboard(\"/oban\")\nend\n```","ref":"v2-0.html#update-oban-web-optional"},{"type":"extras","title":"Upgrading to v2.6","doc":"# Upgrading to v2.6\n\nFor Oban OSS users the v2.6 upgrade is a drop in replacement—there isn't\nanything to do! However, Web+Pro users will need to make some changes to unlock\nthe goodness of engines.","ref":"v2-6.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.6","doc":"Update Oban, Web, and Pro to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.6\"},\n  {:oban_web, \"~> 2.6\", repo: \"oban\"},\n  {:oban_pro, \"~> 0.7\", repo: \"oban\"}\n  ...\n]\n```\n\nBe sure to specify **both `:oban_web` and `:oban_pro`** if you use them both.\nThere aren't any dependencies between Web and Pro now. That means you're free to\nuse Pro for workers and only include Web for Phoenix servers, etc.","ref":"v2-6.html#bump-your-deps"},{"type":"extras","title":"Switch to the SmartEngine - Upgrading to v2.6","doc":"The `SmartEngine` uses centralized records to track and exchange state globally,\nenabling features such as global concurrency.\n\nFirst, create a migration to add the new `oban_producers` table:\n\n```bash\n$ mix ecto.gen.migration add_oban_producers\n```\n\nWithin the migration module:\n\n```elixir\nuse Ecto.Migration\n\ndefdelegate change, to: Oban.Pro.Migrations.Producers\n```\n\nIf you have multiple Oban instances or use prefixes, you can specify the prefix\nand create multiple tables in one migration:\n\n```elixir\nuse Ecto.Migration\n\ndef change do\n  Oban.Pro.Migrations.Producers.change()\n  Oban.Pro.Migrations.Producers.change(prefix: \"special\")\n  Oban.Pro.Migrations.Producers.change(prefix: \"private\")\nend\n```\n\nNext, update your config to use the `SmartEngine`:\n\n```elixir\nconfig :my_app, Oban,\n  engine: Oban.Pro.Queue.SmartEngine,\n  ...\n```\n\nIf you have multiple Oban instances you need to configure each one to use the\n`SmartEngine`, otherwise they'll default to the `Basic` engine.","ref":"v2-6.html#switch-to-the-smartengine"},{"type":"extras","title":"Start Gossiping - Upgrading to v2.6","doc":"Oban Pro no longer writes heartbeat records to `oban_beats`. Instead, any Oban\ninstance that runs queues must use the `Gossip` plugin to broadcast status via\nPubSub.\n\nTo start, include the `Gossip` plugin in your Oban config:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    Oban.Plugins.Gossip\n    ...\n```\n\nWith the default configuration the plugin will broadcast every 1 second. If that\nis too frequent you can configure the interval:\n\n```elixir\n  plugins: [\n    {Oban.Plugins.Gossip, interval: :timer.seconds(5)}\n    ...\n```","ref":"v2-6.html#start-gossiping"},{"type":"extras","title":"Remove the Workflow Manager - Upgrading to v2.6","doc":"Due to an improvement in how configuration is passed to workers the\n`WorkflowManager` plugin is no longer needed. You can remove it from your list\nof plugins:\n\n```diff\nplugins: [\n  Oban.Pro.Plugins.Lifeline,\n- Oban.Pro.Plugins.WorkflowManager,\n  ...\n```","ref":"v2-6.html#remove-the-workflow-manager"},{"type":"extras","title":"Remove Extra Lifeline Options - Upgrading to v2.6","doc":"The `Lifeline` plugin is simplified and doesn't accept as many configuration\noptions. If you previously configured the `record_interval` or `delete_interval`\nyou can remove them:\n\n```diff\nplugins: [{\n  Oban.Pro.Plugins.Lifeline,\n- delete_interval: :timer.minutes(10),\n- record_interval: :timer.seconds(10),\n  rescue_interval: :timer.minutes(5)\n}]\n```","ref":"v2-6.html#remove-extra-lifeline-options"},{"type":"extras","title":"Drop the Beats Table - Upgrading to v2.6","doc":"Once you've rolled out the switch to producer records, the smart engine and the\ngossip plugin you are free to remove the `oban_beats` table at your discretion\n(preferably in a follow up release, to prevent errors):\n\n```bash\n$ mix ecto.gen.migration drop_oban_beats\n```\n\nWithin the generated migration module:\n\n```elixir\nuse Ecto.Migration\n\ndef up do\n  drop_if_exists table(\"oban_beats\")\n  drop_if_exists table(\"oban_beats\", prefix: \"private\") # If you have any prefixes:\nend\n\ndef down do\n  # No going back!\nend\n```","ref":"v2-6.html#drop-the-beats-table"},{"type":"extras","title":"Upgrading to v2.11","doc":"# Upgrading to v2.11\n\nThis Oban release includes a required migration and a couple of optional, but\nrecommended, changes.","ref":"v2-11.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.11","doc":"Update Oban, Web, and Pro to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.11\"},\n  {:oban_pro, \"~> 0.10\", repo: \"oban\"},\n  {:oban_web, \"~> 2.9\", repo: \"oban\"}\n]\n```","ref":"v2-11.html#bump-your-deps"},{"type":"extras","title":"Run Oban.Migrations for v11 - Upgrading to v2.11","doc":"Oban's new leadership mechanism uses an unlogged table to track state globally.\nThe v11 migration creates a new `oban_peers` table, and is required for\nleadership—without it many plugins won't run.\n\nTo get started, create a migration to create the table:\n\n```bash\n$ mix ecto.gen.migration create_oban_peers\n```\n\nWithin the generated migration module:\n\n```elixir\nuse Ecto.Migration\n\ndef up, do: Oban.Migrations.up(version: 11)\n\ndef down, do: Oban.Migrations.down(version: 11)\n```\n\nIf you have multiple Oban instances or use an alternate prefix you'll need to\nrun the migration for each prefix.\n\nThe `Oban.Peer` module will safely handle a missing `oban_peers` table and log a\nwarning.","ref":"v2-11.html#run-oban-migrations-for-v11"},{"type":"extras","title":"Update Notifier Names - Upgrading to v2.11","doc":"Now that we've pulled the `PG` notifier in from Oban Pro there are a few\nnaming changes you should make.\n\n1. If your config explicitly declares `Oban.PostgresNotifier`, change it to the\n   new namespaced version: `Oban.Notifiers.Postgres`:\n\n```diff\n-notifier: Oban.PostgresNotifier\n+notifier: Oban.Notifiers.Postgres\n```\n\n2. If you are using Pro's `PG` notifier, change it to the new namespaced version\n   from Oban:\n\n```diff\n-notifier: Oban.Pro.Notifiers.PG\n+notifier: Oban.Notifiers.PG\n```","ref":"v2-11.html#update-notifier-names"},{"type":"extras","title":"Check Configuration for Multi-Node Setups - Upgrading to v2.11","doc":"This release introduces centralized leadership through the `Oban.Peer`\nbehaviour. To prevent duplicate plugin work across nodes, only one Oban instance\nwithin a cluster may be the leader. Unfortunately, if a node that doesn't run\nplugins becomes the leader then jobs may get stuck as `available` and plugins\nlike `Cron` or `Pruner` won't run.\n\nThe simplest solution is avoid `plugins: false` altogether:\n\n```diff\n-plugins: false\n+plugins: []\n```\n\nSee the [Troubleshooting guide](./troubleshooting.md) for more context.","ref":"v2-11.html#check-configuration-for-multi-node-setups"},{"type":"extras","title":"Swap the Compound Index (Optional, but Recommended) - Upgrading to v2.11","doc":"Oban uses a single compound index for most queries. The index is comprised of\njob `state`, `queue`, `priority`, `scheduled_at`, and `id`. That single index is\nflexible enough to power most of Oban's queries. However, the column order is\nimportant, and the order created by Oban's migrations isn't optimal in all\nsituations.\n\nIf you're experiencing slow plugin queries, e.g. the `Stager`, then you may\nbenefit from swapping the indexes. To do so, create a migration:\n\n```bash\n$ mix ecto.gen.migration swap_primary_oban_indexes\n```\n\nWithin the generated migration module:\n\n```elixir\n@disable_ddl_transaction true\n@disable_migration_lock true\n\ndef change do\n  create_if_not_exists index(\n    :oban_jobs,\n    [:state, :queue, :priority, :scheduled_at, :id],\n    concurrently: true,\n    prefix: \"public\"\n  )\n\n  drop_if_exists index(\n    :oban_jobs,\n    [:queue, :state, :priority, :scheduled_at, :id],\n    concurrently: true,\n    prefix: \"public\"\n  )\nend\n```\n\nBe sure to reference the correct prefix if your `oban_jobs` table uses a prefix\nother than `public`.","ref":"v2-11.html#swap-the-compound-index-optional-but-recommended"},{"type":"extras","title":"Upgrading to v2.12","doc":"# Upgrading to v2.12\n\nThis Oban release includes a couple of optional configuration changes to aid in\ntesting and development.","ref":"v2-12.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.12","doc":"Update Oban (and optionally Pro) to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.12\"},\n  {:oban_pro, \"~> 0.11\", repo: \"oban\"}\n]\n```","ref":"v2-12.html#bump-your-deps"},{"type":"extras","title":"Modify Configuration for Testing - Upgrading to v2.12","doc":"The new `:testing` option automates configuring an Oban instance for testing.\nMake the following change to your `test.exs` to opt into `:manual` testing mode:\n\n```diff\n# test.exs\n- config :my_app, Oban, queues: false, plugins: false\n+ config :my_app, Oban, testing: :manual\n```\n\nIf you'd prefer to run jobs inline as they're inserted, without involving the\ndatabase, then you can use `:inline` mode instead:\n\n```elixir\nconfig :my_app, Oban, testing: :inline\n```\n\nSee the [testing guide](testing.html) to learn more about test configuration.","ref":"v2-12.html#modify-configuration-for-testing"},{"type":"extras","title":"Upgrading to v2.14","doc":"# Upgrading to v2.14\n\nThis Oban release includes a number of configuration changes and deprecations for redundant\nfunctionality.","ref":"v2-14.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.14","doc":"Update Oban (and optionally Pro) to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.14\"},\n  {:oban_pro, \"~> 0.13\", repo: \"oban\"}\n]\n```","ref":"v2-14.html#bump-your-deps"},{"type":"extras","title":"Remove Repeater and Stager Plugins - Upgrading to v2.14","doc":"The `Repeater` plugin is no longer necessary as the new `Stager` falls back to polling mode\nautomatically. Remove the `Repeater` from your plugins:\n\n```diff\n plugins: [\n   Oban.Plugins.Lifeline,\n   Oban.Plugins.Pruner,\n-  Oban.Plugins.Repeater\n```\n\nThe `Stager` is no longer a plugin because it's essential for queue operation. If you've\noverridden the staging interval:\n\n1. Reconsider whether that's necessary, staging is optimized to be a light-weight operation.\n2. If you're set on using a different interval, move it to `:stage_interval`\n\n```diff\n plugins: [\n   Oban.Plugins.Lifeline,\n   Oban.Plugins.Pruner,\n-  {Oban.Plugins.Stager, interval: 5_000}\n ],\n+ stage_interval: 5_000\n```","ref":"v2-14.html#remove-repeater-and-stager-plugins"},{"type":"extras","title":"Ensure Configuration for Testing - Upgrading to v2.14","doc":"Now that `Stager` isn't a plugin, it isn't disabled by `plugins: false`. Be sure\nto use the `:testing` option [introduced in v2.12][v212] to automate configuration:\n\n```diff\n# test.exs\n- config :my_app, Oban, queues: false, plugins: false\n+ config :my_app, Oban, testing: :manual\n```\n\nWithout this change you may see a flurry of `DBConnection.OwnershipError` errors\nduring test runs.\n\n[v212]: v2-12.html#modify-configuration-for-testing","ref":"v2-14.html#ensure-configuration-for-testing"},{"type":"extras","title":"Upgrading to v2.17","doc":"# Upgrading to v2.17\n\nThis Oban release includes an optional, but recommended migration.\n\n> #### Prevent Duplicate Insert Notifications {: .warning}\n>\n> You must either [run the v12 migrations](#run-oban-migrations-for-v12-optional) or [disable\n> insert triggers](#disable-insert-notifications-optional) in your Oban configuration, otherwise\n> you'll receive duplicate insert notifications for each job.","ref":"v2-17.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.17","doc":"Update Oban (and optionally Pro) to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.17\"},\n  {:oban_pro, \"~> 1.2\", repo: \"oban\"}\n]\n```","ref":"v2-17.html#bump-your-deps"},{"type":"extras","title":"Run Oban.Migrations for v12 (Optional) - Upgrading to v2.17","doc":"The v12 migration removes insert triggers and relaxes the `priority` column's check constraint to\nallow values in the new range of `0..9`.\n\nTo get started, create a migration to create the table:\n\n```bash\n$ mix ecto.gen.migration upgrade_oban_jobs_to_v12\n```\n\nWithin the generated migration module:\n\n```elixir\nuse Ecto.Migration\n\ndef up, do: Oban.Migrations.up(version: 12)\n\ndef down, do: Oban.Migrations.down(version: 12)\n```\n\nIf you have multiple Oban instances, or use an alternate prefix, you'll need to run the migration\nfor each prefix.","ref":"v2-17.html#run-oban-migrations-for-v12-optional"},{"type":"extras","title":"Disable Insert Notifications (Optional) - Upgrading to v2.17","doc":"If you opt not to run the v12 migration to disable Postgres triggers, then you should disable\ninsert notifications in your configuration:\n\n```diff\n config :my_app, Oban,\n+  insert_trigger: false,\n   ...\n```","ref":"v2-17.html#disable-insert-notifications-optional"},{"type":"extras","title":"Remove the Gossip Plugin - Upgrading to v2.17","doc":"The Gossip plugin is no longer used by Oban Web and now useless. You can safely remove it from\nyour configuration:\n\n```diff\n config :my_app, Oban,\n   plugins: [\n-    Oban.Plugins.Gossip,\n   ]\n```","ref":"v2-17.html#remove-the-gossip-plugin"}],"proglang":"elixir","producer":{"name":"ex_doc","version":"0.37.1"},"content_type":"text/markdown"}